/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rdm.web.suspectlinks.ui.PublisherWidget");

(function(){

// Dojo dependencies
dojo.require("dojo.i18n");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

// Foundation dependencies
dojo.require("jazz.ui.ResourceLink");

// SL dependencies
dojo.require("com.ibm.rdm.web.suspectlinks.ui.ServiceProviderWidget");
dojo.require("com.ibm.rdm.web.suspectlinks.ui.TrackedAppActionWidget");

dojo.requireLocalization("com.ibm.rdm.web.suspectlinks.ui","PublisherWidget");

var serviceProviderCompareFunction = function(a, b) {
	if(a.title > b.title) { return 1; }
	if(a.title < b.title) { return -1; }
	return 0;
};

/**
 * Widget which represents a single OSLC application (such as a CLM
 * application) in the Suspect Links Overview page.
 * 
 * @name com.ibm.rdm.web.suspectlinks.ui.PublisherWidget
 * @class <p>Widget which represents a single OSLC application (such as a CLM
 *  application) in the Suspect Links Overview page.</p>
 * @since 4.0
 */
dojo.declare("com.ibm.rdm.web.suspectlinks.ui.PublisherWidget",
	[dijit._Widget, dijit._Templated],
/** @lends com.ibm.rdm.web.suspectlinks.ui.PublisherWidget.prototype */ {
	
	templateString: dojo.cache("com.ibm.rdm.web.suspectlinks.ui",
		"templates/PublisherWidget.html"),
	
	// Dojo attach points
	_actionArea: null,
	_nameSpan: null,
	_instanceSpan: null,
	_serviceProviderNode: null,
	_wrapper: null,
	_editText: null,
	_heading: null,
		
	/**
	 * Contains the name of this publisher (project area.)
	 * 
	 * @private
	 * @field
	 * @since 4.0
	 * @type String
	 */
	_title: null,

	/**
	 * Contains the URI for this publisher.
	 * 
	 * @private
	 * @field
	 * @since 4.0
	 * @type String
	 */
	_uri: null,
	
	/**
	 * The specific application instance name for this publisher.
	 * 
	 * @private
	 * @field
	 * @since 4.0
	 * @type String
	 */
	_instanceName: null,

	/**
	 * Contains the icon of this publisher (if any.)
	 * 
	 * @private
	 * @field
	 * @since 4.0
	 * @type String
	 */
	_iconUri: null,

	/**
	 * An array of service provider data.
	 * 
	 * @private
	 * @field
	 * @since 4.0
	 * @type Array
	 */
	_serviceProviderData: null,
	
	_trsUri: null,
	
	_trackedApplicationData: null,
	
	/**
	 * An array of widgets (and other objects) to clean up on
	 * uninitialize.
	 * 
	 * @private
	 * @field
	 * @since 4.0
	 * @type Array
	 * @default null
	 */
	_toDestroy: null,
	
	_messages: null,
	
	constructor: function(args) {
		this._toDestroy = [];
		if(args["title"]) {
			this._title = args.title;
		}
		if(args["uri"]) {
			this._uri = args.uri;
		}
		if(args["instanceName"]) {
			this._instanceName = args.instanceName;
		}
		if(args["icon"]) {
			this._iconUri = args.icon;
		}
		if(args["trsUri"]) {
			this._trsUri = args.trsUri;
		}
		if(args["trackedApplication"]) {
			this._trackedApplicationData = args.trackedApplication;
		}
		if(args["serviceProviders"]) {
			this._serviceProviderData = args.serviceProviders;
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);
	
		// Only display the suspicion profiles for RM since they currently can't be used by the other apps.			
		if (this._uri.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) !== -1) {
			var editText = this._messages.editProfiles.toUpperCase();
			dojo.attr(this._editText, "innerHTML", editText);
		}

		dojo.attr(this._nameSpan, "innerHTML", this._title);
		dojo.attr(this._instanceSpan, "innerHTML", this._instanceName);
		
		if(this._iconUri) {
			dojo.style(this._heading, "backgroundImage",
				"url(\"" + this._iconUri + "\")");
		}
		
		this._renderTrackedAppActionWidget();
		
		// Only display the suspicion profiles for RM since they currently can't be used by the other apps.
		if (this._uri.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) !== -1) {
			if(this._serviceProviderData
				&& dojo.isArray(this._serviceProviderData)) {
				this._serviceProviderData.sort(serviceProviderCompareFunction);
				dojo.forEach(this._serviceProviderData,
					function(serviceProviderInfo) {

					this._renderServiceProviderWidget(serviceProviderInfo);
				}, this);
			}
		}
	},
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.suspectlinks.ui", "PublisherWidget");
	},

	addServiceProvider: function(serviceProviderInfo) {
		this._serviceProviderData.push(serviceProviderInfo);
		this._renderServiceProviderWidget(serviceProviderInfo);
	},
	
	_renderServiceProviderWidget: function(serviceProviderInfo) {
		var nServiceProviderWidget = 
			new com.ibm.rdm.web.suspectlinks.ui.ServiceProviderWidget(
				serviceProviderInfo);
		this._toDestroy.push(nServiceProviderWidget);
		dojo.place(nServiceProviderWidget.domNode, this._serviceProviderNode,
			"last");
	},
	
	_renderTrackedAppActionWidget: function() {
		var nTrackedAppActionWidget =
			new com.ibm.rdm.web.suspectlinks.ui.TrackedAppActionWidget({
				trsUri: this._trsUri,
				applicationName: this._title,
				trackedApplicationData: this._trackedApplicationData
			});
		this._toDestroy.push(nTrackedAppActionWidget);
		dojo.place(nTrackedAppActionWidget.domNode, this._trackingNode,
			"last");
	}
	
}); // End com.ibm.rdm.web.suspectlinks.ui.PublisherWidget

})();