/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.model.attributes.ui.EnumListMultiValueAttributeWidget2");
dojo.require("com.ibm.rdm.web.model.attributes.ui.BaseAttributeWidget");

dojo.require("com.ibm.rdm.web.util.keyutils");
dojo.require("com.ibm.rdm.web.server.client.utils.UserCache");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi
dojo.require("com.ibm.rdm.web.common.TeamContributorSelectionDialog");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
//Values that indicate which widget is to be given focus in the EnumListMultiValueAttributeWidget2
var FOCUS_WIDGET_TOOLBAR = 1;
var FOCUS_WIDGET_FILTER = 2;
var FOCUS_WIDGET_LIST = 3;

var KeyUtils = com.ibm.rdm.web.util.keyutils;
var UserCache = com.ibm.rdm.web.server.client.utils.UserCache;
var domutils = com.ibm.rdm.web.util.domutils;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi

dojo.declare(
	"com.ibm.rdm.web.model.attributes.ui.EnumListMultiValueAttributeWidget2",
	com.ibm.rdm.web.model.attributes.ui.BaseAttributeWidget,
{
	templateString: '<span dojoAttachPoint="_rootNode">'+
		'<div dojoAttachPoint="_chosenListDiv" class="chosenList"></div>'+
		'<div dojoAttachPoint="_filterBoxDiv"></div>'+
		'<div dojoAttachPoint="_inputBoxDiv"></div>'+
	'</span>',
	widgetsInTemplate: true,

	//Determines if the control is currently showing only a truncated list of selected entries 
	showTruncated: true,
	TRUNCATED_LENGTH: 10,
	

	//Options for the type of dialog to show
	//Can be a number >=1, or ""/null for many (ie no limit)
	selectionSize: null,

	//If true, the selection will be 'tri state' (and there will not be a list of selected options displayed)
	manyResources: false,

	//This reflects the values that have been positively set to true in the control. A hashmap of enum value URIs (to true)
	setValues: null,

	//This reflects the values that have been positively set to false (ie removed from the selection) 
	//in the control. A hashmap of enum value URIs (to true). Only used if manyResources is true.
	unsetValues: null,
	
	//Keeps a note of info to be used for each enum value. Hashmap of enum URI to {title: string, dom: node, pos: int postion}.
	enumEntriesMap: null,
	
	//Keeps a mapping of int position to uri.
	enumPosUri: null,
	
	//Input of the values that are set, used when editing more than one resource.
	allSet: null,
	
	//Input of the values that are mixed - the remaing are unset. Used when editing more than one resource.
	allMixed: null,
	
	//Record where last 'clicked' position was
	lastPos: 0,
	
	//Numer of enumeration values
	enumCount: 0,
	
	subWidget: null,
	
	_hoverPopup: null,
	_optionCount: 0,
	

	postCreate: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.model.attributes.ui", "AttributeInputWidget");

		this._optionCount = 0;
		
		this.setValues = {};
		this.unsetValues = {};
		
		var isUsers = (this.dataType === com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.UserType);

		//Make the parent element nowrap if necessary
		if (this.adjustInputForMVA) {
			dojo.removeAttr(this.adjustInputForMVA, "noWrap");
			this.adjustInputForMVA = null;
		}

		dojo.attr(this._inputBoxDiv, "class", "EnumListMultiValueAttributeWidget");

		this._listNode = dojo.create("span", null, this._inputBoxDiv);
		dojo.setSelectable(this._listNode, false);
		this.connect(this._listNode, "onclick", dojo.hitch(this, this._onClickOrKeyDown));
		// Note by SPC on 10/08/2013:
		// The following "onkeydown" was originally an "onkeyup", but I've changed it to
		// "onkeydown" to avoid the issue described by Defect 79600.  I did try incorporating
		// a 'keyCode: dojo.keys.SPACE' based element in the subsequent KeyUtils.getKeyHandler
		// call, but when I use that approach it appears to be in conflict with some with
		// some other handler.
		this.connect(this._listNode, "onkeydown", dojo.hitch(this, this._onClickOrKeyDown));

		// Define the navigation related key press handlers.
		var toolbarButtonOnKeyPressHandler =
			KeyUtils.getKeyHandler({
				keyCode: dojo.keys.TAB,
				shift: false,
				dontStopEvent: true,
				onKeyPress: dojo.hitch(this, function(event){
					// Tab goes to filter box if it is shown; otherwise to the toolbar.
					if (this._isFilterWidgetVisible()) {
						this._inputWidget.focus();
					} else {
						this._setListFocus(this._listFocusPosition);
					}
					dojo.stopEvent(event);
				})
			});
		var filterBoxOnKeyPressHandler =
			KeyUtils.getKeyHandlers([
				{
					keyCode: dojo.keys.TAB,
					shift: false,
					dontStopEvent: true,
					onKeyPress: dojo.hitch(this, this._tabFilterKeyPress)
				},
				{
					keyCode: dojo.keys.TAB,
					shift: true,
					dontStopEvent: true,
					onKeyPress: dojo.hitch(this, function(event){
						this._toolbar.focus();
						dojo.stopEvent(event);
					})
				}
			]);
		var listNodeOnKeyPressHandler = 
			KeyUtils.getKeyHandlers([
				{
					keyCode: dojo.keys.UP_ARROW,
					onKeyPress: dojo.hitch(this, this._upDownListKeyPress, true)
				},
				{
					keyCode: dojo.keys.DOWN_ARROW,
					onKeyPress: dojo.hitch(this, this._upDownListKeyPress, false)
				},
				{
					keyCode: dojo.keys.TAB,
					shift: true,
					dontStopEvent: true,
					onKeyPress: dojo.hitch(this, function(event){
						// Shift-Tab goes to filter box if it is shown; otherwise to the toolbar.
						if (this._isFilterWidgetVisible()) {
							this._inputWidget.focus();
						} else {
							this._toolbar.focus();
						}
						dojo.stopEvent(event);
					})
				}
			]);
		
		this._inputWidget = new jazz.ui.FilterBox({
				onChange: dojo.hitch(this, this.handleFilterChange),
				defaultText: this._messages.typeToFilter
			},
			this._filterBoxDiv);

		// Add the toolbar and add navigation key press handlers to the
		// buttons found within it.
		this._toolbar = new jazz.ui.Toolbar();
		var addUsersButton = null, clearButton = null;
		if (isUsers) {
			addUsersButton =
				new jazz.ui.toolbar.Button({
					title: this._messages.addUsers,
					imgClass: "add-user-icon",
					onClick: dojo.hitch(this, this._addUsers),
					onKeyPress: dojo.hitch(this, toolbarButtonOnKeyPressHandler)
				});
			this._toolbar.addChild(addUsersButton);
		}
		clearButton =
			new jazz.ui.toolbar.Button({
				title: this._messages.clear,
				imgClass: "clear-icon",
				onClick: dojo.hitch(this, this._clearAll),
				onKeyPress: dojo.hitch(this, toolbarButtonOnKeyPressHandler)
			});
		this._toolbar.addChild(clearButton);

		// Add navigation key press handlers to the filter and list areas.
		this.connect(this._inputWidget._box, "onkeypress", filterBoxOnKeyPressHandler);
		this.connect(this._listNode, "onkeypress", listNodeOnKeyPressHandler);

		this.createReadOnlyNode();
		this._chosenListDiv.appendChild(this._toolbar.domNode);
		
		//Ensure value is an array and preselect it/them.
		if (this.value) {
			if (!dojo.isArray(this.value)) {
				this.value = [this.value];
			}
			for (var i = 0; i < this.value.length; i++) {
				this.value[i] = UserCache.convertRDMUserToJTS(this.value[i]);
			}
		} else {
			this.value = [];
		}
				
		if (this.allSet || this.allMixed) {
			//We have set and mixed values - must be displaying multiple resources
			this.manyResources = true;
			//We don't populate 'setValues' here, as 1) we don't use it to display the summary for manyResoures, and
			//	2) we don't want to cause unnecessary writes when exiting the control.
		} else {
			this.manyResources = false;
			this.allSet = {}; 
			this.allMixed = {};
			if (this.value) {
				//we just have the one value to show, but may have many entries
				var j;
				for (j=0; j<this.value.length; j++) {
					this.allSet[this.value[j]] = true;
					this.setValues[this.value[j]] = true;
				}				
			}
		}
		
		if (isUsers) {
			this._initFromUsers();
		} else {
			//An enumerated type
			this._initFromEnumentries();
		}

		//Accessibility Change: The focusNode should be the focusNode of the FilterBox
		this.focusNode = this._toolbar.id;
		
		this.inherited(arguments);
	},

	_isFilterWidgetVisible: function() {
		return dojo.style(this._inputWidget.domNode, "display") !== "none";
	},

	_enableOrDisableWidget: function(disable) {
		this.inherited(arguments);

		this.updateDisplay();
		if (this.value.length > 0) {
			// Ensure this multi-value enum attribute is up-to-dated in the resourceProperties.attributes list.
			// onChange calls _setAttributeValue which will set the attribute with the updated value. Pass in
			// a boolean true to ignore dirty state.
			this.onChange(this.value, true);
		}
	},
	
	_createSortedArray: function(selectedHash) {
		var sortedArray = [];
		var uri;
		for (uri in selectedHash) { 
			sortedArray.push(uri);
		} 
		var pos = this.enumEntriesMap;
		
		sortedArray.sort(function(a, b) {
			var av = pos[a].pos, bv = pos[b].pos;
			if (av < bv) {
				return -1; 
			} else if (av > bv) {
				return 1; 
			} else {
				return 0;
			}
		});
		
		return sortedArray;
	},
	
	updateReadOnlyAttribute: function() {
		this.updateDisplay();
	},

	handleFilterChange: function(value){
		var uri, dom, entry;
		if (value) {
			//We have a filter value - change non-matching to display:none
			for(uri in this.enumEntriesMap) {
				entry = this.enumEntriesMap[uri];
				var title = entry.title.toLowerCase();
				dom = entry.dom;
				if (title.indexOf(value) >= 0) {
					dojo.style(dom, "display", "block");
					delete entry["invisible"];
				} else {
					dojo.style(dom, "display", "none");
					entry.invisible = true;
				}
			}
		} else {
			for(uri in this.enumEntriesMap) {
				entry = this.enumEntriesMap[uri];
				dom = entry.dom;
				dojo.style(dom, "display", "block");
				delete entry["invisible"];
			}
		}
	},
	
	focus: function(event) {
		// summary:
		//		Takes focus, taking into account the event that has triggered focus.
		// event:
		//		Optional.  The event that has triggered the focus.
		
		// Set the focus to one of the input controls, depending on whether we have got a
		// forward tab or back tab into the editor.  We remember this value is _focusWidget because
		// when the row is refreshed we get a subsequent focus without a keypress event, in which case
		// we will want to re-focus on the appropriate field.

		var filterVisible = this._isFilterWidgetVisible();
		var listVisible = this._getNextVisibleEntryPos(-1) >= 0;
		if (!filterVisible && !listVisible) {
			// should never happen, but just in case... 
			return;
		}
		var firstTabTarget = FOCUS_WIDGET_TOOLBAR;
		var lastTabTarget = listVisible ? FOCUS_WIDGET_LIST : FOCUS_WIDGET_FILTER;
		
		if (event) {
			if (event.keyCode === dojo.keys.TAB) {
				this._focusWidgetIndicator = event.shiftKey ? lastTabTarget : firstTabTarget;
			}
		}
		// just in case initially we get a focus before _focusWidgetIndicator has been set.
		if (!this._focusWidgetIndicator) {
			this._focusWidgetIndicator = firstTabTarget;
		}

		switch (this._focusWidgetIndicator) {
		case FOCUS_WIDGET_TOOLBAR:
			this._toolbar.focus();
			break;
		case FOCUS_WIDGET_FILTER:
			this._inputWidget.focus();
			break;
		case FOCUS_WIDGET_LIST:
			this._setListFocus(this._listFocusPosition);
			break;
		}
	},
	
	_getPreviousVisibleEntryPos: function(pos) {
		// summary:
		//		Given a position in the list, finds the position of the
		//		previous visible entry.
		// pos:
		//		The starting position.  Must be zero or a positive integer.
		// returns:
		//		The position of a visible entry, or -1 if none is found.
		while (--pos >= 0) {
			this.enumPosUri[pos];
			var entry = this.enumEntriesMap[this.enumPosUri[pos]];
			if (!entry.invisible) {
				return pos;
			}
		}
		return -1;
	},
	
	_getNextVisibleEntryPos: function(pos) {
		// summary:
		//		Given a position in the list, finds the position of the
		//		next visible entry.
		// pos:
		//		The starting position.  Should normally be zero or a positive integer.
		//		Passing -1 effectively finds the first visible item in the list, so can
		//		be used as a test for at least on visible item. 
		// returns:
		//		The position of a visible entry, or -1 if none is found.
		while (++pos < this.enumCount) {
			var entry = this.enumEntriesMap[this.enumPosUri[pos]];
			if (!entry.invisible) {
				return pos;
			}			
		}
		return -1;
	},
	
	_setListFocus: function(pos) {
		// summary:
		//		Sets the focus to the given position in the list.
		if (!pos || pos < 0 || pos >= this.enumCount) {
			pos = 0;
		}
		var uri = this.enumPosUri[pos];
		var entry = this.enumEntriesMap[uri];
		entry.dom.focus();
		this._listFocusPosition = pos;
	},
	
	_tabFilterKeyPress: function(event) {
		// summary:
		//		Handles a tab keypress in the filter field.
		
		// Move focus to the an entry in the list.  Choose the previously focused one, if visible, or the
		// closest earlier visible entry if that entry is not available.
		var pos = this._listFocusPosition;
		if (!pos) {
			pos = 0;
		}
		var uri = this.enumPosUri[pos];
		var entry = this.enumEntriesMap[uri];
		
		if (entry.invisible) {
			// look for a nearby visible entry.
			pos = this._getPreviousVisibleEntryPos(pos);
			if (pos < 0) {
				// no visible entry before - look for one after
				pos = this._getNextVisibleEntryPos(0);
			}
		}
		if (pos >= 0) {
			this._setListFocus(pos);
			
			// This method is called from a key handler that has dontStopHandledEvent==true, so
			// we have to stop the event manually to stop it propagating.
			dojo.stopEvent(event);
		}
		// No visible entries were found.  Do not stop the event, which lets it bubble up and
		// change focus out of this editor.
	},
	
	_upDownListKeyPress: function(goUp, event) {
		// summary:
		//		Handles up or down arrows on the list area.  Moves the focus up or down
		//		in the list.
		// goUp:
		//		true for up-arrow, false for down-arrow.
		// event:
		//		The keyboard event.
		var target = event.target;
		var currentPosition = parseInt(dojo.attr(target, 'pos'));
		
		// Find the next visible entry in the appropriate direction.
		var nextPosition = goUp ? this._getPreviousVisibleEntryPos(currentPosition) : this._getNextVisibleEntryPos(currentPosition);
		if (nextPosition >= 0) {
			this._setListFocus(nextPosition);
		}
	},
	
	_onClickOrKeyDown: function(event) {
		// summary:
		//		Handle mouse click or Space key events that take place in the
		//		"check box" like inputs that this widget presents.

		var target = event.target;
		if (target.tagName === "LABEL") {
			// Note: if the target was the label, we reset it to instead be the
			// associated div that represents the checkbox.
			target = dojo.doc.getElementById(target.getAttribute("for"));
		}

		if (event.type !== "click") {
			if (event.keyCode === dojo.keys.SPACE) {
				dojo.stopEvent(event);
			} else {
				return;
			}
		}

		var shiftKey = event.shiftKey;
		
		var className = dojo.attr(target, 'class');
		var pos = parseInt(dojo.attr(target, 'pos'));
		var newName, newAriaState, start, end;

		if (!shiftKey) {
			newName = this._setState(pos, className);
			newAriaState = this._setAriaState(newName);
			dojo.attr(target, "class", newName);		
			dojo.attr(target, "role", "checkbox");
			dijit.setWaiState(target, "checked", newAriaState);
			this.lastPos = pos;
			this._listFocusPosition = pos;
		} else {
			//We need to set state of all visible entries between this and last
			if (this.lastPos < pos) {
				start = this.lastPos;
				end = pos;
			} else {
				end = this.lastPos;
				start = pos;
			}
			var x;
			for (x = start; x <= end; x++) {
				var uri = this.enumPosUri[x];
				var entry = this.enumEntriesMap[uri];
				
				if (!entry.invisible) {
					newName = this._setState(x, className);
					var dom = entry.dom;
					newAriaState = this._setAriaState(newName);	
					dojo.attr(dom, "class", newName);
					dojo.attr(dom, "role", "checkbox");
					dijit.setWaiState(dom, "checked", newAriaState);
				}
			}
		}
		this.value = this._createSortedArray(this.setValues);
		this.updateDisplay();
		this._callChange();
	},
	
	_callChange: function() {
		if (this.manyResources) {
			this.onChange({set: this.setValues, unset: this.unsetValues});
		} else {
			//Simple
			this.onChange(this.value);
		}	
	},
	
	_setAriaState: function(newName){
		var ariaState = null;
		switch(newName) {
		case "on":
			ariaState = "true";
			break;
		case "grey":
			ariaState = "mixed";
			break;
		case "off":
			ariaState = "false";
			break;
		}
		return ariaState; 
	},
	
	_setState: function(pos, className) {
		var uri = this.enumPosUri[pos];
		
		if (this.manyResources && this.isMultiValued) {
			//Need to deal with grey state, bearing in mind the original state of the values.
			if (className === "on") {
				className = "off";
				this.unsetValues[uri] = true;
				delete this.setValues[uri];
			} else if (className === "grey") {
				this.setValues[uri] = true;
				delete this.unsetValues[uri];
				className = "on";
			} else {
				if (this.allMixed[uri]) {
					className = "grey";
					delete this.unsetValues[uri];
					delete this.setValues[uri];
				} else {
					this.setValues[uri] = true;
					delete this.unsetValues[uri];
					className = "on";
				}
			}		
		} else {
			if (className === "on") {
				delete this.setValues[uri];
				this.unsetValues[uri] = true;
				className = "off";
			} else {
				if (!this.isMultiValued) {
					//We must turn all other setValues off
					var uri2;
					for (uri2 in this.setValues) {
						var entry = this.enumEntriesMap[uri2];
						var dom = entry.dom;
						dojo.attr(dom, "class", "off");
					}
					this.setValues = {};
				}
				this.setValues[uri] = true;
				delete this.unsetValues[uri];
				className = "on";
			}
		}

		return className;
	},

	updateDisplay: function(){
		var isTruncated = false,
			text = "", selectedValues, i, clickTo, connect,
			self = this;
		
		//Switch off the readonly display if multivalued
		if (this.manyResources) {
			dojo.attr(this._inputWidgetReadOnly,"display","none");
		} else {
			//We display the summary
			dojo.removeAttr(this._inputWidgetReadOnly,"display");
			if(this._isEmpty(this.setValues)) {
				dojo.place(document.createTextNode(""), this._inputWidgetReadOnly, "only");
			} else {
				selectedValues = this.value;
				for (i = 0; i < selectedValues.length; i++) {
					if (this.showTruncated && i >= this.TRUNCATED_LENGTH-1) {
						isTruncated = true;
						break;
					}
					if (this.enumEntriesMap[selectedValues[i]]) {
						text += this.enumEntriesMap[selectedValues[i]].title + ", ";
					} else {
						text += selectedValues[i] + ", ";
					}
				}
				if (text.length >= 2) {
					text = text.substring(0, text.length - 2);
				}

				dojo.place(document.createTextNode(text), this._inputWidgetReadOnly, "only");
				var showAll = dojo.string.substitute(this._messages.showAll, [selectedValues.length]);
				if (isTruncated) {
					clickTo = dojo.create("span", {innerHTML: showAll, className: "enumExp"});
					connect = dojo.connect(clickTo, "onclick", function() {
						self.showTruncated = false; dojo.disconnect(connect); self.updateDisplay();
					});
					dojo.place(clickTo, this._inputWidgetReadOnly, "last");
				} else if (!this.showTruncated && selectedValues.length >= this.TRUNCATED_LENGTH) {
					clickTo = dojo.create("span", {innerHTML: "hide", className: "enumExp"});
					connect = dojo.connect(clickTo, "onclick", function() {
						self.showTruncated = true; dojo.disconnect(connect); self.updateDisplay();
					});
					dojo.place(clickTo, this._inputWidgetReadOnly, "last");
				}
			}
		}
		
		if (this.disabled) {
			dojo.style(this._inputBoxDiv, "display", "none");
			dojo.style(this._inputWidget.domNode, "display", "none");
			dojo.removeClass(this._chosenListDiv, "editing");
		} else {
			dojo.style(this._inputBoxDiv, "display", "block");
			dojo.style(
				this._inputWidget.domNode, "display",
				(this.enumCount > 9) ? "block": "none"
			);
			dojo.addClass(this._chosenListDiv, "editing");
		}
	},
	
	_isEmpty: function(obj) { 
		var prop;
		for (prop in obj) { 
			if(obj.hasOwnProperty(prop)) {
				return false; 
			}
		} 
		return true; 
	},
	
	_clearAll: function() {
		var uri;
		this.setValues = {};
				
		//Clear all the tick boxes
		for(uri in this.enumEntriesMap) {
			var entry = this.enumEntriesMap[uri];
			var dom = entry.dom;
			dojo.attr(dom, "class", "off");
			dojo.attr(dom, "role", "checkbox");
			dojo.attr(dom, "aria-checked", "false");
			this.unsetValues[uri] = true;
		}
		this.value = this._createSortedArray(this.setValues);
		this.updateDisplay();
		
		this._callChange();
	},

	isValid: function(){
		//no isValid for dijit.multiselect????
		//return this._inputWidget.isValid();
		return true;
	},

	_generateOptionId: function() {
		this._optionCount++;
		return this.id + "_" + this._optionCount;
	},
	
	_addFocusAndBlurTreatmentForCheckBoxDiv: function(checkBoxDiv, label) {
		// summary:
		//		Listen for appropriate DOM events to make sure that whenever
		//		the "check box" has focus, the associated label appears to
		//		also have focus (i.e similate the behavior we would see
		//		when using of a "normal" checkbox/label combo).

		this.connect(checkBoxDiv, "onfocus", dojo.hitch(this, function() {
			dojo.addClass(label, "dijitFocusedLabel");
		}));
		this.connect(checkBoxDiv, "onblur", dojo.hitch(this, function() {
			dojo.removeClass(label, "dijitFocusedLabel");
		}));
	},

	_createLabelForCheckBox: function(checkBoxDiv, checkBoxId, title) {
		var label = dojo.create("label", {
				"for": checkBoxId,
				"innerHTML": domutils.getHTMLEncodedText(BidiUtils.setTextDirUCC(title))
			},
			checkBoxDiv);
		if (dojo.isIE) {
			this.connect(label, "onclick", this._onClickOrKeyDown);
		}
		return label;
	},

	_initFromEnumentries: function() {
		//Pre-process the enumeration values
		this.enumEntriesMap = {};
		this.enumPosUri = [];

		var i;
		for (i=0; i<this.enumEntries.length; i++) {
			var optionId = this._generateOptionId();
			var option = dojo.create(
				"div",
				{
					pos: i,
					tabindex: 0,
					id: optionId
				}
			);
			
			var title = this.enumEntries[i].enumTitle;
			var about  = this.enumEntries[i].enumAbout;

			var label = this._createLabelForCheckBox(option, optionId, title);

			this._listNode.appendChild(option);

			this.enumEntriesMap[about] = {title: title, dom: option, pos: i};
			this.enumPosUri[i] = about;
			
			if (this.manyResources) {
				if (this.allSet[about]){
					dojo.attr(option, "class", "on");
					dojo.attr(option, "role", "checkbox");
					dojo.attr(option, "aria-checked", "true");
				} else if (this.allMixed[about]) {
					dojo.attr(option, "class", "grey");
					dojo.attr(option, "role", "checkbox");
					dojo.attr(option, "aria-checked", "mixed");
				} else {
					dojo.attr(option, "class", "off");
					dojo.attr(option, "role", "checkbox");
					dojo.attr(option, "aria-checked", "false");
				}
			} else {
				if (this.allSet[about]){
					dojo.attr(option, "class", "on");
					dojo.attr(option, "role", "checkbox");
					dojo.attr(option, "aria-checked", "true");
				} else {
					dojo.attr(option, "class", "off");
					dojo.attr(option, "role", "checkbox");
					dojo.attr(option, "aria-checked", "false");
				}
			}

			this._addFocusAndBlurTreatmentForCheckBoxDiv(option, label);
		}
				
		//Now ensure any invalid values are removed.
		for (var uri in this.allSet) {
			if (!this.enumEntriesMap[uri]) {
				delete this.allSet[uri];
			}
		}

		for (var uri in this.allMixed) {
			if (!this.enumEntriesMap[uri]) {
				delete this.allMixed[uri];
			}
		}
		
		for (var uri in this.setValues) {
			if (!this.enumEntriesMap[uri]) {
				delete this.setValues[uri];
			}
		}		
		
		this.enumCount = i;
		this.value = this._createSortedArray(this.setValues);
		this.updateDisplay();	
	},
	
	/////////////////////////////////////////////
	//// Functions to deal with users.
	
	_initFromUsers: function() {
		// summary:
		//		initialises the control from the available Users in the UserCache.
		//		Ensure that the current values are available as ids at least
		this.enumEntriesMap = {};		
		if (this.value.length > 0) {
			var userDeferred = UserCache.getUsers(this.value);
			userDeferred.addCallback(this, function(users){
				try {
					dojo.forEach(users, function(user) {UserCache.addToRcentUsersStore(user);});
					this._buildUsersEnum();
				}
				catch(e) {
					//console.log("usersDeferred error!", e);
					this.showErrorValue();
				}
			});
			userDeferred.addErrback(this, function(users){
				//console.log("usersDeferred errback", users);
				this._buildUsersEnum();
				this.showErrorValue();
			});
			//Build the users enum synchronously anyway
			/* defect 99142 : Below line is commented because  _buildUsersEnum() is called before the response 
			 * from UserCache.getUsers() deferred  function.
			  */
			//this._buildUsersEnum();
		} else {
			this._buildUsersEnum();
		}
	},

	_buildUsersEnum: function() {
		var users = UserCache.getRecentUsers();

		//Pre-process the enumeration values
		if (!this.enumEntriesMap) {
			this.enumEntriesMap = {};
		}
		if (!this.enumPosUri) {
			this.enumPosUri = {};
		}

		//Get a map of the current actual values, to ensure they are present
		var currentValues = {};
		dojo.forEach(this.value, function(value) {
			currentValues[value] = true;
		});
		
		var i;
		for (i=0; i<users.length; i++) {

			var title = users[i].name;
			var about  = users[i].uri;
			
			//Remove from the current values
			delete currentValues[about];
			
			this._addValue(title, about, i);
		}
		
		//Now add any missing values as uri only
		for (var missing in currentValues) {
			this._addValue(missing, missing, i++);				
		}
		
		this.enumCount = i;
		this.value = this._createSortedArray(this.setValues);
		this.updateDisplay();	
	},

	_addValue: function(title, about, i) {
		var entry = this.enumEntriesMap[about];
		if (!entry) {
			var optionId = this._generateOptionId();
			var option = dojo.create(
				"div",
				{
					pos: i,
					tabindex: 0,
					id: optionId
				}
			);
			var label = this._createLabelForCheckBox(option, optionId, title);

			this._addFocusAndBlurTreatmentForCheckBoxDiv(option, label);

			this._listNode.appendChild(option);

			this.enumEntriesMap[about] = {title: title, dom: option, pos: i};
			this.enumPosUri[i] = about;

			if (this.manyResources) {
				if (this.allSet[about]){
					dojo.attr(option, "class", "on");
				} else if (this.allMixed[about]) {
					dojo.attr(option, "class", "grey");
				} else {
					dojo.attr(option, "class", "off");
				}
			} else {
				if (this.allSet[about]){
					dojo.attr(option, "class", "on");
				} else {
					dojo.attr(option, "class", "off");
				}
			}
		} else {
			//Update the title if the old is the same as the URI
			if (entry.title === about) {
				entry.title = title;
				entry.dom.innerHTML = domutils.getHTMLEncodedText(title);
			}
		}				
	},

	_addUserEnum: function(user) {
		var userUri = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS + '/' + encodeURIComponent(user.userId);
		
		//If we don't already have it, add it
		if (!this.enumEntriesMap[userUri]) {
			var optionId = this._generateOptionId();
			var option = dojo.create(
				"div",
				{
					pos: this.enumCount,
					tabindex: 0,
					id: optionId
				}
			);
			var title = user.name;
			var label = this._createLabelForCheckBox(option, optionId, title);

			this._addFocusAndBlurTreatmentForCheckBoxDiv(option, label);

			this._listNode.appendChild(option);

			this.enumEntriesMap[userUri] = {title: title, dom: option, pos: this.enumCount};
			this.enumPosUri[this.enumCount++] = userUri;
		}
		
		//Ensure it is switched on
		this._switchValueOn(userUri);
		
		this.value = this._createSortedArray(this.setValues);
		this.updateDisplay();
		
		this._callChange();
	},
	
	_switchValueOn: function(userUri) {
		var uri, entry;
		//If we are singlevalued, all others should be off
		if (!this.isMultiValued) {
			
			for(uri in this.setValues) {
				entry = this.enumEntriesMap[uri];
				dojo.attr(entry.dom, "class", "off");
			}
			
			this.setValues = {};
		}	
		entry = this.enumEntriesMap[userUri];
		//Ensure it is set
		this.setValues[userUri] = true;
		dojo.attr(entry.dom, "class", "on");
	},
	
	_addUsers: function(event) {
		var form = new com.ibm.rdm.web.common.TeamContributorSelectionDialog({
			allowMultiple: this.isMultiValued,
			disableUsersOnAdd: false,
			onOK: dojo.hitch(this, function(users){
				if (!dojo.isArray(users)) {
					users = [users];
				}
				var self = this;
				dojo.forEach(users, function(user) {
					self._addUserEnum(user);
				});
				this.value = this._createSortedArray(this.setValues);
				this.updateDisplay();	
			})
		});
		this.subWidget = form;
	},

	showErrorValue: function(){
		this.onChange(null, true);
		var errorIcon = document.createElement("IMG");
		errorIcon.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/error.gif");
		dojo.addClass(errorIcon, "errorIcon");
		this._rootNode.appendChild(errorIcon);
		this._hoverPopup = new com.ibm.rdm.web.common.HoverPopup({
			around: [errorIcon],
			destroyContents: true,
			orientation: "R",
			createContents: dojo.hitch(this, function(){
					var div = document.createElement("div");
					div.appendChild(document.createTextNode(this._messages.deletedListItem));
					return div;
				})
		}, null);
	},
	
	uninitialize: function() {
		if(this._toolbar){
			this._toolbar.destroyRecursive();
		}
		if(this._hoverPopup){
			this._hoverPopup.destroyRecursive();
		}
		this.inherited(arguments);
	}
});

})();
