/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.rdm.web.model.types.TypesManager");

dojo.require("dojo.DeferredList");

dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.links.LinkType");
dojo.require("com.ibm.rdm.web.model.types.ObjectType");
dojo.require("com.ibm.rdm.web.model.attributes.AttributeDefinition");
dojo.require("com.ibm.rdm.web.model.attributes.AttributeType");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.util.vvcutils");
//dojo.require("com.ibm.rdm.web.links.LinkTypeUtils"); CIRC

var ObjectType = com.ibm.rdm.web.model.types.ObjectType;
var AttributeDefinition = com.ibm.rdm.web.model.attributes.AttributeDefinition;
var AttributeType = com.ibm.rdm.web.model.attributes.AttributeType;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var queryutils = com.ibm.rdm.web.util.queryutils;
var domutils = com.ibm.rdm.web.util.domutils;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var Headers = com.ibm.rdm.web.server.Headers;
var VVCUtils = com.ibm.rdm.web.util.vvcutils;

var OBJECT_TYPES = "ObjectTypes";
var ATTRIBUTE_DEFINITIONS = "AttributeDefinitions";
var WELL_KNOWN_ATTRIBUTE_DEFINITIONS = "WellKnownAttributeDefinitions";
var ATTRIBUTE_TYPES = "AttributeTypes";
var LINK_TYPES = "LinkTypes";

com.ibm.rdm.web.model.types.TypesManager = {
		
	_allTypesCache: new dojox.collections.Dictionary(),
	_changeListeners: new dojox.collections.Dictionary(),
	
	
	/************* GET INDIVIDUAL TYPES ***************/
	getObjectType: function(args) {
		// summary:
		//		Get the type for one or more artifact from any number
		//		of projects.
		//		NOTE: It is VERY important that you use the batch
		//		call whenever possible! It will greatly reduce the
		//		amount of round trips to the server, and the wait
		//		time for the client!
		// args:
		//		This can be either an object or an array of objects
		//		of the following shape:
		//		{
		//			projectURI,			Required, String
		//			projectSnapshotURI, Optional, String
		//			typeURI,			Required, String
		//			configurationURI,	Optional, String - if not provided the current configuration is used
		//			isReadOnly,			Optional, Boolean - Defaults to true
		//			skipCache			Optional, Boolean - Defaults to false
		//		}
		// returns:
		//		If args is an object, then a Deferred will be
		//		returned. Its callback value will be the type object
		//		of the typeURI specified in args.
		//		If args was an array, then a DeferredList will be
		//		returned, and the callback will be an array of 
		//		aggregated type objects.
		
		if(dojo.isArray(args)) {
			args = dojo.map(args, function(arg) {
				return {
					projectURI: arg.projectURI,
					projectSnapshotURI: arg.projectSnapshotURI,
					typeURI: arg.typeURI,
					configurationURI: arg.configurationURI,
					feedType: OBJECT_TYPES,
					isReadOnly: (arg.isReadOnly == undefined)||arg.isReadOnly,
					skipCache: arg.skipCache
				};
			}, this);
		}
		else {
			args = {
				projectURI: args.projectURI,
				projectSnapshotURI: args.projectSnapshotURI,
				typeURI: args.typeURI,
				configurationURI: args.configurationURI,
				feedType: OBJECT_TYPES,
				isReadOnly: (args.isReadOnly == undefined)||args.isReadOnly,
				skipCache: args.skipCache
			};
		}

		return this._getTypes(args);
	},

	getAttributeDefinition: function(args) {
		return this._getTypes({
			projectURI: args.projectURI,
			projectSnapshotURI: args.projectSnapshotURI,
			typeURI: args.attributeDefinitionURI,
			configurationURI: args.configurationURI,
			feedType: ATTRIBUTE_DEFINITIONS,
			skipCache: args.skipCache,
			isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
		});
	},

	getAttributeType: function(args) {
		return this._getTypes({
			projectURI: args.projectURI,
			projectSnapshotURI: args.projectSnapshotURI,
			typeURI: args.attributeTypeURI,
			feedType: ATTRIBUTE_TYPES,
			skipCache: args.skipCache,
			isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
		});
	},
	
	getLinkType: function(args) {
		return this._getTypes({
			projectURI: args.projectURI,
			projectSnapshotURI: args.projectSnapshotURI,
			typeURI: args.linkTypeURI,
			feedType: LINK_TYPES,
			forceUseDefaultConfiguration: args.forceUseDefaultConfiguration,
			skipCache: args.skipCache,
			isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
		});
	},
	
	/************* GET TYPES FOR PROJECT ***************/
	getObjectTypesForProject: function(args) {
		//		NOTE: It is VERY important that you use the batch
		//		call whenever possible! It will greatly reduce the
		//		amount of round trips to the server, and the wait
		//		time for the client!
		
		if(dojo.isArray(args)) {
			args = dojo.map(args, function(arg) {
				return {
					projectURI: arg.projectURI,
					projectSnapshotURI: arg.projectSnapshotURI,
					configurationURI: arg.configurationURI,
					globalConfigurationURI: arg.globalConfigurationURI,
					forceUseDefaultConfiguration: arg.forceUseDefaultConfiguration,
					feedType: OBJECT_TYPES,
					isReadOnly: (arg.isReadOnly == undefined)||arg.isReadOnly,
					skipCache: arg.skipCache
				};
			}, this);
		}
		else {
			args = {
				projectURI: args.projectURI,
				projectSnapshotURI: args.projectSnapshotURI,
				feedType: OBJECT_TYPES,
				configurationURI: args.configurationURI,
				globalConfigurationURI: args.globalConfigurationURI,
				forceUseDefaultConfiguration: args.forceUseDefaultConfiguration,
				skipCache: args.skipCache,
				isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
			};
		}
		
		return this._getTypes(args);
	},

	getAttributeDefinitionsForProject: function(args) {
		return this._getTypes({
			projectURI: args.projectURI,
			projectSnapshotURI: args.projectSnapshotURI,
			configurationURI: args.configurationURI,
			feedType: ATTRIBUTE_DEFINITIONS,
			includeWellKnown: args.includeWellKnown,
			skipCache: args.skipCache,
			isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
		});
	},

	getAttributeTypesForProject: function(args) {
		return this._getTypes({
			projectURI: args.projectURI,
			configurationURI: args.configurationURI,
			projectSnapshotURI: args.projectSnapshotURI,
			feedType: ATTRIBUTE_TYPES,
			skipCache: args.skipCache,
			isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
		});
	},
	
	getLinkTypesForProject: function(args){
		// summary: 
		//		Gets the link types for the project, potentially excluding the CLM link types when opted-in. 
		// I'm only describing some of the new arguments below
		// args: {
		//	filterCLMLinks,		boolean - (optional) Use true to indicate that CLM links should be filtered out in an opted-in project. Otherwise, includes all links.
		// }
		//		Object with arguments, some described below (as seen at time of writing):
		//		
		//	
		// returns: 
		//		A deferred of all the link types in the project, potentially with CLM types filtered if CM enabled.
		
		var responseDeferred = new dojo.Deferred(); 
		var isOptedIn = com.ibm.rdm.web.common.ProjectCache.getCmState(args.projectURI);

		var filterOSLCLinks = function(links) {
			if(isOptedIn && links && args.filterCLMLinks === true) { 
				var filteredLinks = dojo.filter(links, function(link) {
					return !com.ibm.rdm.web.links.LinkTypeUtils.isCalmLink(link.getSameAs());
				});
				responseDeferred.callback(filteredLinks);
			} else {
				responseDeferred.callback(links);
			}
			return responseDeferred;
		};
		
		this._getTypes({
			projectURI: args.projectURI,
			configurationURI: args.configurationURI,
			globalConfigurationURI: args.globalConfigurationURI,
			projectSnapshotURI: args.projectSnapshotURI,
			feedType: LINK_TYPES,
			skipCache: args.skipCache,
			isReadOnly: (args.isReadOnly == undefined)?true:args.isReadOnly //for performance reasons this reduces memory footprint since we would not have to clone type classes
		}).then(filterOSLCLinks);
		
		return responseDeferred;
	},
	
	
	/************* CREATE INDIVIDUAL TYPES ***************/
	createObjectType: function(args) {
		return new ObjectType({
			projectURI: args.projectURI
		});
	},
	createAttributeDefinition: function(args) {
		return new AttributeDefinition({
			projectURI: args.projectURI
		});
	},
	createAttributeType: function(args) {
		return new AttributeType({
			projectURI: args.projectURI 
		});
	},	
	createLinkType: function(args) {
		return new com.ibm.rdm.web.links.LinkType(args);	
	},
	
	/************** GET SELECTED TYPES ********************/
	getArtifactTypes: function (projectURI, artifactTypeURIs) {
		//Summary: goes through	getting all the artifact types given, and then calling the callback with the list of types
		var deferred = new dojo.Deferred();
		
		if (artifactTypeURIs.length > 0) {
			var results = [];
			var current = 0;
			var self = this;

			var getAllValues = 	function (response) { 
				current++;
				results.push(response);
				if (current < artifactTypeURIs.length) {
					var uri = artifactTypeURIs[current];
					
					self.getObjectType({typeURI: uri, projectURI: projectURI}).addCallback(getAllValues);
				} else {
					deferred.callback(results);
				}
				return response;
			};
			var artifactType = self.getObjectType({typeURI: artifactTypeURIs[current], projectURI: projectURI}).addCallback(getAllValues);
			
		} else {
			deferred.callback([]);
		}
		
		return deferred;
	},
	
	
	getAttributeDefinitions: function(projectURI, attributeDefinitionURIs, /*Optional*/configurationURI) {
		//	summary: Gets all the required definitions.
		//	returns: a deferred that will be called back when all
		//	definitions are obtained.
		var deferred = new dojo.Deferred();
		
		//Ensure an empty list is handled ok
		if (attributeDefinitionURIs.length === 0) {
			deferred.callback([]);
			return deferred;
		}
		var definitionDeferreds = dojo.map(attributeDefinitionURIs,
			function(uri) {
			return this.getAttributeDefinition({
				attributeDefinitionURI: uri,
				projectURI: projectURI,
				configurationURI: configurationURI
			});
		}, this);
		
		var definitionDeferredList = new dojo.DeferredList(definitionDeferreds);
		definitionDeferredList.addCallback(this, function(definitionResults) {
			var definitions = dojo.map(definitionResults, function(d) {
				return d[1];
			});
			deferred.callback(definitions);
		});
		
		return deferred;
	},
	
	getLinkTypes: function(projectURI, linkTypeURIs) {
		//Summary: Gets all the required link types.
		// returns: a deferred that will be called back when all link Types are obtained.
		var deferred = new dojo.Deferred();
		
		if (linkTypeURIs.length > 0) {
			var results = [];
			var current = 0;
			var self = this;

			var getAllValues = 	function (response) { 
				current++;
				results.push(response);
				if (current < linkTypeURIs.length) {
					var uri = linkTypeURIs[current];
					self.getLinkType({linkTypeURI: uri, projectURI: projectURI}).addCallback(getAllValues);
				} else {
					deferred.callback(results);
				}
				return response;
			};
			
			var linkType = self.getLinkType({linkTypeURI: linkTypeURIs[current], projectURI: projectURI}).addCallback(getAllValues);
		} else {
			deferred.callback([]);
		}
		
		return deferred;
	},
	
	/************* HELPERS ***************/
	getArtifactFormatsForProjectFormattedForStore: function(projectURI) {
		return this.getAttributeType({
			projectURI:projectURI, 
			attributeTypeURI: com.ibm.rdm.web.model.types.RRMTypes.getArtifactFormatsURI(projectURI)
			}).addBoth(this, function(type){
				var formatEntries = type.getEnumEntries();
				var formattedEntriesArray = [];
				for (var i=0; i<formatEntries.length; i++) {
					formattedEntriesArray.push({
						name: formatEntries[i].enumTitle,
						value:  formatEntries[i].enumAbout
					});
				}
				return formattedEntriesArray;
		});
	},
	getArtifactFormatDisplayValue: function(projectURI, artifactFormatURI) {
		return this.getAttributeType({
			projectURI:projectURI, 
			attributeTypeURI: com.ibm.rdm.web.model.types.RRMTypes.getArtifactFormatsURI(projectURI)
			}).addBoth(this, function(type){
				try {
					var formatEntries = type.getEnumEntries();
					var formattedEntriesArray = [];
					for (var i=0; i<formatEntries.length; i++) {
						if (formatEntries[i].enumAbout === artifactFormatURI) {
							return formatEntries[i].enumTitle;
						}
					}
				}
				catch(e) {
					console.error(e);
				}
				return "";
		});
	},
	
	isValid: function(attributeDef, attributeType, value) {
		//Summary: Returns true iff the value given is valid for this attributeDefinition and attribute type.
		//   attributeDef: the full attribute definition to use
		//   attributeType: the full attribute type concerned
		//   value: the value to check
		//   return: True if the value given is valid, false otherise.
		
		// At the moment, this check is really to avoid the server failing out-of-bounds defaults for ranges and multi-value enums,
		// so we will return true if there is any doubt whether the items checked fall into that category. This is to be safe.
	
		//Min and Max Values
		var type = attributeType.getValueType();
		var minValue = attributeType.getMinValue();
		if (minValue !== null && this._lessThan(type, value, minValue)) {
			//If the value is less than the min
			return false;
		}
		var maxValue = attributeType.getMaxValue();
		if (maxValue !== null && this._greaterThan(type, value, maxValue)) {
			//If the value is greater than the max
			return false;
		}
		
		//Deal with User Attributes - no validity checking
		if (type === com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.UserType) {
			return true;
		}
		
		//Now deal with multi-value enumerations
		if ((attributeDef.isMultiValued() && dojo.isArray(value)) || attributeType.isEnum()) {
			var result = true;
			//Ensure the array iterated around is not the characters of the string!
			if (dojo.isString(value)) {
				value = [value];
			}
			
			//Convert allowed values into a hashmap
			var legalEnums = {};
			dojo.forEach(attributeType.getEnumEntries(), function(legalEnum) {
				legalEnums[legalEnum.enumAbout] = 1;
			});
			if (attributeDef.isMultiValued()) { 
				//Check given values against legal items
				dojo.forEach(value, function (suggestedEnum) {
					if (!legalEnums[suggestedEnum]) {
						result = false;
					}
				});
				
				return result;
			} else {
				//A single valued enum - check
				return !!legalEnums[value];
			}
		}
		
		return true;
	},
	
	removeInvalidEnums: function (attributeDef, attributeType, value) {
		//Summary: Returns the value given stripped of any invalid enumeration values it may have.
		//   attributeDef: the full attribute definition to use
		//   attributeType: the full attribute type concerned
		//   value: the value to adjust
		//   return: the new value to use.
		if ((attributeDef.isMultiValued() && dojo.isArray(value)) || attributeType.isEnum()) {
			var result;
			
			//Convert allowed values into a hashmap
			var legalEnums = {};
			dojo.forEach(attributeType.getEnumEntries(), function(legalEnum) {
				legalEnums[legalEnum.enumAbout] = 1;
			});
			
			if (attributeDef.isMultiValued()) {
				result = [];
				//Check given multi values against legal items, keeping only legal ones
				dojo.forEach(value, function (suggestedEnum) {
					if (legalEnums[suggestedEnum]) {
						result.push(suggestedEnum);
					}
				});
			
				return result;
			} else if (attributeType.isEnum()) {
				//only return legal single values
				if (legalEnums[value]) {
					return value;
				} else {
					return null;
				}
			}
		}
		//In all other cases return value unchanged
		return value;
		
	},
	
	/************* PRIVATE API ***************/
	_lessThan: function(type, a, b) { 
		//Summary: Compares a and b based on the type given by attrType

		if (this._isDateType(type)) {
			//Need to compare as dates
			var da = new Date(a);
			var db = new Date(b);
			var comp = dojo.date.compare(da, db);
			return comp < 0;
		} else {
			//int and float, compares ok
			return (a*1) < (b*1);
		}
	},
	
	_greaterThan: function(type, a, b) {
		//Summary: Compares a and b based on the type given by attrType

		if (this._isDateType(type)) {
			//Need to compare as dates
			var da = new Date(a);
			var db = new Date(b);
			var comp = dojo.date.compare(da, db);
			return comp > 0;
		} else {
			//int and float, compares ok
			return (a*1) > (b*1);
		}
	},
	
	_isDateType: function(type) {
		if (type == com.ibm.rdm.web.model.types.RRMTypes.getWellKnownAttributeTypes()["dateTime"] ||
			type == com.ibm.rdm.web.model.types.RRMTypes.getWellKnownAttributeTypes()["date"] )
		{
			return true;
		} else {
			return false;
		}
	},
	
	_prefetchTypes: function(multiArgs) {
		// summary:
		//		Use the Multi-Request service to do a bulk prefetch
		//		of the type system for the specified projects. This
		//		will prevent the firing of multiple GET requests
		//		to retrieve the types on demand.
		// multiArgs:
		//		An array of arguments. Takes the following format:
		//
		//		multiArgs: [{
		//			projectURI,			Required, String
		//			projectSnapshotURI 	Optional, String
		//		}, ...]
		// returns:
		//		A deferred. The callback will be fired when the 
		//		prefetch is complete. No data will be passed to the
		//		callback; instead, the _getType(..) calls may then
		//		be made as they normally would.


		var multiProjectTypesDfd = new dojo.Deferred();
		var parsersAndArgs = [];
		var multiGetArgs = [];
		
		dojo.forEach(multiArgs, function(args, idk) {
			if (!args.projectURI) {
				throw new Error("com.ibm.rdm.web.model.types.TypesManager: " +
						"missing required projectURI argument");
			}

			var cacheKeyAndUri = this._getURIAndCacheKey(args.projectURI, args.projectSnapshotURI, args.configurationURI),
			    uri = cacheKeyAndUri.uri,
			    cacheKey = cacheKeyAndUri.cacheKey;
			
			if((args.skipCache || !this._allTypesCache.contains(cacheKey))) {
				var listener = com.ibm.rdm.web.notifications.ChangeNotifier.addListener({
					eventTopics: [
		              com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_CREATED,
		              com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_CHANGED,
		              com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_DELETED,
		              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_DEFINITION_CREATED,
		              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_DEFINITION_CHANGED,
		              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_DEFINITION_DELETED,
		              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_TYPE_CREATED,
		              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_TYPE_CHANGED,
		              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_TYPE_DELETED,
		              com.ibm.rdm.web.notifications.EventTopics.CURRENT_CONFIGURATION_CHANGED],
					projectURI: args.projectURI,
					callbackScope: this,
					neverDelayCallback: true,
					callbackFunction: dojo.partial(function(uri) {
						this._expireCache(uri);
					}, cacheKey)
				});
				this._changeListeners.add(uri, listener);

				var parser = dojo.hitch(this, this.typesFeedParser,
					args.projectURI, args.snapshotURI);
				var requestArgs = {
					uri: uri,
					method: RDMClient.HTTP_METHOD_GET
				};
				
				if (args.configurationURI) {
                    var xhrArgs = {
                        "headers": {}
                    };
                
                    xhrArgs.headers[Headers.VVC_CONFIGURATION] = args.configurationURI;
                    requestArgs.xhrArgs = xhrArgs;
                }

				parsersAndArgs.push({args: args, parser: parser});
				multiGetArgs.push(requestArgs);
				
				//We *must* new-up a deferred object upfront here, and add it to the
				//appropriate cache object. Otherwise subsequent iterations, requests, etc 
				//will end up requesting their own instances for the type system for this particular
				//combination rather than using hte one which may already be in-flight or being handled
				var respDfd = new dojo.Deferred();
				this._allTypesCache.add(cacheKeyAndUri.cacheKey, respDfd);
			}
		}, this);
		
		if(multiGetArgs.length) {
		
			var multiReqDfd = RDMClient.doMultiRequest(multiGetArgs);
			multiReqDfd.addCallback(this, function(multiGetResponses) {
	
				dojo.forEach(multiGetResponses, function(response, idx) {
					var parser = parsersAndArgs[idx].parser;
					var args = parsersAndArgs[idx].args;
					var etag = response.headers.ETag;
					// As long as response has an attribute doc which
					// contains the DOMDocument for the response, this
					// will work. --Christian
					var result = parser(response);
					if(etag) {
						result.version = etag;
					}

					var cacheKeyAndUri = this._getURIAndCacheKey(args.projectURI, args.projectSnapshotURI, args.configurationURI);
					var respDfd = this._allTypesCache.item(cacheKeyAndUri.cacheKey);
					respDfd.callback(result);
				}, this);
				multiProjectTypesDfd.callback();
				return multiGetResponses;
			});

			multiReqDfd.addErrback(this, function(error) {
				multiProjectTypesDfd.errback(error);
			});
		}
		else {
			multiProjectTypesDfd.callback();
		}

		return multiProjectTypesDfd;
	},
	
	clearCache: function(projectURI, projectSnapshotURI, configurationURI) {
		var key = this._getURIAndCacheKey(projectURI, projectSnapshotURI, configurationURI).cacheKey;
		this._expireCache(key);
	},
	
	_expireCache: function(uri) {
		if(this._allTypesCache.contains(uri)) {
			this._allTypesCache.remove(uri);
		}
		if(this._changeListeners.contains(uri)) {
			var listener = this._changeListeners.item(uri);
			this._changeListeners.remove(uri);
			com.ibm.rdm.web.notifications.ChangeNotifier.removeListener(listener);
		}
	},
	
	_expireFullCache: function() {
		// summary:
		//		Expire the entire type system cache for all projects.
		//		This should never be used for anything but testing!
		console.warn("TypesManager._expireFullCache should not be used in production!");
		dojo.forEach(this._allTypesCache.getKeyList(), this._expireCache, this);
	},
	
	_cacheUserMemberProjects: function(/*Boolean?*/ skipCache) {
		// summary:
		//		Cache all the projects of which the current user is
		//		a member. Never ever use this for anything but
		//		for testing! It will lead to misery.
		console.warn("TypesManager._cacheUserMemberProjects should not be used in production!");
		var cacheAllDfd = new dojo.Deferred();
		var projectsDfd =
			com.ibm.rdm.web.common.ProjectCache.getUserMemberProjects();
		
		projectsDfd.addCallback(this, function(projects) {
			var projectsToFetchList = [];
			for(var uri in projects) {
				projectsToFetchList.push({projectURI: uri, skipCache: skipCache});
			}
			var prefetchDfd = this._prefetchTypes(projectsToFetchList);
			prefetchDfd.addCallback(this, function(/*empty*/) {
				cacheAllDfd.callback(projects);
			});
			prefetchDfd.addErrback(this, function(error) {
				cacheAllDfd.errback(error);
			});
			return projects;
		});
		projectsDfd.addErrback(this, function(error) {
			cacheAllDfd.errback(error);
		});
		
		return cacheAllDfd;
	},
	
	_getTypes: function(args) {
		if(dojo.isArray(args)) {
			var multiResponseDfd = new dojo.Deferred();
			var prefetchDfd = this._prefetchTypes(args);
			prefetchDfd.addCallback(this, function(/*empty*/) {
				var multiProjectDfds = [];
				dojo.forEach(args, function(arg) {
					multiProjectDfds.push(this._singleProjectTypesLookup(arg));
				}, this);
				
				var multiProjectDfdList = new dojo.DeferredList(multiProjectDfds);
				multiProjectDfdList.addCallback(this, function(results) {
					
					multiResponseDfd.callback(results);
					return results;
				});
				multiProjectDfdList.addErrback(this, function(error) {
					multiResponseDfd.errback(error);
				});
			});
			prefetchDfd.addErrback(this, function(error) {
				multiResponseDfd.errback(error);
			});

			return multiResponseDfd;
		}
		else {
			return this._singleProjectTypesLookup(args);
		}
	},
	
	_singleProjectTypesLookup: function(args) {
		if (!args.projectURI) {
			throw new Error("com.ibm.rdm.web.model.types.TypesManager: " +
					"missing required projectURI argument");
		}

		var singleProjectTypesDfd = new dojo.Deferred();
		var cacheKeyAndURI = this._getURIAndCacheKey(args.projectURI, args.projectSnapshotURI, args.configurationURI);
		var uri = cacheKeyAndURI.uri;
		var cacheKey = cacheKeyAndURI.cacheKey;

		if(args.skipCache && this._allTypesCache.contains(cacheKey)) {
			this._allTypesCache.remove(cacheKey);
		}
		
		var typesDeferred, listener;
		if(this._allTypesCache.contains(cacheKey)) {
			typesDeferred = this._allTypesCache.item(cacheKey);
		}
		else {
			var typesRequest = {
				uri: uri,
				skipCache: args.skipCache || false,
				// SW: Fix for WI 86344. It appears as if the only
				// differences between two requests are the VVC configurations
				// then FF 24 caches too aggressively. So, if we are going through
				// our own cache, then make sure we definitely go to the server
				// as well.
				preventCache: true, 
				skipErrors: !!args.projectSnapshotURI,
				forceUseDefaultConfiguration: args.forceUseDefaultConfiguration,
				parser: dojo.hitch(this, this.typesFeedParser,
						args.projectURI, args.projectSnapshotURI)
			};
			
			// Make sure these type system requests are in some sort of configuration
			// GC, LC or both are possible.

			if ( typesRequest.xhrArgs == null ) {
				var xhrArgs = {
				        "headers": {}
				    };
				typesRequest.xhrArgs = xhrArgs;
			}

			if (args.globalConfigurationURI) {
				typesRequest.xhrArgs.headers[Headers.VVC_OSLC_CONFIGURATION] = args.globalConfigurationURI;
			}
			else
			{
				typesRequest.xhrArgs.headers[Headers.VVC_OSLC_CONFIGURATION] = VVCUtils.getGlobalConfigurationUri();
			}
			
			if (args.configurationURI) {
			    xhrArgs.headers[Headers.VVC_CONFIGURATION] = args.configurationURI;
			}
			else if(!typesRequest.xhrArgs.headers[Headers.VVC_OSLC_CONFIGURATION])
			{
				xhrArgs.headers[Headers.VVC_CONFIGURATION] = VVCUtils.getCurrentConfigurationUri();
			}
						
			typesDeferred = RDMClient.doGet(typesRequest);
			this._allTypesCache.add(cacheKey, typesDeferred);
			
			listener = com.ibm.rdm.web.notifications.ChangeNotifier.addListener({
				eventTopics: [
	              com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_CREATED,
	              com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_CHANGED,
	              com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_DELETED,
	              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_DEFINITION_CREATED,
	              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_DEFINITION_CHANGED,
	              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_DEFINITION_DELETED,
	              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_TYPE_CREATED,
	              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_TYPE_CHANGED,
	              com.ibm.rdm.web.notifications.EventTopics.ATTRIBUTE_TYPE_DELETED,
	              com.ibm.rdm.web.notifications.EventTopics.CURRENT_CONFIGURATION_CHANGED],
				projectURI: args.projectURI,
				callbackScope: this,
				neverDelayCallback: true,
				callbackFunction: dojo.partial(function(uri) {
					this._expireCache(uri);
				}, uri)
			});
			this._changeListeners.add(uri, listener);
		}

		typesDeferred.addCallback(this, function(response) {

			var result = null;
			if (args.typeURI) {
				result = response[args.feedType].map[args.typeURI];
				if (!result && args.feedType == ATTRIBUTE_DEFINITIONS) {
					result = response[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].map[args.typeURI];
				}
			} else if (args.feedType == ATTRIBUTE_DEFINITIONS && args.includeWellKnown) {
				var attributeDefinitions = response[args.feedType].array;
				result = attributeDefinitions.concat(response[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].array);
			} else {
				result = response[args.feedType].array;
			}
			if (args.isReadOnly) {
				singleProjectTypesDfd.callback(result);
			}
			else {
				singleProjectTypesDfd.callback(dojo.clone(result));
			}

			return response;
		});

		typesDeferred.addErrback(this, function(err){
			singleProjectTypesDfd.errback(err);
		});

		return singleProjectTypesDfd;
	},
	
	_getURIAndCacheKey: function(/*String*/projectURI, /*String?*/ projectSnapshotURI, /*String?*/ configurationURI) {
		if(projectURI) {
			var uri = com.ibm.rdm.web.FRONTING_SERVICE_TYPES +
				"?resourceContext=" + encodeURIComponent(projectURI),
				cacheKey = uri;
				
			if(projectSnapshotURI) {
				uri += "&baseline=" + encodeURIComponent(projectSnapshotURI);
				cacheKey = uri;
			}
			
			if (configurationURI) {
			    cacheKey += "&configurationUri=" + encodeURIComponent(configurationURI);
			}
			
			return {
			    uri: uri,
			    cacheKey: cacheKey
			};
		}
		return null;
	},

	typesFeedParser: function(projectURI, projectSnapshotURI, response) {
		var typesMap = {};
		typesMap[OBJECT_TYPES] = {"array":[], "map":{}};
		typesMap[ATTRIBUTE_DEFINITIONS] = {"array":[], "map":{}};
		typesMap[WELL_KNOWN_ATTRIBUTE_DEFINITIONS] = {"array":[], "map":{}};
		typesMap[ATTRIBUTE_TYPES] = {"array":[], "map":{}};
		typesMap[LINK_TYPES] = {"array":[], "map":{}};

		//get root node
		var rootRdfNode = domutils.getElementsByTagNameNS( 
				response.doc,
				"RDF",
				queryutils.RDF_NAMESPACE, 
				queryutils.RDF_QUALIFIER
		)[0];

		var i;
		for (i=0; i<rootRdfNode.childNodes.length; i++) {
			var typeNode = rootRdfNode.childNodes[i];
			// 65775 The baseName/localName attribute seems to be used interchangeably on IE9 response.
			var typeNodeLabel = typeNode.baseName ? typeNode.baseName : typeNode.localName;
			if (!typeNodeLabel) {
				continue;
			}
			var typeURI = domutils.getAttributeNS(
				typeNode, 
				"about",
				queryutils.RDF_NAMESPACE, 
				queryutils.RDF_QUALIFIER
			);
			var type = null;
			switch (typeNodeLabel) {
				case 'ObjectType':
					type = new ObjectType({
						projectURI: projectURI,
						typeURI: typeURI,
						snapshotURI: projectSnapshotURI,
						xmlContent: typeNode
					});
					typesMap[OBJECT_TYPES].array.push(type);
					typesMap[OBJECT_TYPES].map[typeURI] = type;
					break;
				case 'AttributeDefinition':
					type = new AttributeDefinition({
						projectURI: projectURI,
						attributeDefinitionURI: typeURI,
						snapshotURI: projectSnapshotURI,
						xmlContent: typeNode
					});
					// separate well known attribute definitions from user defined ones for easier searching 
					if (type.isWellKnown()) {
						typesMap[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].array.push(type);
						typesMap[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].map[typeURI] = type;
					}
					else {
						typesMap[ATTRIBUTE_DEFINITIONS].array.push(type);
						typesMap[ATTRIBUTE_DEFINITIONS].map[typeURI] = type;
					}
					break;
				case 'AttributeType':
					type = new AttributeType({
						projectURI: projectURI,
						attributeTypeURI: typeURI,
						snapshotURI: projectSnapshotURI,
						xmlContent: typeNode
					});
					typesMap[ATTRIBUTE_TYPES].array.push(type);
					typesMap[ATTRIBUTE_TYPES].map[typeURI] = type;
					break;
				case 'LinkType':					
					var LinkType = new com.ibm.rdm.web.links.LinkType({
						xmlContent: typeNode,
						projectURI: projectURI,
						snapshotURI: projectSnapshotURI
					});
					
					typesMap[LINK_TYPES].array.push(LinkType);
					typesMap[LINK_TYPES].map[typeURI] = LinkType;				
					break;
			}
		}
		
		//set a reference to attribute type in every attribute definition
		var attributeDefinition;
		for (i=0; i<typesMap[ATTRIBUTE_DEFINITIONS].array.length; i++) {
			attributeDefinition = typesMap[ATTRIBUTE_DEFINITIONS].array[i];
			attributeDefinition.setAttributeTypeObject(typesMap[ATTRIBUTE_TYPES].map[attributeDefinition.getRange()]);
		}
		for (i=0; i<typesMap[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].array.length; i++) {
			attributeDefinition = typesMap[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].array[i];
			attributeDefinition.setAttributeTypeObject(typesMap[ATTRIBUTE_TYPES].map[attributeDefinition.getRange()]);
		}
		
		//sort results
		typesMap[OBJECT_TYPES].array.sort(this.nameComparator);
		typesMap[ATTRIBUTE_DEFINITIONS].array.sort(this.nameComparator);
		typesMap[WELL_KNOWN_ATTRIBUTE_DEFINITIONS].array.sort(this.nameComparator);
		typesMap[ATTRIBUTE_TYPES].array.sort(this.nameComparator);
		
		//Populate the ArtifactIcons cache - if it is around
		var artifactIcons = dojo.getObject("com.ibm.rdm.web.common.ArtifactIcons");
		if (artifactIcons) {
			artifactIcons.addTypes(projectURI, typesMap[OBJECT_TYPES].array);
		}
		
		return typesMap;
	},
	
	nameComparator: function(item1, item2){
		var typeTitle1 = item1.getTitle();
		var typeTitle2 = item2.getTitle();
		return ((typeTitle1 == typeTitle2) ? 0 : ((typeTitle1.toLowerCase() < typeTitle2.toLowerCase()) ? -1 : 1));
	}
};
})();