/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.AttributeChooser");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.query.AttributeSummarySection");
dojo.require("com.ibm.rdm.web.query.ValueConditionStatement");
dojo.require("com.ibm.rdm.web.query.conditionTypes");
dojo.require("com.ibm.rdm.web.server.client.utils.AttributeDefinitionInfo");
dojo.require("com.ibm.rdm.web.util.bidiutils");

dojo.requireLocalization("com.ibm.rdm.web.query", "AttributeChooser");


var AttributeDefinitionInfo = com.ibm.rdm.web.server.client.utils.AttributeDefinitionInfo;
var AttributeSummarySection = com.ibm.rdm.web.query.AttributeSummarySection;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;
var ValueConditionStatement = com.ibm.rdm.web.query.ValueConditionStatement;
var conditionTypes = com.ibm.rdm.web.query.conditionTypes;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;

var	FILTERING_SELECT_RIGHT_GAP = 4;


// TODO Rename this class to instead be ConditionLabelSection (as this class now
// allows selecting artifact types as well as attribute values).
dojo.declare(
	"com.ibm.rdm.web.query.AttributeChooser",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//		A widget which guides users toward selecting a particular attribute.
	//		When an attribute selection is made, a ValueConditionStatement
	//		is then presented below this widget to enable the user to fill in
	//		the query conditions for this attribute.
	// description:
	//		Initially this widget uses a presentation such as the following:
	//		    <Area displaying label>                        [based on an AttributeSummarySection]
	//		    <Area for selecting the attribute name to use> [based on a dojo FilteringSelect]
	//		After the user selects an Attribute Name the presentation becomes ...
	//		for Enumeration, User and ArtifactFormats:
	//		    <Area showing the selected attribute name and values>   [based on an AttributeSummarySection]
	//		    <Area for selecting conditions>                         [based on a ValueConditionStatement]
	//		for Integer, Float, String, Date, Time, DateTime, Duration:
	//		    <Area showing the selected attribute name and operator> [based on an AttributeSummarySection]
	//		    <Area for selecting conditions>                         [based on a ValueConditionStatement]
	//		for Boolean:
	//		    <Area showing the selected attribute name>              [based on an AttributeSummarySection]
	//		    <Area for selecting conditions>                         [based on a ValueConditionStatement]

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "query/templates/AttributeChooser.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,

	// Please see ConditionChooser for documentation on the following
	// fields...
	doAllowAnyAsTheSelectedValue: null,
	isEnvelopedInLinkedArtifactCondition: null,
	scopedArtifactTypeURIs: null,
	previouslySelectedAttributeDefinitionURIs: null,
	restrictedValuesToUseInArtifactFormats: null,
	conditionType: null,
	fixedAttributeDefinitionURI: null,
	fixedAttributeName: null,
	onEditFilter: null,
	onRemoveFilter: null,
	onInvokeApplyAction: null,
	onEnableApplySection: null,
	onDisableApplySection: null,
	onContentAreaHeightHasChanged: null,
	showAttributeConditionSummary: null,
	includeLinkTypes: null,
	filterCLMLinks: null,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_attributeTitleDiv: null,
	_attributeContentDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_isExpanded: null,
	_mapAttributeDefinitionUriToAttributeName: null,
	_attributeSummarySection: null,
	_filteringSelect: null,
	_valueConditionStatement: null,
	_lastAttributeDefinitionInfos: null,
	_isInvalid: null,

	///////////////////////////////////////////////////////////////////////////
	// Public API

	postCreate: function() {
		var itemFileReadStore = this._createItemFileReadStore([]);
		var doShowRemoveFilterControlWhenExpanded;
		switch (this.conditionType) {
		case conditionTypes.AttributeCondition:
			doShowRemoveFilterControlWhenExpanded =
				!this.fixedAttributeDefinitionURI;
			break;
		case conditionTypes.ArtifactTypeCondition:
			doShowRemoveFilterControlWhenExpanded =
				this.isEnvelopedInLinkedArtifactCondition;
			break;
		case conditionTypes.LinkTypeCondition:
			doShowRemoveFilterControlWhenExpanded = true;
		}
		var attributeSummarySection = new AttributeSummarySection({
			onClearFilter: dojo.hitch(this, this.clearFilter, true),
			onEditFilter: this.onEditFilter,
			onRemoveFilter: this.onRemoveFilter,
			doShowRemoveFilterControlWhenExpanded: doShowRemoveFilterControlWhenExpanded,
			onContentAreaHeightHasChanged: this.onContentAreaHeightHasChanged,
			showAttributeConditionSummary: this.showAttributeConditionSummary
		});
		var filteringSelect = new dijit.form.FilteringSelect({
			store: itemFileReadStore,
			autoComplete: true,
			query: { name: "*" },
			selectOnClick: false, //Prevent dijit drop-down issue
								 //http://bugs.dojotoolkit.org/ticket/12714
			onChange: dojo.hitch(this, this._onValueChanged)
			//_onBlur: function() {this._hideResultList();}
		});
		this._isExpanded = true;
		this._attributeTitleDiv.appendChild(attributeSummarySection.domNode);
		this._attributeContentDiv.appendChild(filteringSelect.domNode);
		this._attributeSummarySection = attributeSummarySection;
		this._filteringSelect = filteringSelect;
		if (this.fixedAttributeDefinitionURI
			|| this.conditionType === conditionTypes.ArtifactTypeCondition
			|| this.conditionType === conditionTypes.LinkTypeCondition
			|| this.conditionType === conditionTypes.ModuleFormatInstancesCondition
			) {
			this._openValueSelection(this.fixedAttributeDefinitionURI, true);
		} else {
			this.onDisableApplySection();
		}
		this._updateContent();
	},

	destroy: function() {
		this._filteringSelect.destroy();
		this._attributeSummarySection.destroy();
		if (this._valueConditionStatement) {
			this._valueConditionStatement.destroy();
		}
		this.inherited(arguments);
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.query", "AttributeChooser");
	},

	startup: function() {
		if (!this._started) {
			var widthToUseForFilteringSelect =
				dojo.position(this._attributeSummarySection.domNode).w - FILTERING_SELECT_RIGHT_GAP;
			if (widthToUseForFilteringSelect > 0) {
				this._filteringSelect.domNode.style.width = widthToUseForFilteringSelect + "px";
			}
		}
		this.inherited(arguments);
	},

	getSelectedAttributeDefinition: function() {
		return this._getSelectedAttributeDefinitionUri();
	},

	getConditionInfo: function() {
		// summary:
		//		Please refer to AttributeValueConditionStatement.js for
		//		documentation on a function by this same name.
		return this._valueConditionStatement.getConditionInfo();
	},

	setConditionInfo: function(conditionInfo, onConditionLoaded) {
		// summary:
		//		Please see the documentation for a function by this same name in
		//		ConditionChooser for details.
		// onConditionLoaded:
		//		This function will be called back as soon as the
		//		conditionInfo has been fully loaded into the UI.
		var self = this;
		if ((this.conditionType === conditionTypes.AttributeCondition
				&& this.fixedAttributeDefinitionURI)
			|| this.conditionType === conditionTypes.ArtifactTypeCondition
			|| this.conditionType === conditionTypes.LinkTypeCondition
			) {
			// There's no need to populate a filtering select ... instead just
			// set up the value area.
			this._valueConditionStatement.setConditionInfo(
				conditionInfo, onConditionLoaded);
		}
		else {
			// First fill in the filtering select and THEN set up the value
			// area.
			var onContentUpdated = function() {
				self._openValueSelection(conditionInfo.attributeDefinitionURI, true);
				self._valueConditionStatement.setConditionInfo(
					conditionInfo, onConditionLoaded);
			};
			this._updateContent(onContentUpdated);
		}
	},

	refresh: function(onRefreshComplete) {
		// summary:
		//		Please see ConditionChooser's documentation for a function by
		//		this same name.
		var self = this;
		var onContentUpdated = function() {
			// Update the displayed "title" (e.g. "myPriority") if this chooser
			// is for an attribute (as opposed to a link or an artifact type).
			if (self.conditionType === conditionTypes.AttributeCondition
				&& !self.fixedAttributeName
				&& self._valueConditionStatement
				) {
				var attributeDefinitionUri = self._getSelectedAttributeDefinitionUri();
				if (attributeDefinitionUri) {
					var attributeName =
						self._mapAttributeDefinitionUriToAttributeName[attributeDefinitionUri];
					if (attributeName) {
						self._attributeSummarySection.showConditionTitle(attributeName);
					} else {
						// The selected attribute has been deleted ... so this
						// chooser should no longer be displayed.
						self._isInvalid = true;
					}
				}
			}
			// Update the values found within this chooser (e.g. perhaps an
			// enum value label has changed from "High" to "Highest").
			if (self._valueConditionStatement && !self._isInvalid) {
				var performRefreshComplete = function() {
					if (!self._isExpanded) {
						var summaryLabel = self._valueConditionStatement.getSummaryLabel();
						self._attributeSummarySection.updateCollapasedSummaryLabel(summaryLabel);
					}
					onRefreshComplete();
				};
				self._valueConditionStatement.refresh(performRefreshComplete);
			} else {
				onRefreshComplete();
			}
		};
		// Update the store found in the filtering select to contain updated
		// values (as values may have been added, deleted or renamed).
		this._updateContent(onContentUpdated);
	},

	isInvalid: function() {
		// summary:
		//		Please see ConditionChooser's documentation for a function by
		//		this same name.
		return this._isInvalid;
	},

	clearFilter: function(/*Boolean?*/isCalledFromAttributeSummarySection) {
		if (!isCalledFromAttributeSummarySection) {
			this._attributeSummarySection.clearFilter();
		}
		if (this.fixedAttributeDefinitionURI
			|| this.conditionType === conditionTypes.ArtifactTypeCondition
			|| this.conditionType === conditionTypes.LinkTypeCondition
			) {
			this._valueConditionStatement.clearFilter(true);
			this._attributeSummarySection.showConditionTitle(this.fixedAttributeName);
			if (!this._isExpanded) {
				this.onRemoveFilter();
			}
		} else {
			// Note by SPC: On 10/23/2010 I'm changing the behavior of clear
			// filter to NOT blow away the condition statement and re-enable
			// picking a new attribute ... instead it's now just clearing
			// the content of the previously selected condition statement.
			// However, we may wish to change this in the future to use the
			// old behavior ... so I'm just commenting the old behavior out
			// for now.
/*
			if (this._valueConditionStatement) {
				this._valueConditionStatement.clearFilter();
				this._valueConditionStatement.domNode.style.display = "none";
			}
			if (!this._isExpanded) {
				this.onRemoveFilter();
			} else {
				this._filteringSelect.domNode.style.display = "";
				if (isCalledFromAttributeSummarySection) {
					// Select the text in the filtering select.
					var textbox = this._filteringSelect.textbox;
					var label = textbox.value;
					dijit.selectInputText(textbox);
				}
			}
*/
			this._valueConditionStatement.clearFilter(true);
		}
		if (!this._destroyed
			// Note: For most choosers, clearing the filter should result in
			// disabling apply.  However, this is not true for the "Link type"
			// filter or the "Artifact Types" filter [when used in "Linked
			// artifacts"] as these filters allow for "(any)".  Therefore
			// we need to make the following call to the
			// isConditionStatementSpecified function ...
			&& !this._valueConditionStatement.isConditionStatementSpecified()
			) {
			this.onDisableApplySection();
		}
	},

	isConditionStatementSpecified: function(/*Boolean?*/skipTimeCheck) {
		var isAttributeValueSpecified = false;
		if (this._valueConditionStatement) {
			isAttributeValueSpecified =
				this._valueConditionStatement.isConditionStatementSpecified(skipTimeCheck);
		}
		return isAttributeValueSpecified;
	},

	collapse: function() {
		if (this._isExpanded) {
			this._isExpanded = false;
		
			var summaryLabel = "";
			if (this._valueConditionStatement) {
				summaryLabel = this._valueConditionStatement.getSummaryLabel();
			}
			this._attributeContentDiv.style.display = "none";
			this._attributeSummarySection.handleParentHasCollapsed(summaryLabel);
		}
	},

	expand: function() {
		if (!this._isExpanded) {
			this._isExpanded = true;
			this._attributeContentDiv.style.display = "";
			this._attributeSummarySection.handleParentHasExpanded();
		}
	},

	isExpanded: function() {
		return this._isExpanded;
	},

	isCollapsed: function() {
		return !this._isExpanded;
	},

	showToolbarButtons: function() {
		this._attributeSummarySection.showToolbarButtons();
	},

	hideToolbarButtons: function() {
		this._attributeSummarySection.hideToolbarButtons();
	},

	focus: function() {
		this._attributeSummarySection.focus();
	},

	///////////////////////////////////////////////////////////////////////////
	// Private API

	_setPreviouslySelectedAttributeDefinitionURIsAttr: function(
			previouslySelectedAttributeDefinitionURIs) {
		this.previouslySelectedAttributeDefinitionURIs =
			previouslySelectedAttributeDefinitionURIs;
		// Update the items appearing in the filtering select to account for
		// the fact that the "black list items" have changed.
		if (this._lastAttributeDefinitionInfos) {
			this._filteringSelect.set(
				"store", this._createItemFileReadStore(this._lastAttributeDefinitionInfos)
			);
		}
	},

	_setScopedArtifactTypeURIsAttr: function(scopedArtifactTypeURIs) {
		this.scopedArtifactTypeURIs = scopedArtifactTypeURIs;
		// Update the items appearing in the filtering select to account for
		// the fact that the "black list items" have changed.
		if (this._lastAttributeDefinitionInfos) {
			this._updateContent();
		}
	},

	_appendWellKnownSecondaryAttributeDefinitions: function(
			customAttributeDefinitionInfos) {
		// summary:
		//		Return a list of AttributeDefintionInfo instances in which these
		//		instances are comprised of those found in
		//		customAttributeDefinitionInfos PLUS some well known ones for
		//		attributes "Created on", "Created by", "Artifact format" and "Alternative Spellings".
		var attributeDefintionInfos = customAttributeDefinitionInfos.slice(0);
		var self = this;
		var getWellKnownAttribute = function(attributeName) {
			var attributeUri =
				RRMTypes.getWellKnownAttributeURI(
					self.projectURI, attributeName);
			return (
				RRMTypes.getWellKnownAttributeObjectFromURI(
					attributeUri, self.projectURI)
			);
		};
		var createAttributeDefinitionInfo = function(title, attribute) {
			return (
				new AttributeDefinitionInfo({
					title: title,
					sameAs: attribute.sameAs,
					resourceURI: attribute.uri
				})
			);
		};
		var creatorAttribute =
			getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Creator);
		var createdAttribute =
			getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Created);
		var artifactFormatAttribute =
			getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat);
		var alternateSpellingAttribute =
			getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.AlternateSpelling);
		attributeDefintionInfos.push(
			createAttributeDefinitionInfo(
				this._messages["createdBy"], creatorAttribute));
		attributeDefintionInfos.push(
			createAttributeDefinitionInfo(
				this._messages["createdOn"], createdAttribute));
		attributeDefintionInfos.push(
			createAttributeDefinitionInfo(
				this._messages["artifactFormat"], artifactFormatAttribute));
		attributeDefintionInfos.push(
			createAttributeDefinitionInfo(
				this._messages["alternativeSpelling"], alternateSpellingAttribute));
		attributeDefintionInfos.push(
				new AttributeDefinitionInfo({
					title: this._messages["lockedBy"],
					sameAs: RRMTypes.RDF_LOCKED_BY,
					resourceURI: RRMTypes.RDF_LOCKED_BY
				})
		);
				
		if (this.isEnvelopedInLinkedArtifactCondition) {
			var artifactIdentifierAttribute =
				getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Identifier);
			var modifiedByAttribute =
				getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Contributor);
			var modifiedOnAttribute =
				getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ModifiedOn);
			var nameAttribute =
				getWellKnownAttribute(RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title);
			attributeDefintionInfos.push(
				createAttributeDefinitionInfo(
					this._messages["artifactIdentifier"], artifactIdentifierAttribute));
			attributeDefintionInfos.push(
				createAttributeDefinitionInfo(
					this._messages["modifiedBy"], modifiedByAttribute));
			attributeDefintionInfos.push(
				createAttributeDefinitionInfo(
					this._messages["modifiedOn"], modifiedOnAttribute));
			attributeDefintionInfos.push(
				createAttributeDefinitionInfo(
					this._messages["name"], nameAttribute));
		}
		return attributeDefintionInfos;
	},

	_isAtLeastOneScopedArtifactTypeUriSpecified: function() {
		return (
			this.scopedArtifactTypeURIs &&
			this.scopedArtifactTypeURIs.length > 0
		);
	},

	_filterOutAttributesNotFoundInScopedArtifactTypes: function(
			customAttributeDefinitionInfos, objectTypeInfos) {
		// summary:
		//		Return an array that represents all customAttributeDefinitionInfos
		//		that are found in at least one of the artifact types specified
		//		by this.scopedArtifactTypeURIs.
		// customAttributeDefinitionInfos:
		//		All current custom attribute defintion infos.
		// objectTypeInfos:
		//		All current object type infos.
		var filteredAttributeDefinitionURIs = [];
		var mapScopedArtifactTypeURIsToBoolean = {};
		var mapAttributeDefintionURIToBoolean = {};
		var i;
		// Initialize mapScopedArtifactTypeURIsToBoolean.
		for (i = 0; i < this.scopedArtifactTypeURIs.length; i++) {
			mapScopedArtifactTypeURIsToBoolean[this.scopedArtifactTypeURIs[i]] = true;
		}
		// Initialize mapAttributeDefintionURIToBoolean.
		for (i = 0; i < objectTypeInfos.length; i++) {
			var objectTypeInfo = objectTypeInfos[i];
			var artifactTypeURI = objectTypeInfo.getResourceURI();
			if (mapScopedArtifactTypeURIsToBoolean[artifactTypeURI]) {
				var attributeDefintionURIs = objectTypeInfo.getAttributeDefinitions();
				for (var j = 0; j < attributeDefintionURIs.length; j++) {
					mapAttributeDefintionURIToBoolean[attributeDefintionURIs[j]] = true;
				}
			}
		}
		// Intialize filteredAttributeDefinitionURIs to contain appropriate values.
		for (i = 0; i < customAttributeDefinitionInfos.length; i++) {
			var customAttributeDefintionInfo = customAttributeDefinitionInfos[i];
			var attributeDefinitionURI = customAttributeDefintionInfo.getResourceURI();
			if (mapAttributeDefintionURIToBoolean[attributeDefinitionURI]) {
				filteredAttributeDefinitionURIs.push(customAttributeDefintionInfo);
			}
		}
		return filteredAttributeDefinitionURIs;
	},

	_updateContent: function(/*Callback?*/onContentUpdated) {
		// summary:
		//		Update the items found in the filtering select (via a call to
		//		_createItemFileReadStore) ... but only AFTER obtaining the
		//		the appropriate information needed to create these items
		//		(and in the case of an attribute condition, we also take care
		//		here to prune the attribute defintions down to just those
		//		pertaining to the attributes found in this.scopedArtifactTypeURIs).
		// onContentUpdated:
		//		An optional callback that will be invoked with the filtering
		//		select that has been updated.
		var self = this;
		var isAtLeastOneScopedArtifactTypeUriSpecified =
			this._isAtLeastOneScopedArtifactTypeUriSpecified();
		var doQueryForObjectTypes = (
			isAtLeastOneScopedArtifactTypeUriSpecified
			&& this.conditionType === conditionTypes.AttributeCondition
		);
		var doQueryForAttributeDefinitions =
			(this.conditionType === conditionTypes.AttributeCondition);
		var numOutstandingDeferreds = 0;
		if (doQueryForAttributeDefinitions) { numOutstandingDeferreds++; }
		if (doQueryForObjectTypes) { numOutstandingDeferreds++; }
		var deferredContent = {
			// This property will only be filled in by onAttributeDefintionsComplete
			// if doQueryForAttributeDefinitions is true.
			customAttributeDefinitionInfos: null,
			// This property will only be filled in by onObjectTypesComplete if
			// doQueryForObjectTypes is true.
			objectTypeInfos: null
		};
		var updateContentIfAppropriate = function() {
			if (numOutstandingDeferreds === 0) {
				// Update the items appearing in the filtering select.
				if (self.conditionType === conditionTypes.AttributeCondition) {
					var customAttributeDefinitionsInfosToUse = (
						isAtLeastOneScopedArtifactTypeUriSpecified ?
						self._filterOutAttributesNotFoundInScopedArtifactTypes(
							deferredContent.customAttributeDefinitionInfos,
							deferredContent.objectTypeInfos
						) :
						deferredContent.customAttributeDefinitionInfos
					);
					var attributeDefinitionInfosToUse =
						self._appendWellKnownSecondaryAttributeDefinitions(
							customAttributeDefinitionsInfosToUse
						);
					self._lastAttributeDefinitionInfos = attributeDefinitionInfosToUse;
					self._filteringSelect.set(
						"store", self._createItemFileReadStore(attributeDefinitionInfosToUse)
					);
				} else {
					self._createItemFileReadStore([]);
				}
				if (onContentUpdated) {
					onContentUpdated();
				}
			}
		};
		var onObjectTypesComplete = function(objectTypeInfos) {
			numOutstandingDeferreds--;
			deferredContent.objectTypeInfos = objectTypeInfos;
			updateContentIfAppropriate();
			return objectTypeInfos;
		};
		var onAttributeDefintionsComplete = function(customAttributeDefinitionInfos) {
			numOutstandingDeferreds--;
			deferredContent.customAttributeDefinitionInfos = customAttributeDefinitionInfos;
			updateContentIfAppropriate();
			return customAttributeDefinitionInfos;
		};
		if (doQueryForObjectTypes) {
			var objectTypesDeferred = TypesManager.getObjectTypesForProject({
				projectURI: this.projectURI,
				projectSnapshotURI: this.baselineURI
			});
			objectTypesDeferred.addCallback(this, onObjectTypesComplete);
		}
		if (doQueryForAttributeDefinitions) {
			var attributeDefinitionsDeferred =
				TypesManager.getAttributeDefinitionsForProject({
					projectURI: this.projectURI,
					projectSnapshotURI: this.baselineURI
				});
			attributeDefinitionsDeferred.addCallback(this, onAttributeDefintionsComplete);
		}
		if (! (doQueryForObjectTypes || doQueryForAttributeDefinitions)) {
			updateContentIfAppropriate();
		}
	},

	_createItemFileReadStore: function(attributeDefinitionInfos) {
		attributeDefinitionInfos.sort(
			function (attributeDefinitionInfo1, attributeDefinitionInfo2) {
				var value = 0;
				var title1 = attributeDefinitionInfo1.getTitle();
				var title2 = attributeDefinitionInfo2.getTitle();
				if (title1 < title2) {
					value = -1;
				} else if (title2 < title1) {
					value = 1;
				}
				return value;
			}
		);
		var mapOfPreviouslySelectedAttributeDefinitionUris = {};
		var mapAttributeDefinitionUriToAttributeName = {};
		if (this.previouslySelectedAttributeDefinitionURIs) {
			for (var i = 0; i < this.previouslySelectedAttributeDefinitionURIs.length; i++) {
				var uri = this.previouslySelectedAttributeDefinitionURIs[i];
				mapOfPreviouslySelectedAttributeDefinitionUris[uri] = true;
			}
		}
		var count = 0;
		var items = [];
		// Fill items while taking care to omit previously selected attr defintion uris.
		dojo.forEach(
			attributeDefinitionInfos,
			function(attributeDefinitionInfo) {
				var attributeName = attributeDefinitionInfo.getTitle();
				var attributeDefinitionUri = attributeDefinitionInfo.getResourceURI();
				mapAttributeDefinitionUriToAttributeName[attributeDefinitionUri] = attributeName;
				if (!mapOfPreviouslySelectedAttributeDefinitionUris[attributeDefinitionUri]) {
					items[count++] = {
						name: BidiUtils.setTextDirUCC(attributeName),
						uri: attributeDefinitionUri
					};
				}
			}
		);
		this._mapAttributeDefinitionUriToAttributeName = mapAttributeDefinitionUriToAttributeName;
		return new dojo.data.ItemFileReadStore({
			data: {
				identifier: "uri",
				name: "label",
				items: items
			}
		});
	},

	_getSelectedAttributeDefinitionUri: function() {
		var attributeDefinitionUri = this._filteringSelect.get("value");
		if (!attributeDefinitionUri) {
			if (this._valueConditionStatement) {
				var conditionInfo = this._valueConditionStatement.getConditionInfo();
				attributeDefinitionUri = conditionInfo.attributeDefinitionURI;
			}
		}
		return attributeDefinitionUri;
	},

	_onValueChanged: function() {
		var attributeDefinitionUri = this._getSelectedAttributeDefinitionUri();
		if (attributeDefinitionUri.length > 0) {
			this._openValueSelection(attributeDefinitionUri);
		}
	},

	_openValueSelection: function(attributeDefinitionUri, /*Boolean?*/doAvoidChangingFocus) {
		// summary:
		//		An attribute as been specified.  Now show a value selector so
		//		the user can select the value(s) for this attribute.
		var attributeName = (
			this.fixedAttributeName ?
			this.fixedAttributeName :
			this._mapAttributeDefinitionUriToAttributeName[attributeDefinitionUri]
		);
		this._attributeSummarySection.showConditionTitle(attributeName);
		// Hide the FilteringSelect widget and show the ValueConditionStatement widget
		// instead.  Make sure this ValueConditionStatement is showing a ConditionSection
		// that pertains to the selected attribute definition.
		this._filteringSelect.domNode.style.display = "none";
		if (!this._valueConditionStatement) {
			this._valueConditionStatement = new ValueConditionStatement({
				projectURI: this.projectURI,
				baselineURI: this.baselineURI,
				doAllowAnyAsTheSelectedValue: this.doAllowAnyAsTheSelectedValue,
				isEnvelopedInLinkedArtifactCondition: this.isEnvelopedInLinkedArtifactCondition,
				conditionType: this.conditionType,
				restrictedValuesToUseInArtifactFormats: this.restrictedValuesToUseInArtifactFormats,
				attributeSummarySection: this._attributeSummarySection,
				onInvokeApplyAction: this.onInvokeApplyAction,
				onEnableApplySection: this.onEnableApplySection,
				onDisableApplySection: this.onDisableApplySection,
				onContentAreaHeightHasChanged: this.onContentAreaHeightHasChanged,
				includeLinkTypes: this.includeLinkTypes, 
				filterCLMLinks: this.filterCLMLinks
			});
			this._attributeContentDiv.appendChild(this._valueConditionStatement.domNode);
		} else {
			this._valueConditionStatement.domNode.style.display = "";
		}
		switch (this.conditionType) {
			case conditionTypes.AttributeCondition:
				this._valueConditionStatement.setSelectedAttributeDefinition(attributeDefinitionUri);
				break;
			// TODO Add a case statement for conditionTypes.LinkTypeCondition as well ...
		}
		if (!doAvoidChangingFocus) {
			this._attributeSummarySection.focus();
		}
	}
});


})();
