/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.CompoundConditionChooser");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");

dojo.require("com.ibm.rdm.web.query.ConditionChooser");
dojo.require("com.ibm.rdm.web.query.ViewServiceClient");
dojo.require("com.ibm.rdm.web.query.artifactTypeUtils");
dojo.require("com.ibm.rdm.web.query.conditionTypes");

dojo.requireLocalization("com.ibm.rdm.web.query", "CompoundConditionChooser");


var artifactTypeUtils = com.ibm.rdm.web.query.artifactTypeUtils;
var conditionTypes = com.ibm.rdm.web.query.conditionTypes;
var ConditionChooser = com.ibm.rdm.web.query.ConditionChooser;

// The following color can be thought of as "medium orange" ...
var COLOR_TO_USE_WHEN_MOUSEOVER_OCCURS_IN_CONDITION_CHOOSER = "#ffeacf";

var	IE7_VERTICAL_GAP_DIV_HEIGHT = "20px";
var IE7_VERTICAL_GAP_HR_PADDING_TOP = "";
var IE7_VERTICAL_GAP_HR_MARGIN_TOP = "7px";


dojo.declare(
	"com.ibm.rdm.web.query.CompoundConditionChooser",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//		Provides the "condition chooser" used by the "Linked artifacts"
	//		filter.
	// description:
	//		This class enables users to specify any of the following as a
	//		"linked artifact condition":
	//		1. A link condition.
	//		   Example:
	//		       "Link type exists Link To"
	//		2. A link condition, followed by an artifact type condition.
	//		   Example:
	//		       "Link type exists Link To, Parent Of"
	//		       "Artifact type is SUPP"
	//		3. A link condition, followed by an artifact type condition,
	//		   followed by one or more attribute condtions for the artifact(s)
	//		   found on the other end of the link(s).
	//		   Example:
	//		       "Link type exists Link To"
	//		       "Artifact type is SUPP"
	//		       "Priority is high"
	//		4. A link condition, followed by one or more attribute condtions
	//		   for the artifact(s) found on the other end of the link(s).
	//		   Example:
	//		       "Link type exists Link To"
	//		       "Priority is high"
	//		   [ Note: In this case the "Artifact type" is implicitly set to
	//		     "(any)". ]

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "query/templates/CompoundConditionChooser.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,
	baselineURI: null,

	// onRemoveFilter: [const] Callback
	//		This callback is invoked whenever the user clicks on the "Remove
	//		filter" icon when this widget is displayed in the collapsed state.
	onRemoveFilter: null,

	// onEditFilter: [const] Callback
	//		This callback is invoked whenever the user clicks on the "Edit
	//		filter" icon from one of the condition choosers that are found
	//		in this CompoundConditionChooser.
	onEditFilter: null,

	// onInvokeApplyAction: [const] Callback
	//		This callback is invoked whenever an implicit apply is performed.
	//		For example, if the user sets the Supplemental -> Contact text
	//		value to "Palin" and then hits the Enter key, we'll invoke this
	//		callback to apply the appropriate action.
	onInvokeApplyAction: null,

	// onEnableApplySection: [const] Callback
	//		This callback is invoked whenever it's noticed that the user has
	//		provided enough information to enable an Apply action.  For
	//		example, as soon as the user has selected one of the Feature ->
	//		Priority values (e.g. "High") we'll invoke this callback.
	onEnableApplySection: null,

	// onDisableApplySection: [const] Callback
	//		This callback is invoked whenever it's noticed that the user has
	//		hasn't provided enough information to enable an Apply action.  For
	//		example, if the user removes the only selected Feature -> Priority
	//		(e.g. "High") we'll invoke this callback.
	onDisableApplySection: null,

	// onContentAreaHeightHasChanged: [const] Callback
	//		This callback is invoked whenever it's noticed that the chooser's
	//		content height has changed.
	onContentAreaHeightHasChanged: null,
	
	isEnvelopedInLinkedArtifactCondition: true,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_conditionChoosers: null,
	_nextStepAnchor: null,
	_scopedArtifactTypeURIs: null,
	_toolbar: null,
	_isPerformingSetConditionInfo: null,
	_isFocused: null,
	_isMouseOver: null,
	_isExpanded: null,
	filterCLMLinks: null,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_expansionStateDiv: null,
	_conditionChoosersDiv: null,
	_nextStepDiv: null,
	_bottomDiv: null,
	_toolbarDiv: null,
	_verticalGapWithDottedLineDiv: null,
	_verticalGapHR: null,

	///////////////////////////////////////////////////////////////////////////
	// Public API

	postCreate: function() {
		this.inherited(arguments);

		this._nextStepDiv.style.display = "none";

		// Create the Link Type condition chooser that is always the first
		// chooser presented.
		var linkConditionChooser =
			new ConditionChooser({
				projectURI: this.projectURI,
				baselineURI: this.baselineURI,
				colorToUseWhenMouseOverOccurs: COLOR_TO_USE_WHEN_MOUSEOVER_OCCURS_IN_CONDITION_CHOOSER,
				conditionType: conditionTypes.LinkTypeCondition,
				isEnvelopedInLinkedArtifactCondition: this.isEnvelopedInLinkedArtifactCondition,
				fixedAttributeName: this._messages["linkType"],
				onRemoveFilter: dojo.hitch(this, this._onRemoveFilter),
				onEditFilter: dojo.hitch(this, this._onEditFilter),
				onInvokeApplyAction: dojo.hitch(this, this._onInvokeApplyAction),
				onEnableApplySection: dojo.hitch(this, this._onEnableApplySection),
				onDisableApplySection: dojo.hitch(this, this._onDisableApplySection),
				onContentAreaHeightHasChanged: dojo.hitch(this, this._onContentAreaHeightHasChanged), 
				filterCLMLinks: this.filterCLMLinks
			});
		this._conditionChoosersDiv.appendChild(linkConditionChooser.domNode);

		// Create the anchor that's used to guide the user into creating the
		// next consecutive chooser.  This anchor appears beneath the current
		// chooser as soon as the current chooser is in a state where it can
		// be applied (e.g. for an string attribute condition chooser, it can
		// applied as soon as the first character has been entered into its
		// text field).
		var nextStepAnchor =
			dojo.create(
				"a",
				{ href: "#" },
				this._nextStepDiv
			);
		nextStepAnchor.appendChild(document.createTextNode(""));
		this.connect(
			nextStepAnchor,
			"onclick",
			function(event) {
				dojo.stopEvent(event);
				this._openNextConditionChooser();
				return arguments;
			}
		);
		this.connect(
			nextStepAnchor,
			"onkeydown",
			function(event) {
				if (event.keyCode === dojo.keys.ENTER) {
					dojo.stopEvent(event);
					this._openNextConditionChooser();
				}
				return arguments;
			}
		);

		// Initialize the controls found in this._bottomDiv.
		this._initBottomIcons();
		this.connect(this._expansionStateDiv, "onmouseover", function(event) {
			if (!this._isMouseOver
				&& this._eventLocationIsWithinThisWidget(event)
				) {
				this._showAsMouseOver();
				this._updateStatePresentation();
			}
		});
		this.connect(this._expansionStateDiv, "onmouseout", function(event) {
			if (this._isMouseOver
				&& !this._eventLocationIsWithinThisWidget(event)
				) {
				this._stopShowingAsMouseOver();
				this._updateStatePresentation();
			}
		});

		// Initialize the class fields.
		this._setBottomDivContentToDivider();
		this._isExpanded = true;
		this._nextStepAnchor = nextStepAnchor;
		this._conditionChoosers = [ linkConditionChooser ];
		this._scopedArtifactTypeURIs = [];
	},

	destroy: function() {
		dojo.forEach(
			this._conditionChoosers,
			function(conditionChooser) {
				conditionChooser.destroyRecursive();
			}
		);
		if (this._toolbar) {
			this._toolbar.destroyRecursive();
		}
		this.inherited(arguments);
	},

	startup: function() {
		this._updateNextStepAnchor();
		for (var i = 0; i < this._conditionChoosers.length; i++) {
			this._conditionChoosers[i].startup();
		}
		this._updateStatePresentation();
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.query", "CompoundConditionChooser");
	},

	refresh: function(onRefreshComplete) {
		// summary:
		//		Please see ConditionChooser's documentation for a function by
		//		this same name.
		// description:
		//		This function is similar to the one found in ConditionChooser,
		//		but this version needs to account for all of the individual
		//		CondtionChoosers that are found in a CompoundConditionChooser.
		var self = this;
		var numChoosers = this._conditionChoosers.length;
		var numOutstandingRefreshOperations = numChoosers;
		var onChooserRefreshComplete = function() {
			numOutstandingRefreshOperations--;
			if (numOutstandingRefreshOperations === 0) {
				// If this chooser involved a Link type that's been deleted
				// it's not an issue (as "(any)" would be used).
				// If this chooser involved an Attribute Type that's been
				// deleted, this is ALSO not an issue (as "(any)" would be
				// used).  However, some of the attribute type choosers found
				// in this compound chooser MAY have been deleted (i.e. those
				// for which the associated attribute type has been deleted)
				// and we need to remove these choosers from this compound
				// chooser.  This is why we start at index "2" in the
				// following 'for' loop ... as we want to skip over the Link
				// and Artifact Type choosers.
				for (var i = 2; i < self._conditionChoosers.length; i++) {
					var conditionChooser = self._conditionChoosers[i];
					if (conditionChooser.isInvalid()) {
						self._onRemoveFilter(
							conditionChooser,
							true // isRefresh
						);
					}
				}
				onRefreshComplete();
			}
		};
		for (var i = 0; i < numChoosers; i++) {
			this._conditionChoosers[i].refresh(onChooserRefreshComplete);
		}
	},

	clearFilter: function(skipApply) {
		// TODO Implement?
	},

	focus: function() {
		if (this._conditionChoosers.length > 0) {
			var firstChooser = this._conditionChoosers[0];
			firstChooser.focus();
		}
	},

	onFocus: function() {
		this._showAsMouseOver();
		this._showAsFocused();
		this._updateStatePresentation();
	},

	onBlur: function() {
		this._stopShowingAsMouseOver();
		this._stopShowingAsFocused();
		this._updateStatePresentation();
	},

	isConditionStatementSpecified: function() {
		// summary:
		//		Return true if and only if at least one of this
		//		CompoundConditionChooser's constituent ConditionChoosers is
		//		currently collapsed.
		return this.isAtLeastOneChooserCollapsed();
	},

	getConditionInfo: function() {
		// summary:
		//		Get information about the link condition that the user
		//		has	specified.
		// description:
		//		A precondition to calling this function is that the caller
		//		has already called isConditionStatementSpecified to know
		//		that this CompoundConditionChooser SHOULD be involved in the
		//		query (i.e. that this CompoundConditionChooser is known to
		//		contain at least one collapsed ConditionChooser).
		//		Returns a JSON object that represents query information for
		//		all of the currently collapsed constituent ConditionChoosers
		//		found within this CompoundConditionChooser.
		// returns:
		//		A JSON object in the following format:
		//		  {
		//		    linkConditionInfo:
		//		      {
		//		        attributeValues: [ <operator>, <attributeValue> * ],
		//		      },
		//		    artifactTypeConditionInfo:
		//		      {
		//		         attributeValues: [ <attributeValue> * ],
		//		      },
		//		    attributeConditionInfos:
		//		      [
		//		        {
		//		          attributeDefinitionURI: <attributeDefinitionURI>
		//		          attributeType: <a type from RRMTypes.ATTRIBUTE_TYPES>
		//		          attributeValues: [ <operator>, <attributeValue> * ],
		//		          isEnum: <true or false>
		//		        },
		//		        ...
		//		      ]
		//		  }
		var conditionInfo = {};
		conditionInfo.linkConditionInfo = this._getLinkTypeConditionInfo();
		if (this._conditionChoosers.length > 1) {
			var attributeConditionInfos = [];
			if (this._conditionChoosers.length > 2) {
				// We know that at least one attribute condition chooser has been
				// specified ... but we only want to include those attribute
				// condition choosers that are currently collapsed as an expanded
				// chooser should NOT be involved in the query until it's applied
				// (or reapplied [if editing a pre-existing ConditionChooser]).

				// Fill attributeConditionInfos with appropriate values.
				var attributeConditionChoosers = this._conditionChoosers.slice(2);
				dojo.forEach(
					attributeConditionChoosers,
					function(attributeConditionChooser) {
						if (attributeConditionChooser.isCollapsed()) {
							// This attribute chooser is collapsed so we SHOULD
							// involve it in the query.
							attributeConditionInfos.push(
								attributeConditionChooser.getConditionInfo()
							);
						}
					}
				);

				if (attributeConditionInfos.length > 0) {
					conditionInfo.attributeConditionInfos = attributeConditionInfos;
				}
			}
			// The artifact type condition should only be involved in the query if
			// at least one attribute chooser is involved (i.e. is collapsed) OR
			// if the artifact type condition chooser is currently collapsed.
			if (attributeConditionInfos.length > 0 || this._conditionChoosers[1].isCollapsed()) {
				conditionInfo.artifactTypeConditionInfo =
					this._getArtifactTypeConditionInfo();
			}
		}
		return conditionInfo;
	},

	setConditionInfo: function(conditionInfo) {
		// summary:
		//		Set this chooser up so that it's completely initialized based on
		//		the content found in the passed conditionInfo.
		// description:
		//		Calling this function will result in the chooser being
		//		initialized to a pre-set "compound" condition.  This "compound"
		//		condition must use the format specified by the associated
		//		getConditionInfo function found in this class.
		var self = this;
		var allConditionChoosersHaveLoadedDeferred = new dojo.Deferred();
		var deferredArray = [];
		var linkConditionInfo = conditionInfo.linkConditionInfo;
		var artifactTypeConditionInfo = conditionInfo.artifactTypeConditionInfo;
		var attributeConditionInfos = (conditionInfo.attributeConditionInfos || []);
		var numSetConditionInfoDeferredsRequired = (
			(linkConditionInfo ? 1 : 0)
			// Note: if artifactTypeConditionInfo is null AND we've got a least
			// one element in attributeConditionInfos, this means we're creating
			// an artifact type chooser based on "(any)" and we need to account
			// for the deferred used to initialize this chooser.
			+ ((artifactTypeConditionInfo || attributeConditionInfos.length > 0) ? 1 : 0)
			+ attributeConditionInfos.length
		);
		var tallyOfSetConditionInfoDeferreds = 0;
		var trackDeferredAndCreateDeferredListIfNeeded = function(deferred) {
			deferredArray.push(deferred);
			tallyOfSetConditionInfoDeferreds++;
			if (tallyOfSetConditionInfoDeferreds === numSetConditionInfoDeferredsRequired) {
				var deferredList = new dojo.DeferredList(deferredArray);
				deferredList.addCallback(function() {
					self._isPerformingSetConditionInfo = false;
					allConditionChoosersHaveLoadedDeferred.callback();
				});
			}
		};
		var initializeConditionChooser = function(conditionChooser, conditionInfo) {
			var deferred = conditionChooser.setConditionInfo(conditionInfo);
			deferred.addCallback(function(){
				conditionChooser.collapse();
				if (conditionChooser.conditionType === conditionTypes.ArtifactTypeCondition) {
					var conditionInfo = conditionChooser.getConditionInfo();
					artifactTypeUtils.computeSelectedArtifactTypes(
						conditionInfo, self.projectURI, self.baselineURI
					).addBoth(
						function(artifactTypeURIs) {
							// Set the _scopedArtifactTypeURIs field.
							// We make sure to do this here rather than in
							// _openNextConditionChooser because we want to
							// create the choosers as fast as possible and
							// we shouldn't have to wait for the deferred
							// required by Artifact Type BEFORE creating
							// the attribute condition choosers that appear
							// below it.
							self._scopedArtifactTypeURIs = artifactTypeURIs;
						}
					);
				}
			});
			return deferred;
		};
		var addAnotherConditionChooser = function(conditionInfo, onSetConditionInfoDeferredCreated) {
			self._openNextConditionChooser(true, function(conditionChooser) {
				var deferred = initializeConditionChooser(conditionChooser, conditionInfo);
				onSetConditionInfoDeferredCreated(deferred);
			});
		};

		// Set things up so that seeding the "Link type" and "Artifact Type"
		// won't result in showing an "Apply" section.
		this._isPerformingSetConditionInfo = true;

		// If needed, initialize the Link Type condition chooser (which has
		// already been created in postCreate).
		// Note: the Link Type condition chooser by default will be initialized
		// to "Any" as it's value so it shouldn't be a requirement that
		// linkConditionInfo be specified.
		if (linkConditionInfo) {
			var conditionChooser = this._conditionChoosers[0];
			trackDeferredAndCreateDeferredListIfNeeded(
				initializeConditionChooser(conditionChooser, linkConditionInfo));
		}

		// If needed, add and initialize an Artifact Type chooser.
		if (artifactTypeConditionInfo) {
			addAnotherConditionChooser(artifactTypeConditionInfo, function(deferred) {
				trackDeferredAndCreateDeferredListIfNeeded(deferred);
			});
		}
		else if (attributeConditionInfos && attributeConditionInfos.length > 0) {
			// Account for the fact that artifactTypeConditionInfo was not
			// passed, but at least one attribute condition was ... meaning
			// we've got to set up an artifact type chooser based on "(any)"
			// (so that the user can subsequently go back and edit it).
			var artifactTypeConditionInfoBasedOnAny = {
				conditionType: com.ibm.rdm.web.query.conditionTypes.ArtifactTypeCondition,
				attributeValues: [
					com.ibm.rdm.web.query.ViewServiceClient.ARTIFACT_TYPE_OPERATORS.IsAny
				]
			};
			addAnotherConditionChooser(artifactTypeConditionInfoBasedOnAny, function(deferred) {
				trackDeferredAndCreateDeferredListIfNeeded(deferred);
			});
		}

		// If needed, add and initialize attribute choosers.
		if (attributeConditionInfos) {
			for (var i = 0; i < attributeConditionInfos.length; i++) {
				addAnotherConditionChooser(attributeConditionInfos[i], function(deferred) {
					trackDeferredAndCreateDeferredListIfNeeded(deferred);
				});
			}
		}

		return allConditionChoosersHaveLoadedDeferred;
	},

	isAttributeConditionSpecifiedButNoLongerValid: function() {
		// TODO implement
		return false;
	},

	collapse: function() {
		if (this._isExpanded) {
			this._isExpanded = false;
			var lastConditionChooser = this._getLastConditionChooser();
			if (lastConditionChooser
				&& lastConditionChooser.isExpanded()
				&& !lastConditionChooser.isConditionStatementSpecified()
				) {
				lastConditionChooser.destroyRecursive();
				this._conditionChoosers.splice(this._conditionChoosers.length-1, 1);
			}
			for (var i = 0; i < this._conditionChoosers.length; i++) {
				this._conditionChoosers[i].collapse();
			}
			this._updateStatePresentation();
			this._nextStepDiv.style.display = "none";
		}
	},

	expand: function() {
		if (!this._isExpanded) {
			this._isExpanded = true;
			this._updateStatePresentation();
			this._openNextConditionChooser();
		}
	},

	isExpanded: function() {
		return this._isExpanded;
	},

	isCollapsed: function() {
		return !this._isExpanded;
	},

	isInvalid: function() {
		// summary:
		//		The consumer of this class should call this function is after
		//		calling the refresh function to see if this chooser is still
		//		valid.

		// Note: We always return true due to the fact that even if the Link
		// type that this chooser was based on has now been deleted we'd STILL
		// have a valid chooser as the "Link type exist (any)" (or "Link type
		// does not exist (any)") would be the new expression found in the
		// chooser.
		return false;
	},

	isAtLeastOneChooserCollapsed: function() {
		var isAtLeastOneChooserCollapsed = false;
		for (var i = 0; i < this._conditionChoosers.length; i++) {
			if (this._conditionChoosers[i].isCollapsed()) {
				isAtLeastOneChooserCollapsed = true;
				break;
			}
		}
		return isAtLeastOneChooserCollapsed;
	},

	///////////////////////////////////////////////////////////////////////////
	// Private API

	_showAsMouseOver: function() {
		this._isMouseOver = true;
	},

	_stopShowingAsMouseOver: function() {
		this._isMouseOver = false;
	},

	_showAsFocused: function() {
		this._isFocused = true;
	},

	_stopShowingAsFocused: function() {
		this._isFocused = false;
	},

	_getLinkTypeConditionInfo: function() {
		// summary:
		//		Get the condition info associated with the link type chooser.
		// description:
		//		A precondition to calling this function is that it's already
		//		known that it makes sense to involve the link type in the query.
		//		The basic rule is that "a chooser will only be involved in the
		//		query if it's currently collapsed" (as "expanded" choosers must
		//		be applied [or reapplied] before being involved in the query).
		//		Returns a condition info that represents the appropriate query
		//		information for the link type chooser.
		var conditionInfo;
		if (this._conditionChoosers[0].isCollapsed()) {
			conditionInfo = this._conditionChoosers[0].getConditionInfo();
		} else {
			// It must be the case that there's a collapsed artifact type
			// chooser ... in which case we need a "default" value for the
			// link type chooser (as it's currently expanded, and its
			// intermediate changed values should not be considered when
			// performing a query [i.e. not until it's reapplied]).
			// In other words until this link type chooser is reapplied we'll
			// use the default of "Link Type exists (any)" for the link type
			// chooser's query condition.
			var linkConditionChooser =
				new ConditionChooser({
					projectURI: this.projectURI,
					baselineURI: this.baselineURI,
					conditionType: conditionTypes.LinkTypeCondition,
					fixedAttributeName: this._messages["linkType"]
				});
			conditionInfo = linkConditionChooser.getConditionInfo();
		}
		return conditionInfo;
	},

	_getArtifactTypeConditionInfo: function() {
		// summary:
		//		Get the condition info associated with the artifact type chooser.
		// description:
		//		A precondition to calling this function is that it's already
		//		known that it makes sense to involve the artifact type in the
		//		query.
		//		The basic rule is that "a chooser will only be involved in the
		//		query if it's currently collapsed" (as "expanded" choosers must
		//		be applied [or reapplied] before being involved in the query).
		//		Returns a condition info that represents the appropriate query
		//		information for the artifact type chooser.
		var conditionInfo;
		if (this._conditionChoosers[1].isCollapsed()) {
			conditionInfo = this._conditionChoosers[1].getConditionInfo();
		} else {
			// It must be the case that there's an attribute condition chooser
			// that's currently collapsed ... in which case we need a "default"
			// value for the artifact type chooser (as it's currently expanded,
			// and its intermediate changed values should not be considered when
			// performing a query [i.e. not until it's reapplied]).
			// In other words until this artifact type chooser is reapplied
			// we'll use the default of "Artifact Types is (any)" for the
			// artifact type chooser's query condition.
			var artifactTypeConditionChooser =
				new ConditionChooser({
					projectURI: this.projectURI,
					baselineURI: this.baselineURI,
					isEnvelopedInLinkedArtifactCondition: true,
					conditionType: conditionTypes.ArtifactTypeCondition,
					fixedAttributeName: this._messages["artifactTypes"]
				});
			conditionInfo = artifactTypeConditionChooser.getConditionInfo();
		}
		return conditionInfo;
	},

	_eventLocationIsWithinThisWidget: function(event) {
		var position = dojo.position(this._expansionStateDiv);
		// TODO: Revisit this ... I suspect I could avoid this IE7 issue if I
		// add an appropriate margin-top/margin-bottom somewhere in the CSS
		// rules associated with this class.
		var fudgeFactor = ((dojo.isIE && dojo.isIE < 8) ? 1 : 0);
		var coords = {
			x: position.x + fudgeFactor,
			y: position.y + fudgeFactor,
			w: position.w - fudgeFactor,
			h: position.h - fudgeFactor
		};
		return (
			(event.clientX > coords.x
				&& (event.clientX < (coords.x + coords.w)))
			&&
			(event.clientY > coords.y
				&& (event.clientY < (coords.y + coords.h)))
		);
	},

	_initBottomIcons: function() {
		var toolbar = new jazz.ui.Toolbar();
		var onAddNewCondition = function() {
			this.expand();
		};
		var onRemoveFilter = function() {
			this._onRemoveFilter();
		};
		toolbar.addChild(new jazz.ui.toolbar.Button({
			title: this._messages["addCondition"],
			imgClass: "add-image",
			onClick: dojo.hitch(this, onAddNewCondition)
		}));
		toolbar.addChild(new jazz.ui.toolbar.Button({
			title: this._messages["removeFilterGroup"],
			imgClass: "clear-content-image",
			onClick: dojo.hitch(this, onRemoveFilter)
		}));
		this._toolbarDiv.appendChild(toolbar.domNode);
		this._toolbar = toolbar;
	},

	_setBottomDivContentToDivider: function() {
		this._toolbarDiv.style.display = "none";
		this._verticalGapWithDottedLineDiv.style.display = "";
	},

	_setBottomDivContentToToolbar: function() {
		this._toolbarDiv.style.display = "";
		this._verticalGapWithDottedLineDiv.style.display = "none";
	},

	_updateStatePresentation: function() {
		if (this._isExpanded) {
			this._expansionStateDiv.className = "expanded";
			this._bottomDiv.style.display = "none";
			this._setBottomDivContentToDivider();
		}
		else {
			if (this._isMouseOver || this._isFocused) {
				this._expansionStateDiv.className = "collapsedMouseOver";
				this._bottomDiv.style.display = "";
				this._setBottomDivContentToToolbar();
			} else {
				this._expansionStateDiv.className = "collapsedMouseOut";
				this._bottomDiv.style.display = "";
				this._setBottomDivContentToDivider();
			}
		}
	},

	_getLastConditionChooser: function() {
		var lastConditionChooser = null;
		if (this._conditionChoosers.length > 0) {
			lastConditionChooser =
				this._conditionChoosers[this._conditionChoosers.length-1];
		}
		return lastConditionChooser;
	},

	_updateNextStepAnchor: function() {
		var labelToUse = "";
		if (this._conditionChoosers.length === 1) {
			labelToUse = this._messages["chooseArtifactType"];
		}
		else if (this._conditionChoosers.length === 2) {
			labelToUse = this._messages["chooseAttributes"];
		}
		else if (this._conditionChoosers.length > 2) {
			labelToUse = this._messages["addAnotherAttribute"];
		}
		this._nextStepAnchor.firstChild.nodeValue = labelToUse;
	},

	_getCurrentlySpecifiedAttributeDefinitionURIs: function() {
		var currentlySpecifiedAttributeDefinitionURIs = [];
		if (this._conditionChoosers.length > 2) {
			var attributeConditionChoosers = this._conditionChoosers.slice(2);
			dojo.forEach(
				attributeConditionChoosers,
				function(attributeConditionChooser) {
					if (attributeConditionChooser.isCollapsed()) {
						currentlySpecifiedAttributeDefinitionURIs.push(
							attributeConditionChooser.getAttributeDefinitionUri());
					}
				}
			);
		}
		return currentlySpecifiedAttributeDefinitionURIs;
	},

	_openNextConditionChooser: function(
			/*Boolean?*/isCalledBySetConditionInfo, /*Callback?*/onConditionChooserCreated) {
		// summary:
		//		Open up another condition chooser and update the hyperlink that
		//		guides the user to the next logical step to take (e.g. "Choose
		//		artifact type" or "Choose attributes").
		// onConditionChooserCreated: optional Callback
		//		If passed, this callback will be invoked when the chooser has
		//		been created.  This callback will be passed a single argument,
		//		the newly created ConditionChooser instance.
		var self = this;
		if (!isCalledBySetConditionInfo) {
			this._collapseOrRemoveCurrentlyExpandedChooser();
		}
		this._nextStepDiv.style.display = "none";
		var lastConditionChooser = this._getLastConditionChooser();
		if (!isCalledBySetConditionInfo && lastConditionChooser) {
			lastConditionChooser.collapse();
		}
		var invokeCallbackIfNeeded = function(conditionChooser) {
			if (onConditionChooserCreated) {
				onConditionChooserCreated(conditionChooser);
			}
		};
		if (this._conditionChoosers.length === 1) {
			// Add an Artifact Type condition chooser.
			var conditionChooser = new ConditionChooser({
				projectURI: this.projectURI,
				baselineURI: this.baselineURI,
				colorToUseWhenMouseOverOccurs: COLOR_TO_USE_WHEN_MOUSEOVER_OCCURS_IN_CONDITION_CHOOSER,
				isEnvelopedInLinkedArtifactCondition: true,
				conditionType: conditionTypes.ArtifactTypeCondition,
				fixedAttributeName: this._messages["artifactTypes"],
				onRemoveFilter: dojo.hitch(this, this._onRemoveFilter),
				onEditFilter: dojo.hitch(this, this._onEditFilter),
				onInvokeApplyAction: dojo.hitch(this, this._onInvokeApplyAction),
				onEnableApplySection: dojo.hitch(this, this._onEnableApplySection),
				onDisableApplySection: dojo.hitch(this, this._onDisableApplySection),
				onContentAreaHeightHasChanged: dojo.hitch(this, this._onContentAreaHeightHasChanged)
			});
			this._addConditionChooser(conditionChooser);
			invokeCallbackIfNeeded(conditionChooser);
		}
		else {
			// Add another attribute value chooser.
			var previouslySelectedAttributeDefinitionURIs =
				this._getCurrentlySpecifiedAttributeDefinitionURIs();
			var createAttributeConditionChooser = function(scopedArtifactTypeURIs) {
				var conditionChooser = new ConditionChooser({
					projectURI: self.projectURI,
					baselineURI: self.baselineURI,
					colorToUseWhenMouseOverOccurs: COLOR_TO_USE_WHEN_MOUSEOVER_OCCURS_IN_CONDITION_CHOOSER,
					isEnvelopedInLinkedArtifactCondition: true,
					conditionType: conditionTypes.AttributeCondition,
					scopedArtifactTypeURIs: scopedArtifactTypeURIs,
					previouslySelectedAttributeDefinitionURIs: previouslySelectedAttributeDefinitionURIs,
					onRemoveFilter: dojo.hitch(self, self._onRemoveFilter),
					onEditFilter: dojo.hitch(self, self._onEditFilter),
					onInvokeApplyAction: dojo.hitch(self, self._onInvokeApplyAction),
					onEnableApplySection: dojo.hitch(self, self._onEnableApplySection),
					onDisableApplySection: dojo.hitch(self, self._onDisableApplySection),
					onContentAreaHeightHasChanged: dojo.hitch(self, self._onContentAreaHeightHasChanged)
				});
				return conditionChooser;
			};
			if (!isCalledBySetConditionInfo && this._conditionChoosers.length === 2) {
				// The first time we add an attribute value chooser, we must
				// _first_ take care to reset the _scopedArtifactTypeURIs so
				// that the attributes that the user can choose amongst are
				// appropriately scoped to just those that exist for the
				// selected artifact types on the other end of the link.
				// Note: if isCalledBySetConditionInfo is true, it's the
				// caller's responsibility to make sure that the
				// _scopedArtifactTypeURIs is set to an appropriate value.
				var conditionInfo = lastConditionChooser.getConditionInfo();
				artifactTypeUtils.computeSelectedArtifactTypes(
					conditionInfo, this.projectURI, this.baselineURI
				).addBoth(
					function(artifactTypeURIs) {
						// Set the _scopedArtifactTypeURIs field.
						self._scopedArtifactTypeURIs = artifactTypeURIs;
						// Add a new attribute condition chooser.
						var conditionChooser = createAttributeConditionChooser(artifactTypeURIs);
						self._addConditionChooser(conditionChooser);
						invokeCallbackIfNeeded(conditionChooser);
					}
				);
			} else {
				// Add a new attribute condition chooser.
				var conditionChooser =
					createAttributeConditionChooser(this._scopedArtifactTypeURIs);
				this._addConditionChooser(conditionChooser);
				invokeCallbackIfNeeded(conditionChooser);
			}
		}
	},

	_addConditionChooser: function(conditionChooser) {
		this._conditionChoosersDiv.appendChild(conditionChooser.domNode);
		this._conditionChoosers.push(conditionChooser);
		conditionChooser.startup();

		if (!this._isPerformingSetConditionInfo) {
			// Note: If we were to call "conditionChooser.focus()" directly
			// here it would result in the key release being consumed
			// by the first drop down button in the ConditionChooser,
			// which results in opening this drop down.  To avoid this,
			// we're using the workaround of using a settimeout here ...
			window.setTimeout(
				dojo.hitch(conditionChooser, conditionChooser.focus),
				0
			);
		}

		this._updateNextStepAnchor();
		this._onContentAreaHeightHasChanged();
	},

	_onRemoveFilter: function(/*Object?*/conditionChooser, /*Boolean?*/isRefresh) {
		var self = this;
		var index = (
			conditionChooser ?
			dojo.indexOf(this._conditionChoosers, conditionChooser) :
			0
		);
		if (!conditionChooser) {
			conditionChooser = this._conditionChoosers[0];
		}
		var wasLastChooser = (index === this._conditionChoosers.length-1);
		var wasChooserExpanded = conditionChooser.isExpanded();
		var performOnInvokeApplyActionIfAppropriate = function() {
			if (!isRefresh) {
				self._onInvokeApplyAction();
			}
		};
		if (index === 1) {
			// The user is removing the Artifact Type condition chooser.
			// For now, we'll remove this chooser and all choosers below it.
			// TODO When we support editing previously collapsed filters we
			// should change this behavior to just enable editing the existing
			// filter and not remove any that appear below it.
			for (var i = this._conditionChoosers.length-1; i > 0; i--) {
				this._conditionChoosers[i].destroyRecursive();
			}
			this._conditionChoosers = [ this._conditionChoosers[0] ];
			if (wasLastChooser && wasChooserExpanded) {
				this._updateNextStepAnchor();
				this._onEnableApplySection();
			}
			else if (!wasChooserExpanded) {
				performOnInvokeApplyActionIfAppropriate();
			}
		}
		else if (index > 1) {
			// The user is removing an attribute condition chooser.
			this._removeConditionChooser(conditionChooser, index);
			if (wasLastChooser && wasChooserExpanded) {
				this._updateNextStepAnchor();
				this._onEnableApplySection();
			}
			else if (!this._isAtLeastOneChooserExpanded()) {
				performOnInvokeApplyActionIfAppropriate();
			}
		}
		else {
			// The user is removing the Link Type chooser.
			if (dojo.isFunction(this.onRemoveFilter)) {
				this.onRemoveFilter(this, this.skipApply);
			}
		}
	},

	_removeConditionChooser: function(conditionChooser, /*Integer?*/index) {
		if (!index) {
			index = dojo.indexOf(this._conditionChoosers, conditionChooser);
		}
		this._conditionChoosers.splice(index, 1);
		conditionChooser.destroyRecursive();
	},

	_isAtLeastOneChooserExpanded: function() {
		var isAtLeastOneChooserExpanded = false;
		for (var i = 0; i < this._conditionChoosers.length; i++) {
			if (this._conditionChoosers[i].isExpanded()) {
				isAtLeastOneChooserExpanded = true;
				break;
			}
		}
		return isAtLeastOneChooserExpanded;
	},

	_collapseOrRemoveCurrentlyExpandedChooser: function(/*Object?*/conditionChooser) {
		for (var i = 0; i < this._conditionChoosers.length; i++) {
			if (conditionChooser !== this._conditionChoosers[i]) {
				var chooser = this._conditionChoosers[i];
				if (chooser.isExpanded()) {
					if (chooser.isConditionStatementSpecified()) {
						chooser.collapse();
					} else {
						this._removeConditionChooser(chooser);
					}
				}
			}
		}
	},

	_onEditFilter: function(conditionChooser) {
		this._collapseOrRemoveCurrentlyExpandedChooser(conditionChooser);
		if (dojo.isFunction(this.onEditFilter)) {
			this.onEditFilter(this);
		}
		this._isExpanded = true;
		this._updateStatePresentation();
	},

	_onInvokeApplyAction: function() {
		if (dojo.isFunction(this.onInvokeApplyAction)) {
			this.onInvokeApplyAction();
		}
	},

	_onEnableApplySection: function() {
		if (!this._isPerformingSetConditionInfo) {
			this._nextStepDiv.style.display = "";
			this._onContentAreaHeightHasChanged();
			if (dojo.isFunction(this.onEnableApplySection)) {
				this.onEnableApplySection(this);
			}
		}
	},

	_onDisableApplySection: function() {
		if (!this._isPerformingSetConditionInfo) {
			this._nextStepDiv.style.display = "none";
			this._onContentAreaHeightHasChanged();
			if (dojo.isFunction(this.onDisableApplySection)) {
				this.onDisableApplySection(this);
			}
		}
	},

	_onContentAreaHeightHasChanged: function() {
		if (dojo.isFunction(this.onContentAreaHeightHasChanged)) {
			this.onContentAreaHeightHasChanged(this);
		}
	}
});


})();
