/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.ConditionChooser");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("com.ibm.rdm.web.query.AttributeChooser");
dojo.require("com.ibm.rdm.web.query.conditionTypes");


var AttributeChooser = com.ibm.rdm.web.query.AttributeChooser;

var DEFAULT_COLOR_TO_USE_WHEN_MOUSEOVER_OCCURS_IN_CONDITION_CHOOSER = "#fff4e2";


dojo.declare(
	"com.ibm.rdm.web.query.ConditionChooser",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//		A widget for guiding the user into specifying a query condition
	//		involving a single attribute condition (e.g. "Priority is
	//		High, Medium").

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "query/templates/ConditionChooser.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,
	baselineURI: null,

	// skipApply: [const] Boolean
	//		This field is used to avoid refreshing the attribute chooser as a
	//		whole after each attribute is removed.
	skipApply: false,

	// doAllowAnyAsTheSelectedValue: Boolean
	//		This field should only be set to true if the 'conditionType' field
	//		is set to com.ibm.rdm.web.query.conditionTypes.ArtifactTypeCondition
	//		AND you'd like the artifact picker to show "(any)" whenever no
	//		values have been selected.
	doAllowAnyAsTheSelectedValue: false,

	// isEnvelopedInLinkedArtifactCondition: [const] Boolean
	//		Set to true iff this condition chooser is part of a
	//		CompoundConditionChooser used by the "Linked artifacts" filter.
	isEnvelopedInLinkedArtifactCondition: null,

	// TODO Consider renaming this to be colorToUseWhenFocusOrMouseOverOccurs ...
	// colorToUseWhenMouseOverOccurs: [const] String
	//		Sets the color that should be used when mouseover occurs
	//		(e.g. "#ffeacf").
	colorToUseWhenMouseOverOccurs: null,

	// scopedArtifactTypeURIs: Array
	//		An array of the artifact type URIs that, when specified, should be
	//		used to narrow down the set of attributes that the user can select
	//		amongst.  For example if the user has:
	//		- custom attributes A, B, and C
	//		- attribute types MyType1 (using A, B and C), MyType2 (using B)
	//		  and MyType3 (using B and C)
	//		and the user does the following:
	//		- selects attribute types MyType2 and MyType3
	//		- clicks on "More attributes" (to define a new attribute condition)
	//		the resulting attribute condition chooser should ONLY allow
	//		selecting amoung attributes B and C (and not A as neither MyType2
	//		nor MyType3 contains attribute A).
	scopedArtifactTypeURIs: null,

	// previouslySelectedAttributeDefinitionURIs: Array
	//		An array of the "black list" attribute definition URIs that should
	//		NOT be selectable by the user.
	//		Note: If this field is specified, it's expected that the
	//		fixedAttributeDefinitionURI field will not ALSO be specified.
	previouslySelectedAttributeDefinitionURIs: null,

	// restrictedValuesToUseInArtifactFormats: [Object] {[Array] artifactFormats, [Array] mimeTypes}
	//		By default, the "Artifact format" filter shows with artifact formats
	//		followed by all registerd MIME types.  If this field is passed, the filter will instead 
	//		show the artifact formats and/or the mime types that are specified.  If no value is specified for
	//		one of the filters, all values will be displayed for that filter. Example values: 
	//		
	//		Only jpg type files will be displayed, no artifact formats will be displayed.
	//		  {
	//		    artifactFormats: [],
	//			mimeTypes: [com.ibm.rdm.web.common.MimeTypes.JPG]
	//		  }
	//		
	//		jpg type files and all artifact formats will be displayed.
	//		  {
	//			mimeTypes: [com.ibm.rdm.web.common.MimeTypes.JPG]
	//		  }
	restrictedValuesToUseInArtifactFormats: null,

	// conditionType: [const] Object (which acts like an enum)
	//		The value must be one of those defined by:
	//		    com.ibm.rdm.web.query.conditionTypes
	//		For example, com.ibm.rdm.web.query.conditionTypes.AttributeCondition.
	conditionType: null,

	// fixedAttributeDefinitionURI: [const] String
	//		The URI of an attribute definition that this chooser will be
	//		permanently bound to (as in you won't be able to select any other
	//		attribute as the filtering select for other attributes will never
	//		be shown if this field is passed).
	//		Note: If this field is specified, it's expected that the
	//		previouslySelectedAttributeDefinitionURIs field will not ALSO be
	//		specified.
	fixedAttributeDefinitionURI: null,

	// TODO Consider renaming this to something like fixedConditionTitle.
	// fixedAttributeName: [const] String
	//		The attribute name that should appear at the top of the chooser when
	//		fixedAttributeDefinitionURI has been passed.  Note: normally this
	//		field is only be specified if fixedAttributeDefinitionURI is ALSO
	//		specified, but the "Linked artifacts" filter also makes use of it.
	fixedAttributeName: null,

	// onRemoveFilter: [const] Callback
	//		This callback is invoked whenever the user clicks on the "Remove
	//		filter" icon when this widget is displayed in the collapsed state.
	onRemoveFilter: null,

	// onEditFilter: [const] Callback
	//		This callback is invoked whenever the user clicks on the "Edit
	//		filter" icon when this widget is displayed in the collapsed state.
	onEditFilter: null,

	// onInvokeApplyAction: [const] Callback
	//		This callback is invoked whenever an implicit apply is performed.
	//		For example, if the user sets the Supplemental -> Contact text
	//		value to "Palin" and then hits the Enter key, we'll invoke this
	//		callback to apply the appropriate action.
	onInvokeApplyAction: null,

	// onEnableApplySection: [const] Callback
	//		This callback is invoked whenever it's noticed that the user has
	//		provided enough information to enable an Apply action.  For
	//		example, as soon as the user has selected one of the Feature ->
	//		Priority values (e.g. "High") we'll invoke this callback.
	onEnableApplySection: null,

	// onDisableApplySection: [const] Callback
	//		This callback is invoked whenever it's noticed that the user has
	//		hasn't provided enough information to enable an Apply action.  For
	//		example, if the user removes the only selected Feature -> Priority
	//		(e.g. "High") we'll invoke this callback.
	onDisableApplySection: null,

	// onContentAreaHeightHasChanged: [const] Callback
	//		This callback is invoked whenever it's noticed that the chooser's
	//		content height has changed.
	onContentAreaHeightHasChanged: null,
	
	// showAttributeConditionSummary: Boolean
	//		Allow user to toggle whether the summary of chosen conditions
	//		is displayed (true) or hidden (false)
	showAttributeConditionSummary: true,
	
	// includeLinkTypes: Array of linkTypeURIs
	//		Filter for LinkConditionSection.  Includes only the given linkType URIs from
	//		the list of link types.  LinkTypeURIs are sameAs value for the link type, if it exists, otherwise,
	//		resourceURI for the linkType.
	includeLinkTypes: null,
	
	// filterCLMLinks: Boolean
	//		Used to indicate whether CLM link types should be filtered from the list of link types for
	//		CM-enabled projects.  
	filterCLMLinks: null,
	
	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_attributeChooser: null,
	_isFocused: null,
	_isMouseOver: null,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Public API

	postCreate: function() {
		var self = this;
		var attributeChooser =
			new AttributeChooser({
				projectURI: this.projectURI,
				baselineURI: this.baselineURI,
				doAllowAnyAsTheSelectedValue: this.doAllowAnyAsTheSelectedValue,
				isEnvelopedInLinkedArtifactCondition: this.isEnvelopedInLinkedArtifactCondition,
				scopedArtifactTypeURIs: this.scopedArtifactTypeURIs,
				previouslySelectedAttributeDefinitionURIs: this.previouslySelectedAttributeDefinitionURIs,
				restrictedValuesToUseInArtifactFormats: this.restrictedValuesToUseInArtifactFormats,
				conditionType: this.conditionType,
				fixedAttributeDefinitionURI: this.fixedAttributeDefinitionURI,
				fixedAttributeName: this.fixedAttributeName,
				onEditFilter: dojo.hitch(this, this._onEditFilter),
				onRemoveFilter:
					function() {
						if (dojo.isFunction(self.onRemoveFilter)) {
							self.onRemoveFilter(self, self.skipApply);
						}
					},
				onInvokeApplyAction: this.onInvokeApplyAction,
				onEnableApplySection: dojo.hitch(this, this._onEnableApplySection),
				onDisableApplySection: dojo.hitch(this, this._onDisableApplySection),
				onContentAreaHeightHasChanged: dojo.hitch(this, this._onContentAreaHeightHasChanged),
				showAttributeConditionSummary: this.showAttributeConditionSummary,
				includeLinkTypes: this.includeLinkTypes,
				filterCLMLinks: this.filterCLMLinks
			});
		this._contentDiv.appendChild(attributeChooser.domNode);
		this.connect(this._contentDiv, "onmouseover", function(event) {
			if (this.isCollapsed()
				&& this._eventLocationIsWithinThisWidget(event)
				) {
				this._showAsMouseOver();
				this._updateTitleAreaDisplay();
			}
		});
		this.connect(this._contentDiv, "onmouseout", function(event) {
			if (this.isCollapsed()
				&& !this._eventLocationIsWithinThisWidget(event)
				) {
				this._stopShowingAsMouseOver();
				this._updateTitleAreaDisplay();
			}
		});
		this.connect(this._contentDiv, "onkeydown", this._onKeyDown);
		this._attributeChooser = attributeChooser;
	},

	destroy: function() {
		this._attributeChooser.destroy();
		this.inherited(arguments);
	},

	startup: function() {
		if (!this._started) {
			this._attributeChooser.startup();
		}
		this.inherited(arguments);
	},

	refresh: function(onRefreshComplete) {
		// summary:
		//		Refresh the content displayed in this chooser as the type
		//		information may have changed.
		// description:
		//		Here are the things that MAY have been updated ...
		//		If this chooser is for an attribute:
		//		- The selected attribute may have been deleted.
		//		- The selected attribute name may have changed.
		//		- If the selected attribute base type is "enum", some enum
		//		  values may have been added or deleted or some of the value
		//		  labels may have been renamed.
		//		- A new attribute may have been added (so a newly opened
		//		  attribute chooser that's displaying a filtering select
		//		  would need to update its selectable values).
		//		- An attribute that's not the currently selected attribute
		//		  may have been deleted (so a newly opened attribute chooser
		//		  that's displaying a filtering select would need to update
		//		  its selectable values).
		//		If this chooser is for artifact type:
		//		- Some or all of the selected artifact types may have been
		//		  deleted.
		//		- Some or all of the selected artifact types may have been
		//		  renamed.
		//		- New artifact types may have been added.
		//		- Artifact types that aren't currently selected may have been
		//		  deleted.
		//		If this chooser is for a Link type:
		//		- Some or all of the selected link types may have been
		//		  deleted.
		//		- Some or all of the selected artifact types may have been
		//		  renamed.
		//		- New link types may have been added.
		//		- Link types that aren't currently selected may have been
		//		  deleted.
		if (this._started) {
			this._attributeChooser.refresh(onRefreshComplete);
		} else {
			onRefreshComplete();
		}
	},

	clearFilter: function(skipApply) {
		this.skipApply = skipApply;
		this._attributeChooser.clearFilter();
	},

	focus: function() {
		if (this.isCollapsed()) {
			this._showAsFocused();
			this._updateTitleAreaDisplay();
			this._contentDiv.focus();
		} else {
			this._attributeChooser.focus();
		}
	},

	onFocus: function() {
		if (this.isCollapsed()) {
			this._showAsFocused();
			this._updateTitleAreaDisplay();
		}
	},

	onBlur: function() {
		this._stopShowingAsFocused();
		this._updateTitleAreaDisplay(true);
	},

	isConditionStatementSpecified: function(/*Boolean?*/skipTimeCheck) {
		return this._attributeChooser.isConditionStatementSpecified(skipTimeCheck);
	},

	getConditionInfo: function() {
		// summary:
		//		Get information about the query condition that the user has
		//		specified.
		// description:
		//		A precondition is that the isConditionStatementSpecified has
		//		already been called to make sure that the user has already
		//		specified a condition.  Please see the "returns" section for
		//		the format of the returned property list.  Here are details
		//		about to what to expect ...
		//		If this.conditionType is com.ibm.rdm.web.query.conditionTypes.AttributeCondition:
		//		  The returned property list will be in the following format:
		//		|   {
		//		|       conditionType: com.ibm.rdm.web.query.conditionTypes.AttributeCondition,
		//		|       attributeDefinitionURI: "https://localhost:9443/rdm/types/_qDjI0VkAEd-hroJYYjiZFg",
		//		|       attributeType: "http://www.w3.org/2001/XMLSchema#string",
		//		|       attributeValues: [
		//		|           "startsWith",
		//		|           "customer, geography"
		//		|       ],
		//		|       isEnum: false,
		//		|       ...
		//		|   }
		//		  the property values will differ, but all of these properties
		//		  should be returned.  This above example case is for a String
		//		  attributes, and the attributeValues property will differ
		//		  depending on the value type:
		//		    IntegerType         [ <operator>, <integer> ]
		//		    Float               [ <operator>, <float> ]
		//		    BooleanType         [ <boolean> ]
		//		    DateType            [ <operator>, <start-Date-object>, <end-Date-object> ]
		//		    TimeType            [ <operator>, <start-Date-object>, <end-Date-object> ]
		//		    DateTimeType        [ <operator>, <start-Date-object>, <end-Date-object> ]
		//		    StringType          [ <operator>, <string> ]
		//		    DurationType        [ <operator>, <duration-string1>, <duration-string2> ]
		//		    UserType            [ <userURI> [, <userURI>]* ]
		//		    ArtifactFormatsType [ <artifactFormatURI> [, <artifactFormatURI>]* ]
		//		  and, in the case of an enum, the attributeValues property will
		//		  look something like this:
		//		                        [ <enumAboutURI> [, <enumAboutURI>]* ]
		//		If this.conditionType is com.ibm.rdm.web.query.conditionTypes.ArtifactTypeCondition:
		//		  The returned value will instead look like this (from ArtifactTypeConditionSection):
		//		|   {
		//		|       attributeValues: [
		//		|           "https://localhost:9443/rdm/types/_FH09IpQKEd-dVan3opZ38g",
		//		|           "https://localhost:9443/rdm/types/_FH09IpQKEd-dVan3opZ38k"
		//		|       ],
		//		|       ...
		//		|   }
		//		For examples, please see the getConditionInfo function found in
		//		the various *ConditionSection classes (e.g. StringConditionSection,
		//		EnumConditionSection, ArtifactTypeConditionSection, etc.).
		// returns:
		//		A property list in either the "attribute condition info"
		//		format (see above description covering the case of
		//		conditionTypes.AttributeCondition) or the "artifact type
		//		condition info" format (see above description covering the
		//		case of conditionTypes.ArtifactTypeCondition).
		return this._attributeChooser.getConditionInfo();
	},

	setConditionInfo: function(conditionInfo) {
		// summary:
		//		Set this chooser up so that it's completely initialized based on
		//		the content found in the passed conditionInfo.
		// description:
		//		Calling this function will result in the chooser being
		//		initialized to a pre-set condition.
		//		If this.conditionType is conditionTypes.AttributeCondition ...
		//		  The chooser will be initialized to the choice of a
		//		  particular attribute defintion, and for this attribute
		//		  definition, the appropriate UI form controls will also
		//		  be initialized to values found within the passed
		//		  conditionInfo.
		//		otherwise if this.conditionType is conditionTypes.ArtifactTypeCondition ...
		//		  The chooser will be initialized to enable picking
		//		  artifact types, and and the artifact types mentioned
		//		  in the conditionInfo will be pre-selected.
		// returns:
		//		A deferred that will be called back as soon as this chooser
		//		has fully initialized itself based on the passed
		//		conditionInfo.
		// example:
		//		If the passed conditionInfo was the following:
		//			{
		//				conditionType: com.ibm.rdm.web.query.conditionTypes.AttributeCondition,
		//				attributeDefinitionURI:
		//					"https://localhost:9443/rdm/types/_vBVf0XreEd-Z766mfaTFFg",
		//				attributeType: RRMTypes.ATTRIBUTE_TYPES.DateType,
		//				attributeValues: [
		//					com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsBetween,
		//					new Date(2010, 5, 18),
		//					new Date(2010, 5, 22)
		//				],
		//				isEnum: false
		//			},
		//		where that attributeDefinitionURI value represents a "myDate"
		//		attribute, this chooser would show "myDate" as the attribute
		//		that's being edited and the UI controls (found within
		//		DateConditionSection) would be pre-set to the passed dates
		//		and the operator drop down would be pre-set to "is between".
		var deferred = new dojo.Deferred();
		var onConditionLoaded = function() {
			deferred.callback();
		};
		this._attributeChooser.setConditionInfo(
			conditionInfo, onConditionLoaded);
		return deferred;
	},

	isAttributeConditionSpecifiedButNoLongerValid: function() {
		// TODO implement
		return false;
	},

	collapse: function() {
		this._attributeChooser.collapse();
		dojo.attr(this._contentDiv, "tabindex", "0");
		// It _may_ be the case that we're collapsing as a result of
		// _onEditFilter completing ... and that function explicitly sets the
		// background color, so we remove this explicit setting with the
		// following call.
		this._contentDiv.style.background = "";
	},

	expand: function(/*Boolean?*/doAvoidChangingFocus) {
		dojo.attr(this._contentDiv, "tabindex", "-1");
		this._attributeChooser.expand();
		if (!doAvoidChangingFocus) {
			this._attributeChooser.focus();
		}
	},

	isExpanded: function() {
		return this._attributeChooser.isExpanded();
	},

	isCollapsed: function() {
		return this._attributeChooser.isCollapsed();
	},

	getAttributeDefinitionUri: function() {
		// summary:
		//		Get the URI of the attribute definition that is currently
		//		specified within this condition chooser.
		// description:
		//		This function should only be called if this.conditionType
		//		is set to AttributeCondition.
		return this._attributeChooser.getSelectedAttributeDefinition();
	},

	isInvalid: function() {
		// summary:
		//		Return true iff this chooser is no longer "valid".
		// description:
		//		When the refresh function is called, it MAY result in finding
		//		that the attribute that it's associated with has been deleted
		//		from the type system.  When this is detected, the chooser
		//		becomes "invalid" and the caller may call this function to
		//		act on this fact.  For example, if you have a "myAttr" filter
		//		in the "More attributes" section and this attribute has been
		//		deleted, the consumer of this class would call this function
		//		(after calling the refresh function) to know that a required
		//		next step would be to remove this "myAttr" filter from the
		//		"More attributes" section (as the filter is no longer valid).
		return this._attributeChooser.isInvalid();
	},

	///////////////////////////////////////////////////////////////////////////
	// Private API

	_showAsMouseOver: function() {
		this._isMouseOver = true;
	},

	_stopShowingAsMouseOver: function() {
		this._isMouseOver = false;
	},

	_showAsFocused: function() {
		this._isFocused = true;
	},

	_stopShowingAsFocused: function() {
		this._isFocused = false;
	},

	_updateTitleAreaDisplay: function(/*Boolean?*/isBlurScenario) {
		if (!isBlurScenario && (this._isFocused || this._isMouseOver)) {
			var colorToUseWhenMouseOverOccurs = (
				this.colorToUseWhenMouseOverOccurs ?
				this.colorToUseWhenMouseOverOccurs :
				DEFAULT_COLOR_TO_USE_WHEN_MOUSEOVER_OCCURS_IN_CONDITION_CHOOSER
			);
			this._contentDiv.style.background = colorToUseWhenMouseOverOccurs;
			this._attributeChooser.showToolbarButtons();
		}
		else if (!this._isFocused || !this._isMouseOver) {
			this._contentDiv.style.background = "";
			this._attributeChooser.hideToolbarButtons();
		}
	},

	_onKeyDown: function(event) {
		if (event.target === this._contentDiv
			&& (event.keyCode === dojo.keys.SPACE || event.keyCode === dojo.keys.ENTER)
			) {
			if (this.isExpanded()) {
				this.collapse();
			} else {
				// Note: If we were to call "this._onEditFilter()" directly
				// here it would result in the key release being consumed
				// by the first drop down button in the ConditionChooser,
				// which results in opening this drop down.  To avoid this,
				// we're using the workaround of using a settimeout here ...
				window.setTimeout(
					dojo.hitch(this, this._onEditFilter),
					0
				);
			}
			dojo.stopEvent(event);
		}
		return event;
	},

	_eventLocationIsWithinThisWidget: function(event) {
		var position = dojo.position(this._contentDiv);
		// TODO: Revisit this ... I suspect I could avoid this IE7 issue if I
		// add an appropriate margin-top/margin-bottom somewhere in the CSS
		// rules associated with this class.
		var fudgeFactor = ((dojo.isIE && dojo.isIE < 8) ? 1 : 0);
		var coords = {
			x: position.x + fudgeFactor,
			y: position.y + fudgeFactor,
			w: position.w - fudgeFactor,
			h: position.h - fudgeFactor
		};
		return (
			(event.clientX > coords.x
				&& (event.clientX < (coords.x + coords.w)))
			&&
			(event.clientY > coords.y
				&& (event.clientY < (coords.y + coords.h)))
		);
	},

	_setPreviouslySelectedAttributeDefinitionURIsAttr: function(
			previouslySelectedAttributeDefinitionURIs) {
		this.previouslySelectedAttributeDefinitionURIs =
			previouslySelectedAttributeDefinitionURIs;
		if (this._attributeChooser) {
			this._attributeChooser.attr(
				"previouslySelectedAttributeDefinitionURIs",
				previouslySelectedAttributeDefinitionURIs
			);
		}
	},

	_setScopedArtifactTypeURIsAttr: function(scopedArtifactTypeURIs) {
		this.scopedArtifactTypeURIs = scopedArtifactTypeURIs;
		if (this._attributeChooser) {
			this._attributeChooser.attr(
				"scopedArtifactTypeURIs", scopedArtifactTypeURIs
			);
		}
	},

	_onEnableApplySection: function() {
		if (dojo.isFunction(this.onEnableApplySection)) {
			this.onEnableApplySection(this);
		}
	},

	_onDisableApplySection: function() {
		if (dojo.isFunction(this.onDisableApplySection)) {
			this.onDisableApplySection(this);
		}
	},

	_onContentAreaHeightHasChanged: function() {
		if (dojo.isFunction(this.onContentAreaHeightHasChanged)) {
			this.onContentAreaHeightHasChanged(this);
		}
	},

	_onEditFilter: function() {
		this._contentDiv.style.background = "white"; // FIXME
		this._attributeChooser.hideToolbarButtons();
		this.expand();
		if (dojo.isFunction(this.onEditFilter)) {
			this.onEditFilter(this);
		}
		this._onEnableApplySection();
	}
});


})();
