/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.ValueConditionStatement");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.layout.StackContainer");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.model.attributes.AttributeDefinition");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.query.AttributeSummarySection");
dojo.require("com.ibm.rdm.web.query.conditionSectionFactory");
dojo.require("com.ibm.rdm.web.query.conditionTypes");
// Note by SPC: I'm commenting out this require for ViewServiceClient as it
//     causes non-debug IE to never get past "Loading artifacts..."
//dojo.require("com.ibm.rdm.web.query.ViewServiceClient");

dojo.requireLocalization("com.ibm.rdm.web.query", "ValueConditionStatement");


var AttributeDefinition = com.ibm.rdm.web.model.attributes.AttributeDefinition;
var AttributeSummarySection = com.ibm.rdm.web.query.AttributeSummarySection;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;
var conditionSectionFactory = com.ibm.rdm.web.query.conditionSectionFactory;
var conditionTypes = com.ibm.rdm.web.query.conditionTypes;


dojo.declare(
	"com.ibm.rdm.web.query.ValueConditionStatement",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//		A widget which presents the user with an appropriate
	//		ConditionStatement to fill in a query clause for the currently
	//		selected attribute definition.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "query/templates/ValueConditionStatement.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,
	baselineURI: null,
	attributeSummarySection: null,

	// Please see ConditionChooser for documentation on the following
	// seven fields...
	conditionType: null,
	doAllowAnyAsTheSelectedValue: null,
	isEnvelopedInLinkedArtifactCondition: null,
	restrictedValuesToUseInArtifactFormats: null,
	onInvokeApplyAction: null,
	onEnableApplySection: null,
	onDisableApplySection: null,
	onDropDownButtonChanged: null,
	onContentAreaHeightHasChanged: null,
	includeLinkTypes: null,
	filterCLMLinks: null,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentDiv: null,
	// _mapConditionSectionIdToSummarySectionViewTypeInfo: Object
	//		The propreties are ids of ConditionSection instances and each
	//		property value is of the following format:
	//		|   {
	//		|      viewType: <a value from AttributeSummarySection.VIEW_TYPE>
	//		|      dropDownButtonLabelValuePairs: [ [label1, value1], ... [labelN, valueN] ]
	//		|      dropDownButtonDefaultIndex: <integer>
	//		|   }
	_mapConditionSectionIdToSummarySectionViewTypeInfo: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	// TODO Ask UX if this control should be using a stack container based approach.
	_stackContainer: null,
	_mapAttributeDefinitionUriToConditionSection: null,
	_mapAttributeDefinitionUriToDeferred: null,
	_artifactTypeConditionSection: null,
	_artifactFormatInstancesConditionSection: null,
	_linkTypeConditionSection: null,
	_objectTypesDeferred: null,
	_linkTypesDeferred: null,

	///////////////////////////////////////////////////////////////////////////
	// Public API

	postCreate: function() {
		this._mapAttributeDefinitionUriToConditionSection = {};
		this._mapAttributeDefinitionUriToDeferred = {};
		this._stackContainer = new dijit.layout.StackContainer(
			{},
			this._contentDiv
		);
		this._mapConditionSectionIdToSummarySectionViewTypeInfo = [];
		switch (this.conditionType) {
		case conditionTypes.ArtifactTypeCondition:
			this._initArtifactTypeConditionSection();
			break;
		case conditionTypes.LinkTypeCondition:
			this._initLinkTypeConditionSection();
			break;
		case conditionTypes.ModuleFormatInstancesCondition:
			this._initArtifactFormatInstancesConditionSection();
			break;
		}
	},

	destroy: function() {
		this._stackContainer.destroyRecursive();
		this.inherited(arguments);
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.query", "ValueConditionStatement");
	},

	focus: function() {
		var conditionSection = this._stackContainer.get("selectedChildWidget");
		if (conditionSection) {
			conditionSection.focus();
		}
	},

	clearFilter: function(/*Boolean?*/isFixedAttribute) {
		var conditionSection = this._stackContainer.get("selectedChildWidget");
		// TODO As soon as the conditionSectionFactory covers all types we
		// should remove the if clause and just perform the clause statements.
		if (conditionSection) {
			conditionSection.clearFilter();
			var summarySectionViewTypeInfo =
				this._getConditionSectionSummarySectionViewTypeInfo(conditionSection);
			if (isFixedAttribute) {
				if (summarySectionViewTypeInfo) {
					var self = this;
					this.attributeSummarySection.adjustViewToAccommodateAttributeValueType(
						summarySectionViewTypeInfo.viewType,
						summarySectionViewTypeInfo.dropDownButtonLabelValuePairs,
						summarySectionViewTypeInfo.dropDownButtonDefaultIndex,
						function(operator) {
							self._handleDropDownButtonChanged(conditionSection, operator);
						}
					);
				}
			}
			if (summarySectionViewTypeInfo.dropDownButtonLabelValuePairs.length > 0) {
				var defaultLabelValuePair =
					summarySectionViewTypeInfo.dropDownButtonLabelValuePairs[
						summarySectionViewTypeInfo.dropDownButtonDefaultIndex];
				var operatorToUse = defaultLabelValuePair[1];
				this._handleDropDownButtonChanged(conditionSection, operatorToUse);
			}
		}
	},

	isConditionStatementSpecified: function(/*Boolean?*/skipTimeCheck) {
		var conditionSection = this._stackContainer.get("selectedChildWidget");
		if (conditionSection) {
			return conditionSection.isConditionStatementSpecified(skipTimeCheck);
		} else {
			return false;
		}
	},

	isAttributeConditionSpecifiedButNoLongerValid: function() {
		// TODO implement?
		return false;
	},

	setSelectedAttributeDefinition: function(
			attributeDefinitionUri, /*Callback?*/onAttributeLoaded) {
		var self = this;
		var doCallOnAttributeLoadedWhenLeavingFunction = (onAttributeLoaded ? true : false);
		var conditionSection =
			this._mapAttributeDefinitionUriToConditionSection[attributeDefinitionUri];
		if (!conditionSection) {
			var installConditionSection = function(conditionSection) {
				// TODO As soon as that factory covers all types we should remove
				// the if clause and just perform the clause statements.
				if (conditionSection) {
					self._mapAttributeDefinitionUriToConditionSection[attributeDefinitionUri] =
						conditionSection;
					self._stackContainer.addChild(conditionSection);
					self._stackContainer.startup();
					conditionSection.startup();
					self._selectChildConditionSection(conditionSection);
				}
			};
			var createConditionSection = function(attributeDefinition, attributeDefinitionUri) {
				return conditionSectionFactory.create(
					self.projectURI,
					self.doAllowAnyAsTheSelectedValue,
					self.isEnvelopedInLinkedArtifactCondition,
					self.restrictedValuesToUseInArtifactFormats,
					dojo.hitch(self, self._onConditionChanged),
					self.onInvokeApplyAction,
					self.onEnableApplySection,
					self.onDisableApplySection,
					self.onContentAreaHeightHasChanged,
					self.conditionType,
					null,		// objectTypeInfos
					null,       // linkTypes
					null,		// includeLinkTypes
					attributeDefinition,
					attributeDefinitionUri
				);
			};
			// handle the "Locked By" attribute which is not a real attribute
			if (attributeDefinitionUri === RRMTypes.RDF_LOCKED_BY) {
				// Locked By is not a real attribute, so we don't need to ask the type
				// manager for a definition of it, we can just go ahead and 
				// create a section for it.
				conditionSection =	createConditionSection(null, attributeDefinitionUri);
				installConditionSection(conditionSection);
				this._adjustSummarySectionToAccommodateValueType(
						conditionSection,
						null,	// operatorToUse
						attributeDefinitionUri,
						null,
						false,
						false
					);
			} else {
				var deferred =
					this._mapAttributeDefinitionUriToDeferred[attributeDefinitionUri];
				if (deferred) {
					doCallOnAttributeLoadedWhenLeavingFunction = false;
					deferred.addCallback(function(attributeDefinition) {
						if (onAttributeLoaded) { onAttributeLoaded(attributeDefinition); }
					});
				} else {
					deferred = new dojo.Deferred();
					this._mapAttributeDefinitionUriToDeferred[attributeDefinitionUri] = deferred;
					doCallOnAttributeLoadedWhenLeavingFunction = false;
					var attributeDefinitionDeferred =
						TypesManager.getAttributeDefinition({
							attributeDefinitionURI: attributeDefinitionUri,
							projectURI: this.projectURI,
							projectSnapshotURI: this.baselineURI
						});
					attributeDefinitionDeferred.addCallback(this, function(attributeDefinition) {
						conditionSection =
							createConditionSection(attributeDefinition, attributeDefinitionUri);
						installConditionSection(conditionSection);
						var attributeType = attributeDefinition.getAttributeType();
						var isMultiValued = attributeDefinition.isMultiValued();
						// Make sure that the operator drop down (e.g. "is", "is between",
						// etc.) is showing with the right value.
						this._adjustSummarySectionToAccommodateValueType(
							conditionSection,
							null,	// operatorToUse
							attributeDefinitionUri,
							attributeType.getValueType(),
							attributeType.isEnum(),
							isMultiValued
						);
						if (onAttributeLoaded) { onAttributeLoaded(attributeDefinition); }
						this._enableOrDisableApplyAsAppropriate(conditionSection);
						this.onContentAreaHeightHasChanged();
						deferred.callback(attributeDefinition);
					});
				}
			}
		} else {
			// TODO Should we be calling clearFilter on the currently selected child
			// BEFORE selecting this new child?
			this._selectChildConditionSection(conditionSection);
		}
		if (doCallOnAttributeLoadedWhenLeavingFunction) {
			onAttributeLoaded(conditionSection.attributeDefinition);
		}
	},

	getSummaryLabel: function() {
		var conditionSection = this._stackContainer.get("selectedChildWidget");
		var conditionSummary = conditionSection.getConditionSummary();
		return conditionSummary;
	},

	getConditionInfo: function(/*Widget?*/conditionSection) {
		// summary:
		//		Please see the documentation for a function by this same name in
		//		ConditionChooser for details.
		if (!conditionSection) {
			conditionSection = this._stackContainer.get("selectedChildWidget");
		}
		var conditionTitle = this.attributeSummarySection.getConditionTitle();
		var isArtifactIdentifierConditionSection =
			this._isArtifactIdentifierCondition(conditionSection);
		var operator = (
			isArtifactIdentifierConditionSection ?
			com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.Is :
			this.attributeSummarySection.getSelectedDropDownValue()
		);
		var operatorLabel = (
			isArtifactIdentifierConditionSection ?
			this._messages["is"] :
			this.attributeSummarySection.getSelectedDropDownLabel()
		);
		var conditionInfo =
			conditionSection.getConditionInfo(
				conditionTitle, operator, operatorLabel);
		return conditionInfo;
	},

	setConditionInfo: function(conditionInfo, onConditionLoaded) {
		// summary:
		//		Please see the documentation for a function by this same name in
		//		ConditionChooser for details.
		// onConditionLoaded:
		//		This function will be called back as soon as the
		//		conditionInfo has been fully loaded into the UI.
		var attributeDefinitionUri = conditionInfo.attributeDefinitionURI;
		var attributeValues = conditionInfo.attributeValues;
		var attributeType = conditionInfo.attributeType;
		var isEnum = conditionInfo.isEnum;
		var self = this;
		// Load the attribute definition and create an appropriate *ConditionSection
		// to show the UI form controls pertaining to the value type associated with
		// this attribute defintion.
		var onAttributeLoaded = function(attributeDefinition) {
			var operatorToUse = attributeValues[0];
			var conditionSection =
				self._mapAttributeDefinitionUriToConditionSection[attributeDefinitionUri];
			var isMultiValued = (attributeDefinition) ? attributeDefinition.isMultiValued() : false;
			self._adjustSummarySectionToAccommodateValueType(
				conditionSection, operatorToUse, attributeDefinitionUri, attributeType, isEnum, isMultiValued);
			// Initialize the form controls within this conditionSection
			// based on the content found in conditionInfo.
			var setConditionDeferred = conditionSection.setConditionInfo(conditionInfo);
			if (setConditionDeferred){
				setConditionDeferred.addCallback(self, onConditionLoaded);
			}
			else{
				onConditionLoaded();
			}
		};
		var onObjectTypesLoadedAndArtifactTypeConditionSectionCreated = function() {
			var operatorToUse = attributeValues[0];
			var conditionSection = self._artifactTypeConditionSection;
			self._adjustSummarySectionToAccommodateValueType(conditionSection, operatorToUse);
			// Initialize the form controls within this conditionSection
			// based on the content found in conditionInfo.
			conditionSection.setConditionInfo(conditionInfo);
			onConditionLoaded();
		};
		var onLinkTypesLoadedAndArtifactTypeConditionSectionCreated = function() {
			var operatorToUse = attributeValues[0];
			var conditionSection = self._linkTypeConditionSection;
			self._adjustSummarySectionToAccommodateValueType(conditionSection, operatorToUse);
			// Initialize the form controls within this conditionSection
			// based on the content found in conditionInfo.
			conditionSection.setConditionInfo(conditionInfo);
			onConditionLoaded();
		};
		var handleSetArtifactFormatInstancesConditionSection = dojo.hitch(this, function(){
			var operatorToUse = attributeValues[0];
			var conditionSection = this._artifactFormatInstancesConditionSection;
			this._adjustSummarySectionToAccommodateValueType(conditionSection, operatorToUse);
			// Initialize the form controls within this conditionSection
			// based on the content found in conditionInfo.
			conditionSection.setConditionInfo(conditionInfo).then(onConditionLoaded);
		});
		switch (this.conditionType) {
			case conditionTypes.AttributeCondition:
				this.setSelectedAttributeDefinition(attributeDefinitionUri, onAttributeLoaded);
				break;
			case conditionTypes.ArtifactTypeCondition:
				this._objectTypesDeferred.addCallback(this, function(objectTypeInfos) {
					onObjectTypesLoadedAndArtifactTypeConditionSectionCreated();
					return objectTypeInfos;
				});
				break;
			case conditionTypes.ModuleFormatInstancesCondition:
				handleSetArtifactFormatInstancesConditionSection();
				break;
			case conditionTypes.LinkTypeCondition:
				this._linkTypesDeferred.addCallback(this, function(linkTypes) {
					onLinkTypesLoadedAndArtifactTypeConditionSectionCreated();
					return linkTypes;
				});
				break;
		}
	},

	refresh: function(onRefreshComplete) {
		// summary:
		//		Please see ConditionChooser's documentation for a function by
		//		this same name.
		var conditionSection;
		var self = this;
		var refreshAttributeConditionSection = function(conditionSection) {
			// Account for the fact that the attribute name may have changed
			// and/or an enum (i.e. EnumConditionSection) may have changed.
			var deferred =
				TypesManager.getAttributeDefinition({
					attributeDefinitionURI: conditionSection.attr("attributeDefinitionURI"),
					projectURI: self.projectURI,
					projectSnapshotURI: self.baselineURI
				});
			deferred.addBoth(function(attributeDefinition) {
				conditionSection.attr("attributeDefinition", attributeDefinition);
				conditionSection.refresh(onRefreshComplete);
				self._onConditionChanged(conditionSection);
			});
		};
		var refreshArtifactTypeConditionSection = function(conditionSection) {
			// Account for the fact that the artifact types may have been added
			// or deleted and artifact type names may have changed.
			var deferred = TypesManager.getObjectTypesForProject({
				projectURI: self.projectURI,
				projectSnapshotURI: self.baselineURI
			});
			deferred.addBoth(function(objectTypeInfos) {
				conditionSection.attr("objectTypeInfos", objectTypeInfos);
				conditionSection.refresh(onRefreshComplete);
				self._onConditionChanged(conditionSection);
			});
		};
		var refreshLinkTypeConditionSection = function(conditionSection) {
			// Account for the fact that the link types may have been added
			// or deleted and link type names may have changed.
			var deferred = TypesManager.getLinkTypesForProject({
				projectURI: self.projectURI,
				projectSnapshotURI: self.baselineURI,
				filterCLMLinks: this.filterCLMLinks
			});
			deferred.then(function(linkTypes) {
				conditionSection.set("linkTypes", linkTypes);
				conditionSection.refresh(onRefreshComplete);
				self._onConditionChanged(conditionSection);
			});
		};
		// Ask the currently displayed ConditionSection instance to refresh itself.
		switch (this.conditionType) {
		case conditionTypes.AttributeCondition:
			conditionSection = this._stackContainer.attr("selectedChildWidget");
			refreshAttributeConditionSection(conditionSection);
			break;
		case conditionTypes.ArtifactTypeCondition:
			conditionSection = this._artifactTypeConditionSection;
			refreshArtifactTypeConditionSection(conditionSection);
			break;
		case conditionTypes.LinkTypeCondition:
			conditionSection = this._linkTypeConditionSection;
			refreshLinkTypeConditionSection(conditionSection);
			break;
		}
		if (!conditionSection) {
			onRefreshComplete();
		}
	},

	///////////////////////////////////////////////////////////////////////////
	// Private API

	_initConditionSummaryAsAnyIfAppropriate: function(conditionSection) {
		if ((this.conditionType === conditionTypes.LinkTypeCondition
			|| 
			this.conditionType === conditionTypes.ArtifactTypeCondition)
			&& (
				this.doAllowAnyAsTheSelectedValue
				|| this.isEnvelopedInLinkedArtifactCondition
			   )
			) {
			this.attributeSummarySection.showConditionSummary(this._messages["any"]);
		}
	},

	_initArtifactTypeConditionSection: function() {
		var self = this;
		var installConditionSection = function(conditionSection) {
			self._artifactTypeConditionSection = conditionSection;
			self._stackContainer.addChild(conditionSection);
			self._stackContainer.startup();
			conditionSection.startup();
			self._selectChildConditionSection(conditionSection);
		};
		var createConditionSection = function(objectTypeInfos) {
			return conditionSectionFactory.create(
				self.projectURI,
				self.doAllowAnyAsTheSelectedValue,
				self.isEnvelopedInLinkedArtifactCondition,
				self.restrictedValuesToUseInArtifactFormats,
				dojo.hitch(self, self._onConditionChanged),
				self.onInvokeApplyAction,
				self.onEnableApplySection,
				self.onDisableApplySection,
				self.onContentAreaHeightHasChanged,
				self.conditionType,
				objectTypeInfos
			);
		};
		var deferred = TypesManager.getObjectTypesForProject({
			projectURI: this.projectURI,
			projectSnapshotURI: this.baselineURI
		});
		deferred.addCallback(this, function(objectTypeInfos) {
			var conditionSection = createConditionSection(objectTypeInfos);
			installConditionSection(conditionSection);
			this._adjustSummarySectionToAccommodateValueType(conditionSection);
			this._initConditionSummaryAsAnyIfAppropriate();
			this._enableOrDisableApplyAsAppropriate(conditionSection);
			this.onContentAreaHeightHasChanged();
			return objectTypeInfos;
		});
		this._objectTypesDeferred = deferred;
	},

	_initArtifactFormatInstancesConditionSection: function() {
		var installConditionSection = dojo.hitch(this, function(conditionSection){
			this._artifactFormatInstancesConditionSection = conditionSection;
			this._stackContainer.addChild(conditionSection);
			this._stackContainer.startup();
			conditionSection.startup();
			this._selectChildConditionSection(conditionSection);
		});
		var conditionSection = conditionSectionFactory.create(
			this.projectURI,
			false,
			false,
			null,
			dojo.hitch(this, this._onConditionChanged),
			this.onInvokeApplyAction,
			this.onEnableApplySection,
			this.onDisableApplySection,
			this.onContentAreaHeightHasChanged,
			this.conditionType
		);
		installConditionSection(conditionSection);
		this._adjustSummarySectionToAccommodateValueType(conditionSection);
		this._enableOrDisableApplyAsAppropriate(conditionSection);
		this.onContentAreaHeightHasChanged();
	},

	_initLinkTypeConditionSection: function() {
		var self = this;
		var installConditionSection = function(conditionSection) {
			self._linkTypeConditionSection = conditionSection;
			self._stackContainer.addChild(conditionSection);
			self._stackContainer.startup();
			conditionSection.startup();
			self._selectChildConditionSection(conditionSection);
		};
		var createConditionSection = function(linkTypes) {
			return conditionSectionFactory.create(
				self.projectURI,
				self.doAllowAnyAsTheSelectedValue,
				self.isEnvelopedInLinkedArtifactCondition,
				self.restrictedValuesToUseInArtifactFormats,
				dojo.hitch(self, self._onConditionChanged),
				self.onInvokeApplyAction,
				self.onEnableApplySection,
				self.onDisableApplySection,
				self.onContentAreaHeightHasChanged,
				self.conditionType,
				null,
				linkTypes,
				self.includeLinkTypes
			);
		};
		var deferred = TypesManager.getLinkTypesForProject({
			projectURI: this.projectURI,
			projectSnapshotURI: this.baselineURI, 
			filterCLMLinks: this.filterCLMLinks
		});
		deferred.then(dojo.hitch(this, function(linkTypes) {
			var conditionSection = createConditionSection(linkTypes);
			installConditionSection(conditionSection);
			this._adjustSummarySectionToAccommodateValueType(conditionSection);
			this._initConditionSummaryAsAnyIfAppropriate();
			this._enableOrDisableApplyAsAppropriate(conditionSection);
			this.onContentAreaHeightHasChanged();
			return linkTypes;
		}));
		this._linkTypesDeferred = deferred;
	},

	_selectChildConditionSection: function(conditionSection) {
		this._stackContainer.selectChild(conditionSection);
		var summarySectionViewTypeInfo =
			this._getConditionSectionSummarySectionViewTypeInfo(conditionSection);
		if (summarySectionViewTypeInfo) {
			var self = this;
			this.attributeSummarySection.adjustViewToAccommodateAttributeValueType(
				summarySectionViewTypeInfo.viewType,
				summarySectionViewTypeInfo.dropDownButtonLabelValuePairs,
				summarySectionViewTypeInfo.dropDownButtonDefaultIndex,
				function(operator) {
					self._handleDropDownButtonChanged(conditionSection, operator);
				}
			);
		}
	},

	_storeMappingFromConditionSectionIdToSummarySectionViewTypeInfo: function(
			conditionSection, viewType, dropDownButtonLabelValuePairs, dropDownButtonDefaultIndex) {
		this._mapConditionSectionIdToSummarySectionViewTypeInfo[conditionSection.id] =
			{
				viewType: viewType,
				dropDownButtonLabelValuePairs: dropDownButtonLabelValuePairs,
				dropDownButtonDefaultIndex: dropDownButtonDefaultIndex
			};
	},

	_getConditionSectionSummarySectionViewTypeInfo: function(conditionSection) {
		return this._mapConditionSectionIdToSummarySectionViewTypeInfo[conditionSection.id];
	},

	_enableOrDisableApplyAsAppropriate: function(conditionSection) {
		if (conditionSection.isConditionStatementSpecified()) {
			this.onEnableApplySection();
		} else {
			this.onDisableApplySection();
		}
	},

	_handleDropDownButtonChanged: function(conditionSection, operator) {
		conditionSection.onDropDownButtonChanged(operator);
		this._onConditionChanged(conditionSection);
		this._enableOrDisableApplyAsAppropriate(conditionSection);
	},

	_onConditionChanged: function(conditionSection) {
		// Note: Only if the value type is artifact type, link type, artifact
		// format instances or attribute type Enumeration, User or ArtifactFormats
		// do we want to have the summary section list intermediate values selected
		// by the user.
		var summarySectionViewTypeInfo =
			this._getConditionSectionSummarySectionViewTypeInfo(conditionSection);
		var isConditionClearable = conditionSection.isConditionStatementSpecified();
		if (summarySectionViewTypeInfo
			&& (summarySectionViewTypeInfo.viewType
				=== AttributeSummarySection.VIEW_TYPE.NameWithOperatorsAndIntermediateValues)
			) {
			var conditionSummary = conditionSection.getConditionSummary();
			this.attributeSummarySection.showConditionSummary(conditionSummary);
		}
		this.attributeSummarySection.handleConditionHasChanged(isConditionClearable);
	},

	_isArtifactIdentifierCondition: function(conditionSection) {
		var isArtifactIdentifierCondition = false;
		if (conditionSection.conditionType === conditionTypes.AttributeCondition) {
			var artifactIdAttributeDefinitionUri =
				RRMTypes.getWellKnownAttributeURI(
					this.projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Identifier);
			isArtifactIdentifierCondition =
				artifactIdAttributeDefinitionUri === conditionSection.attributeDefinitionURI;
		}
		return isArtifactIdentifierCondition;
	},

	_adjustSummarySectionToAccommodateValueType: function(
			conditionSection, /*String?*/operatorToUse, /*String?*/attributeDefinitionUri,
			/*String*/wellKnownAttributeType, /*Boolean?*/isEnum, /*Boolean*/isMultiValued) {
		var operatorInfoToUse = (
			this._isArtifactIdentifierCondition(conditionSection) ?
			// Note: The chooser for "Artifact ID" has no drop down.
			null :
			this._getOperatorInfoForAttributeSummaryArea(
				operatorToUse, attributeDefinitionUri, isEnum, wellKnownAttributeType, isMultiValued
			)
		);
		var dropDownButtonLabelValuePairs = (
			operatorInfoToUse ?
			operatorInfoToUse.dropDownButtonLabelValuePairs :
			[]
		);
		var dropDownButtonDefaultIndex = (
			operatorInfoToUse ?
			operatorInfoToUse.dropDownButtonDefaultIndex :
			0
		);
		var summaryViewTypeToUse = (
			(this.conditionType === conditionTypes.ArtifactTypeCondition
			 || this.conditionType === conditionTypes.LinkTypeCondition
			 || this.conditionType === conditionTypes.ModuleFormatInstancesCondition
			 || (this.conditionType === conditionTypes.AttributeCondition
				 && (isEnum
					 || wellKnownAttributeType === RRMTypes.ATTRIBUTE_TYPES.UserType
					 || wellKnownAttributeType === RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType))) ?
			AttributeSummarySection.VIEW_TYPE.NameWithOperatorsAndIntermediateValues :
			AttributeSummarySection.VIEW_TYPE.NameWithOperators
		);
		this._storeMappingFromConditionSectionIdToSummarySectionViewTypeInfo(
			conditionSection,
			summaryViewTypeToUse,
			dropDownButtonLabelValuePairs,
			dropDownButtonDefaultIndex
		);
		var self = this;
		this.attributeSummarySection.adjustViewToAccommodateAttributeValueType(
			summaryViewTypeToUse,
			dropDownButtonLabelValuePairs,
			dropDownButtonDefaultIndex,
			function(operator) {
				self._handleDropDownButtonChanged(conditionSection, operator);
			}
		);
	},

	_getOperatorInfoForAttributeSummaryArea: function(
			/*String?*/operatorToUse, /*String?*/attributeDefinitionUri,
			/*Boolean?*/isEnum, /*String?*/wellKnownAttributeType, /*Boolean*/isMultiValued) {
		var self = this;
		var dropDownButtonLabelValuePairs = [];
		var dropDownButtonDefaultIndex = 0;
		var isWellKnownAttributeDefinition =
			RRMTypes.isWellKnownAttributeDefinition(attributeDefinitionUri, this.projectURI);
		var addExistsAndDoesNotExistValuePairs = function(dropDownButtonLabelValuePairs) {
			dropDownButtonLabelValuePairs.push(
				[self._messages["exists"], com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists]
			);
			dropDownButtonLabelValuePairs.push(
				[self._messages["doesNotExist"], com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist]
			);
		};
		
		if (attributeDefinitionUri === RRMTypes.RDF_LOCKED_BY) {
			// The "Locked by" attribute is not really an attribute, but UX wanted it in
			// the "more attributes" section.  As such it has a specific set of operators
			// as below.
			dropDownButtonLabelValuePairs = [
				[this._messages["isAny"],    com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny],
				[this._messages["currentUser"],   com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS.IsCurrentUser],
				[this._messages["anyUser"],    com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS.IsAnyUser]
			];
		} else {	
			if ((isEnum
				 && isMultiValued
				 && this.conditionType !== conditionTypes.ArtifactTypeCondition)
				|| this.conditionType === conditionTypes.ModuleFormatInstancesCondition
				) {
				dropDownButtonLabelValuePairs = [
					[this._messages["isAny"],    com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny],
					[this._messages["isAll"], com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll],
					[this._messages["isNotAny"], com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny],
					[this._messages["isNotAll"], com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll]
				];
				if (!isWellKnownAttributeDefinition
					&& this.conditionType !== conditionTypes.ModuleFormatInstancesCondition
					) {
					addExistsAndDoesNotExistValuePairs(dropDownButtonLabelValuePairs);
				}		
			}
			else if (isEnum
				|| this.conditionType === conditionTypes.ArtifactTypeCondition
				) {
				dropDownButtonLabelValuePairs = [
					[this._messages["isAny"],    com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny],
					[this._messages["isNotAny"], com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny]
				];
				if (!isWellKnownAttributeDefinition
					&& this.conditionType !== conditionTypes.ArtifactTypeCondition
					) {
					addExistsAndDoesNotExistValuePairs(dropDownButtonLabelValuePairs);
				}
			}
			else {
				switch (wellKnownAttributeType) {
					case RRMTypes.ATTRIBUTE_TYPES.IntegerType:
						dropDownButtonLabelValuePairs = [
							[this._messages["is"],    com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.Is],
							[this._messages["isNot"], com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsNot],
							[this._messages["isGreaterThanOrEqualTo"], com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsGreaterThanOrEqualTo],
							[this._messages["isLessThanOrEqualTo"],    com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsLessThanOrEqualTo]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.FloatType:
						dropDownButtonLabelValuePairs = [
							[this._messages["is"],    com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.Is],
							[this._messages["isNot"], com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsNot],
							[this._messages["isGreaterThanOrEqualTo"], com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsGreaterThanOrEqualTo],
							[this._messages["isLessThanOrEqualTo"],    com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsLessThanOrEqualTo]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.DateType:
						dropDownButtonLabelValuePairs = [
							[this._messages["is"],        com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.Is],
							[this._messages["isNot"],     com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsNot],
							[this._messages["isAfter"],   com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsAfter],
							[this._messages["isBefore"],  com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsBefore],
							[this._messages["isBetween"], com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsBetween]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.StringType:
						dropDownButtonLabelValuePairs = [
							[this._messages["contains"],       com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Contains],
							[this._messages["doesNotContain"], com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.DoesNotContain],
							[this._messages["startsWith"],     com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.StartsWith],
							[this._messages["endsWith"],       com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.EndsWith]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.TimeType:
						dropDownButtonLabelValuePairs = [
							[this._messages["isAfter"],   com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS.IsAfter],
							[this._messages["isBefore"],  com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS.IsBefore],
							[this._messages["isBetween"], com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS.IsBetween]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
						dropDownButtonLabelValuePairs = [
							[this._messages["is"],        com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.Is],
							[this._messages["isAfter"],   com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsAfter],
							[this._messages["isBefore"],  com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBefore],
							[this._messages["isBetween"], com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBetween]
						];
						break;				
					case RRMTypes.ATTRIBUTE_TYPES.DurationType:
						dropDownButtonLabelValuePairs = [
							[this._messages["is"],            com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.Is],
							[this._messages["isGreaterThan"], com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsGreaterThan],
							[this._messages["isLessThan"],    com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsLessThan],
							[this._messages["isBetween"],     com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsBetween]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.BooleanType:
						dropDownButtonLabelValuePairs = [
							[this._messages["is"],    com.ibm.rdm.web.query.ViewServiceClient.BOOLEAN_OPERATORS.Is]
						];
						break;
					case RRMTypes.ATTRIBUTE_TYPES.UserType:
						dropDownButtonLabelValuePairs = [
							[this._messages["isAny"],    com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny],
							[this._messages["isNotAny"], com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny]
						];
						break;
				}
				// ALL attribute types need to have an 'exists' and 'does not
				// exists' operator UNLESS these types are well known (for example
				// it doesn't make sense to use 'exists' or 'does not exist' on the
				// last modified by user, as this field will ALWAYS exist).
				if (!isWellKnownAttributeDefinition) {
					addExistsAndDoesNotExistValuePairs(dropDownButtonLabelValuePairs);
				}
			}
		}
		if (operatorToUse) {
			// Set dropDownButtonDefaultIndex to the appropriate index.
			for (var i = 0; i < dropDownButtonLabelValuePairs.length; i++) {
				var labelValuePair = dropDownButtonLabelValuePairs[i];
				if (labelValuePair[1] === operatorToUse) {
					dropDownButtonDefaultIndex = i;
					break;
				}
			}
		}
		return {
			dropDownButtonLabelValuePairs: dropDownButtonLabelValuePairs,
			dropDownButtonDefaultIndex: dropDownButtonDefaultIndex
		};
	}
});

})();
