/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.ManualScriptEditorSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.util.ConfigurationUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.DatapoolInTestScriptClient");
dojo.require("jazz.app.context");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.RemoteExecutionScriptEditorSection");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.defects.web.ui.internal.ClmLinksHelper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.ManualScriptView");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.model.StepEditModel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.compare.CompareModel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.compare.MergeModel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.RequirementClipboardPlugin");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

(function(){ // module container
    var Constants 						= com.ibm.rqm.planning.web.client.internal.Constants;
    var AbstractArtifactEditorSection 	= com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var DatapoolClient 					= com.ibm.rqm.planning.web.client.internal.construction.DatapoolInTestScriptClient;
    var ServiceResponseHandler 			= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var YesNoDialog 					= com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
    var AlertDialog 					= com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    var TestScriptClient 				= com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
    var REScriptEditorSection 			= com.ibm.rqm.execution.web.ui.internal.view.common.RemoteExecutionScriptEditorSection;
	var RmClmLinksHelper 				= com.ibm.rqm.defects.web.ui.internal.RmClmLinksHelper;
    var getCurrentProjectArea 			= jazz.app.context.get;
    var StepEditModel					= com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.model.StepEditModel;
    var CompareModel					= com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.compare.CompareModel;
    var MergeModel						= com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.compare.MergeModel;
    var CommonUtility                   = com.ibm.asq.common.web.ui.internal.util.Utility;
	var EditorUtil 						= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var Preferences                		= com.ibm.asq.common.web.client.internal.CommonPreferenceClient;
	var ConfigUtil 						= com.ibm.asq.common.web.ui.internal.util.ConfigurationUtil;
	
    var COMMAND_CREATE = "create"; //$NON-NLS$
    var COMMAND_UPDATE = "update"; //$NON-NLS$
    var COMMAND_REMOVE = "remove"; //$NON-NLS$
    var CompareViewClassName = "com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.compare.CompareView";

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.extension.ManualScriptEditorSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection, REScriptEditorSection, RmClmLinksHelper], {
        templateString: "<div dojoAttachPoint=\"_manualStepsSection\"></div>",
        
        // initialize the default script editor class
        stepEditorClass: com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.ManualScriptView,

        constructor: function(args){
        	this._stepUUID = args.stepUUID;
        },
        
        postCreate: function(){
			this.inlineRecording = false;
			this.updatingLive = false;
			this.lastReceivedStepsLen = 0;
        },
        
        initialize: function(){
			var script = this.getParentEditor().getArtifact();
            if(!this.isRecording()){
            	if(this._stepsEditor){
            		this._steps = dojo.clone(this._stepsEditor.getElements());
            	}else{
					this._steps = script && script.elements ? dojo.clone(script.elements) : [];
				}
			}
			var steps = this._steps;
            this._sanitizeScript(steps);
            
        	// This is invoked as a compare merge content, show our custom compare
        	// UI.
        	if(this.inCustomCompareMerge || this.getParentEditor().isMerging()) {
        		// if this is the display host for the custom UI,
        		// we will take over the display content. Otherwise,
        		// we won't need to create any content.
        		if(this.isCompareHost) {
        			this._createCustomCompareMergeUI();
        		}
         		return;
        	}
            
        	this._steps.relatedTCs = this.getParentEditor()._relatedTCs;
         	this.elementTypeMap = null;
			var enableValidateRequirement = null;
			if(this.isRecording() && this._stepsEditor){//Save a call here when refresh happens
				this.elementTypeMap = this._stepsEditor.getElementTypeMap();
				enableValidateRequirement = this._stepsEditor.getValidatesPreferences();
			}
			this.unloadContent(this._manualStepsSection);
			var editorClass = this.stepEditorClass;
            this._stepsEditor = new editorClass({
                // don't put the focus on first step if this is a new editor.
                doInitFocusOnFirstStep: !this.getParentEditor().isNew(),
                stepUUID: this._stepUUID,
				exeScriptTypes: this.exeScriptTypes,
				_compositeDTO: this._compositeDTO,
                save: dojo.hitch(this, function(){
                    var editor = this.getParentEditor();
                    if (dojo.isFunction(editor.onSave)) {
                        editor.onSave();
                    }	
                }),
				onInlineRecord: this.isRecordable() && this.isRecordPreferenceSetByAdmin() ? dojo.hitch(this, "_onInlineRecording") : null
            });
            this._manualStepsSection.appendChild(this._stepsEditor.domNode);
            if (steps.length == 0 || this.isRecording() || !script) {
				var self = this;
				if(this.isRecording() && this._stepsEditor && !this.useV2ScriptEditor){
					//This must happen before the initialize is called for the function
					this._stepsEditor.setElementTypeMap(this.elementTypeMap);
					this._stepsEditor.setValidatesPreferences(enableValidateRequirement, false/*dont fetch the preference this time*/);
				}
				setTimeout(function(){
					if(!self.isRecording())
						self._steps.relatedTCs = self.getParentEditor()._relatedTCs;
			        self._initializeStepEditor();
				}, 200);
            }
            else {
                this._initializeExecutionScript();
            }
            //this.createBrowseSection(null, null, document.createElement('div'));
        },
       
		_sanitizeScript: function(steps) {
			var stepLen = steps.length;
			for (var i = 0; i < stepLen; i++) {//Workitem 27332
                var step = steps[i];
                if(step.content)
                    step.content = CommonUtility.removeScriptAndInlinifyStyle(step.content);
                if(step.expectedResult)                    
                    step.expectedResult = CommonUtility.removeScriptAndInlinifyStyle(step.expectedResult);
            }	
			return steps;
		},
		
        // @Override from AbstractArtifactEditorSection
		//
		// Provide custom compare merge display.
		//
        supportCustomCompareMerge: function() {
        	return true;
        },
        
        // @Override from AbstractArtifactEditorSection
		//
		// Receives the compare merge context to product the custom display. The context
        // is the source and target section of the artifact.
		//
        setCustomCompareMergeContext: function (context) {
        	this.inCustomCompareMerge = true;
        	this.isCompareHost = context.isTarget;
        	this.initialized = false;
        	this.customMergeContext = context;

    		if(this._stepsEditor) {
    			this._stepsEditor.destroyRecursive();
    			this._stepsEditor = null;
    		}    
        	this.unloadContent(this._manualStepsSection);
        },
        
        _createCustomCompareMergeUI: function() {  
        	dojo.addClass(this._manualStepsSection, "section-loading-msg");
        	this._manualStepsSection.innerHTML = Constants.LOADING;
        	
        	// Compare and load the annotated source and target contents into our compare panes.
			jazz.core.loader.load_async(CompareViewClassName, dojo.hitch(this, function(){
	        	var context = this.customMergeContext;
	        	var sourceContent = context.sourceSection;
	        	var CompareViewCls = dojo.getObject(CompareViewClassName);
				var srcScript = sourceContent.getParentEditor().getArtifact();
				var srcSteps = srcScript && srcScript.elements ? dojo.clone(srcScript.elements) : [];
				srcSteps = this._sanitizeScript(srcSteps);
				var modelArgs = {
					sourceScript: srcScript,
					targetScript: this.getParentEditor().getArtifact(),
		        	source: srcSteps,
		        	target: this._steps
		        };
				var isMerging = this.isEditable();
	        	var model = isMerging ? new MergeModel(modelArgs) : new CompareModel(modelArgs);
	        	model.initialize();
	        	
	        	if(isMerging) {
	                // let us receive model changed events.
	                this.stepModelChanged = dojo.hitch(this, function() {
	                	var artifactSection = this.getSection();
	                	var parentEditor = this.getParentEditor();
	                	
	                	// the model changed, check if the merged state changed
	                	// for the model. If so, update the section's merged completed
	                	// status.
	                	artifactSection.merged = this.customMergeContext.model.isMerged();
	                	if(parentEditor.updateTableOfContents) {
	                		parentEditor.updateTableOfContents();
	                	}
	                	parentEditor.setDirty(true);
	                });
	                model.addListener(this);
	        	}
	        	var viewer = new CompareViewCls({
                    getContentBottomOffset: this.getContentBottomOffset,
                    maximizeViewProvider: this.maximizeViewProvider,
	        		model: model
	        	});
	        	if(isMerging) {
	        		this._stepsEditor = viewer;
	        		this.customMergeContext.model = model;
	        	}
	        	dojo.removeClass(this._manualStepsSection, "section-loading-msg");
	        	dojo.empty(this._manualStepsSection);	        	
	        	this._manualStepsSection.appendChild(viewer.domNode);
	        	viewer.initialize();
			}));        	
        },

        // @Override from AbstractArtifactEditorSection
        //
        // To add accept from source
        // 
        defaultAcceptSection: function() {
        	var artifactSection = this.getSection();
        	var changeType = artifactSection ? artifactSection.changeType : null;

        	if (changeType == Constants.SECTION_CHANGE_TYPE_MODIFIED) {     
        		this.customMergeContext.model.acceptAllFromSource();
        		artifactSection.merged = true;
        	}        	         
        },
        
        // @Override from AbstractArtifactEditorSection
        //
        // 
        markSectionAsMerged: function() {
        	var model = this.customMergeContext.model;
        	
        	// we have not completed merge yet, this is to just take 
        	// the target changes.
        	if(!model.isMerged()) {
        		model.acceptAllFromTarget();
        	}
        	this.getSection().merged = true;
        },
        
        _initializeStepEditor: function() {
            var isV2 = this.useV2ScriptEditor;
            var stepEditorArgs = {
                parentController: this.parentController,
                steps: this._steps,
                compositeDTO: this._compositeDTO,
                parentCallBack: this.parentEditor,
                initCompleted: this.initCompleted,
                onDatapoolChangeClick: dojo.hitch(this.getParentEditor(), "onDatapoolChangeClick")
            };
            if(isV2) {
                var parentEditor = this.getParentEditor();

                // set up print view mode.
                if (dojo.isFunction(parentEditor.isPrintView)) {
                    this.isPrintView = parentEditor.isPrintView();
                }       
                // don't provide any maximize, or content layout related methods i
                // we're in print view. The page will be layout out in single flow
                if(!this.isPrintView) {
                    stepEditorArgs.getContentBottomOffset = this.getContentBottomOffset;
                    stepEditorArgs.maximizeViewProvider = this.maximizeViewProvider;
                    stepEditorArgs.restoreMaximizePreference = this.restoreMaximizePreference;
                }
            	
                // provide the proper model object for the viewer
            	var script = parentEditor.getArtifact();
                var model = stepEditorArgs.model = new StepEditModel({
                    executionElementTypes: (this._compositeDTO) ? this._compositeDTO.executionElementTypes : null,
                    script: script,
                    steps: this._steps,
                    // set print mode to remove all editing capabilities, including read-only copy. For read only config,
                    // use the this mode to remove all editing as well.
                    isPrintMode: this.isPrintView || this.isReadOnlyConfiguration(),
                    onInlineRecord: this.isRecordable() && this.isRecordPreferenceSetByAdmin() ? dojo.hitch(this, "_onInlineRecording") : null,
                    testDataProvider: dojo.hitch(this.parentEditor, "getSelectedDataPool"),
                    testDataSetter: dojo.hitch(this.parentEditor, "_setDatapool"),
                    // provide implementation of the isEditable method.
                    isEditable: dojo.hitch(this, function() {
                        // If we're in print view mode, the editor needs to be
                        // non-editable. Check for artifact state only if we're not
                        // in print view mode.
                        if(!this.isPrintView) {
                            var parentEditor = this.getParentEditor();
                            if (parentEditor && dojo.isFunction(parentEditor.isReadOnly)) {
                                var isReadOnly = this.parentEditor.isReadOnly();
                                if (isReadOnly) {
                                    return false;
                                }
                                if (dojo.isFunction(parentEditor.isSaveableState)) {
                                    return parentEditor.isSaveableState();
                                }
                            }
                        }
                        return false; //default to non editable mode
                    })
                });
                // if this is merge mode and this is the source container
                // make it non-editable, interactive, i.e., no selection,
                // no step actions.
                if(this.isMerging()) {
                	model.nonInteractiveMode = true;
           			this._stepsEditor.enableActions(false);
                }
                // let us receive model changed events.
                this.stepModelChanged = dojo.hitch(this, function() {
                    var parentEditor = this.getParentEditor();                    
                    if (model.isEditable()) {
	                    // report edited event back to parent editor. But only call it
	                    // if it's not dirty yet to avoid excess processing.
	                    if(parentEditor && !parentEditor.isDirty()) {
	                        parentEditor.setDirty(true);
	                    }
                    }
                    
                });
                model.addListener(this);
            }    
            this._stepsEditor.initialize(stepEditorArgs);
            if(isV2 && this.isRecording()) {
            	this._stepsEditor.setElementTypeMap(this.elementTypeMap);
            }
        },
        
        // get notification on going to print view.
        onGoingToPrintView: function() {
            if(this.useV2ScriptEditor) {
                this._stepsEditor.onBeingHidden();
            }
        },
        
        _initializeExecutionScript: function(){
            var self = this;  
        	// don't call initialize step editor immediately. The section dom at this moment
        	// has not been added yet, we're still in the initialize path. The step editor
        	// needs to be inserted to the dom tree to properly calculate internal dimensions.            
            setTimeout(function() {
            	self._initializeStepEditor();
            });
        },
		
        // @obsolete - no longer called.
		_rebuildLinks:function(links){
			//rebuild steps to display links summary
			var self = this;
			self._steps.relatedTCs = self.getParentEditor()._relatedTCs;
			for(var i = 0; i < self._steps.length; i++){
				var step = self._steps[i];
				var stepOldReqLinksLength = step.oldReqLinks ? step.oldReqLinks.length : 0;
		    	for(var j = 0; j < stepOldReqLinksLength; j++){
					var oldLink = step.oldReqLinks[j];
					var count = 0;
			   		dojo.forEach(links, function(link){
						if(oldLink.uri == link.uri){
							self._steps[i].oldReqLinks[j].summary = link.summary;
							self._steps[i].reqLinks[j].summary = link.summary;
							count ++;
						}
					});
					if(count == 0){
						self._steps[i].oldReqLinks[j].summary = oldLink.uri;
						self._steps[i].reqLinks[j].summary = oldLink.uri;
					}
			   	}
			}
			this._initializeStepEditor();
		},
        
        //@Abstract: Override to contribute attributes to the parent's save
        getAttributes: function(){
        	// If we have not initialized the step editor, return null to 
        	// indicate no changes to the steps.
        	if(!this._stepsEditor) {
        		return null;
        	}        	
        	
            //collect commands
            var commands = [];
            
            //get current steps from step editor
            var script = this.getParentEditor().getArtifact();
            var isCopy = this.getParentEditor().isCopy();
            var oldElements = script && script.elements ? script.elements : [];
            var newElements = this.currentElements ? this.currentElements : [];
            
            //for each element, create the appropriate command to update script
            var numNew = newElements.length;
            for (var i = 0; i < numNew; i++) {
                var element = newElements[i];
                if (!isCopy && element.itemId != null) {
                    //existing element; 'update' command
					var existing = this._findIn(oldElements, element);
                    var command = {
                        cmd: COMMAND_UPDATE,
                        elementObj: this._toCommandObj(element, existing)
                    };
                    commands.push(command);
                    
                } else {
                    //new element; 'create' command
                    var command = {
                        cmd: COMMAND_CREATE,
                        elementObj: this._toCommandObj(element)
                    };
                    commands.push(command);
                }
            }
            
            //handle any deletes
			if (!isCopy) {
				var numOld = oldElements.length;
				for (var i = 0; i < numOld; i++) {
					//if there's no entry for an existing element in the list of current 
					//elements, the existing element has been removed
					var original = oldElements[i];
					var updatedElement = this._findIn(newElements, original);
					if (original.itemId && updatedElement == null) {
						var command = {
							cmd: COMMAND_REMOVE,
							elementObj: {
								num: (i + 1),
								itemId: original.itemId
							}
						};
						commands.push(command);
					}
				}
			};
			var returnVal = {elements: commands};//this.currentElements
            var variables = this._stepsEditor.getExecVariables();
            if(variables){
            	if(variables.variableMap){
            		returnVal.variableJson = dojo.toJson(variables.variableMap);
            	}
	            if(variables.projectVariableMap) {
		            returnVal.addedProjectVariableJson = dojo.toJson(variables.projectVariableMap.addedVariableMap);
	            	returnVal.removedProjectVariableJson = dojo.toJson(variables.projectVariableMap.removedVariableMap);
	            	returnVal.modifiededProjectVariableJson = dojo.toJson(variables.projectVariableMap.modifiededVariableMap);
            	}
            }
			returnVal.includeBuiltinVars = this._stepsEditor.isIncludeBuiltinVars();
			if(variables && variables.parentScriptItemId){
				returnVal.parentScriptItemId = variables.parentScriptItemId;
			}
			if(variables && variables.parentScriptExecutionVariableItemIds){
				returnVal.parentScriptExecutionVariableItemIds = variables.parentScriptExecutionVariableItemIds;
			}        	
            return returnVal;
        },
        
        _toCommandObj: function(element, existing){
            var commandObj = {
                num: element.num,
                typeName: element.elementType.name     
            };
            if (element.itemId) {
                commandObj.itemId = element.itemId;
            }
			if (element.summary != null) {
				if (!existing || (unescape(existing.summary) != unescape(element.summary))) {
					commandObj.summary = element.summary;
				}
			}
            if (element.content != null) {
                if (!existing || (unescape(existing.content) != unescape(element.content))) {
                    commandObj.content = element.content;
                }
            }	
            if (element.expectedResult != null) {
                if (!existing || (unescape(existing.expectedResult) != unescape(element.expectedResult))) {
                    commandObj.expectedResult = element.expectedResult;
                }
            }
            
            if (element.reqLinks && element.reqLinks.length > 0) {
                if (existing) {
                    if (!existing.reqLinks || (existing.reqLinks.length != element.reqLinks.length)) {
                        commandObj.reqLinks = element.reqLinks;
                    }
                }
                else {
                    commandObj.reqLinks = element.reqLinks;
                }
            }
			if(element.deleteReqLinks){
				commandObj.deleteReqLinks = true;
			}
			if (element.oldReqLinks && element.oldReqLinks.length > 0) {
				commandObj.oldReqLinks = element.oldReqLinks;
			}
            
            var comment = element.comment;
            commandObj.comment = comment ? comment : "";

			var compare = element.compare;
			commandObj.compare = compare ? compare : "";

			if(element.keywordItemId) {
				commandObj.keywordItemId= element.keywordItemId;
			}
			var attachments = element.attachments;
			if(attachments && attachments.length > 0){
				commandObj.attachments = [];
				for(var i=0;i< attachments.length;i++) {
					commandObj.attachments.push({itemId:attachments[i].itemId});
				}
			}					
            if (element.properties) {
                commandObj.properties = element.properties;
            }
			return commandObj;
        },
        
        validate: function(){
        	if(!this._stepsEditor) {
        		return true;
        	}
            this.currentElements = [];
            this._stepsEditor.setGlobalSaveInProgress();
            if (!this._stepsEditor.validateElements()) {
                return false;
            }
            this.currentElements = this._stepsEditor.getElements();
            return true;
        },
        
        isEmpty: function(){
            return this.currentElements ? this.currentElements.length == 0 : false;
        },
        
        //@Overriden from AbstractArtifactEditorSection
        getActions: function(){
            return [];
        },
        
        //@Overriden from RemoteExecutionScriptEditorSection
        onSave: function(responseHandler, scriptObj, lastSaved){
            if (this.validate()) {
                if (this.isEmpty()) {
                    var self = this;
                    var args = {
                        primaryTitle: Constants.getMessage("labelConfirmation"),
                        message: Constants.getMessage("msgNoStepStillSave"),
                        yesCallback: function(){
                            self._saveManualScript(responseHandler, scriptObj);
                        },
                        noCallback: function(){
                            self.parentEditor._stopSaving();
                            if (self.parentEditor.isDirty()) {
                                self.parentEditor.setDirty(true);
                            }
                        },
                        onBeforeClose: function(){
                            self.parentEditor._stopSaving();
                            if (self.parentEditor.isDirty()) {
                                self.parentEditor.setDirty(true);
                            }
                        }
                    };
                    var dialog = new YesNoDialog(args);
                    dialog.open();
                }
                else{
                    this._saveManualScript(responseHandler, scriptObj);
                }
            }
            else {
                if(this.useV2ScriptEditor) {
                    var stepNum = this._stepsEditor.getInvalidSteps();
                    var msgKey = this._stepsEditor.invalidSaveMessageKey ? this._stepsEditor.invalidSaveMessageKey : "msgStepContentRequired";
                    var msg = Constants.getMessage(msgKey, stepNum);
                    this.parentEditor.setHeaderStatus(msg, Constants.HEADER_STATUS_ERROR);                    
                }
                // set the proper flags to indicate the save didn't go thru
                // and also, there was an error in the validation. 
                this.parentEditor._stopSaving();
                this.parentEditor.setErrorInSave(true);
            }
        },
        
        _saveManualScript: function(responseHandler, scriptObj){
            var attributes = this.getAttributes();
            if (attributes != false) {
                dojo.mixin(scriptObj, attributes);
				TestScriptClient.saveTestScript(responseHandler, scriptObj);
				if(dojo.isFunction(this.recordedScriptSaveFollowUpTasks))
					this.recordedScriptSaveFollowUpTasks();
				this.scriptObj = scriptObj;
            }
            else {
                this.parentEditor._stopSaving();
            }
        },
		
		/**
		 * need {stepItemId, resourceUri, resourceName, newAddedRequirements[]} for newAddedReqtJson
		 * need  {stepItemId, resourceName, resourceUri} for linkClearedTestSteps
		 * 
		 * @param {Object} results
		 * @param {dojo.deferred} deferred optional, if a deferred object is passed in, we'll call its resolve method when we're done.
		 */
		saveOthers:function(results, /*optional*/deferred){	
			//only call the association service if there are association changes
			//TODO the filter logic is not correct, add it back later.		
			var iLen = this.scriptObj.elements.length;
			var newAddedReqtObj = [];
			var linkClearedTestStepsObj = [];
			
			var removedSteps = this._stepsEditor.getRemovedSteps();
			
			//removedSteps records each "removing" action user was taking since last save, so may 
			//contain a series of steps. Only the 'oldReqLinks' is needed later, so put together 
			//all 'oldReqLinks' at the same position into one entry
			var removedStepLinks = {};
			for (var j = 0; j < removedSteps.length; j++){
				var stepNum = removedSteps[j].stepNum;
				if(!removedStepLinks[stepNum]){
					removedStepLinks[stepNum] = [];
				}
				if(removedSteps[j].step.oldReqLinks){
					dojo.forEach(removedSteps[j].step.oldReqLinks, function(req){
						removedStepLinks[stepNum].push(req);
					});
				}
			}
			
			for (var i = 0; i < iLen; i++) {
				var stepObj = {};
				var element = this.scriptObj.elements[i].elementObj;	
				stepObj.stepItemId = element.itemId ? element.itemId : results.elements[i].itemId;
				stepObj.resourceUri = this._getStepResourceUri(stepObj.stepItemId);
				stepObj.resourceName = this.scriptObj.id + ": " + this.scriptObj.name + " [" + (element.num) + "]";
				
				var allReqLinks = element.reqLinks ? element.reqLinks : [];
				var oldReqLinks = element.oldReqLinks ? element.oldReqLinks : [];
				
				var cmd = this.scriptObj.elements[i].cmd;
				
				if(cmd == "remove"){	//remove whole step
					// also remove requirement links
					if(removedStepLinks[element.num-1]){
						var removedLinks = removedStepLinks[element.num-1];
						if(removedLinks && removedLinks.length > 0){
							stepObj.removedRequirements = removedLinks;
							linkClearedTestStepsObj.push(stepObj);
						}
					}
				}
				else if(element.deleteReqLinks){	// remove req links
					if(oldReqLinks.length > 0){
						stepObj.removedRequirements = oldReqLinks;
						linkClearedTestStepsObj.push(stepObj);
					}
				} else{	// new added req links
					var addedReqs = [];
					for(var k = 0; k < allReqLinks.length; k ++){
						var exist = false;
						for (var j = 0; j < oldReqLinks.length; j++){
							if(allReqLinks[k].uri == oldReqLinks[j].uri){
								exist = true;
								break;
							}
						}
						if(!exist){	//new added one
							addedReqs.push(allReqLinks[k]);
						}
					}
					if(addedReqs && addedReqs.length > 0){
						stepObj.newAddedRequirements = addedReqs;
						newAddedReqtObj.push(stepObj);
					}
				}
			}
			
			if((!newAddedReqtObj || newAddedReqtObj.length == 0) && (!linkClearedTestStepsObj || linkClearedTestStepsObj.length == 0)){
				// console.log("no links added or removed");
				// no links to process, if there's a deferred, we can complete it.
				if(deferred) {
					deferred.resolve();
				}				
			}else{
				EditorUtil.getEnableSyncReqOption(dojo.hitch(this, function(pref){
					this._doSyncRequirement(pref, newAddedReqtObj, linkClearedTestStepsObj, results, deferred);
				}), dojo.hitch(this, function(e){
					console.warn(e); 
					this._doSyncRequirement(false, newAddedReqtObj, linkClearedTestStepsObj, results, deferred);
				}));
			}		
		},
			
			_doSyncRequirement: function(pref, newAddedReqtObj, linkClearedTestStepsObj, results, /*optional*/deferred){
				// for PLE endabled, check whether we've a GC context before doing
				// the sync. Otherwise, the sync would fail. We disallow the user
				// to do any link related tasks without an GC in PLE. Normally, that
				// check is in the OSLC provider actions. But we're make direct link
				// service call here. Need to add the check here too.
				if(ConfigUtil.isPleEnabled()) {
	                var gc = ConfigUtil.getGlobalConfigValue();
	                var lc = ConfigUtil.getLocalConfigValue();
	                
	                // if the gc and the lc are the same, we really don't have a gc context
	                // need to post an message indicating that we can't sync.
	                //
	                // Note: in the normal add/remove links case, we would stop the user from
	                // adding/creating a new link. So, this is really for merging links.
	                if (!gc || gc.indexOf(lc) != -1) {
                        var message = Constants.getMessage("msgNoGcToSyncReqLinkError") + "<br><br>" +
                        Constants.getMessage("msgNoGcToSyncReqLinkErrorExp") + "<br><br>" +
                        Constants.getMessage("msgNoGcToSyncReqLinkErrorUserAction");
                        
                        var alertDialog = new AlertDialog({
                            message: message,
                            primaryTitle: Constants.getMessage("labelInformation"),
                            okCallback: dojo.hitch(this, function() {
                            	if(deferred) {
                            		deferred.resolve();
                            	}
                            })
                        });
                        alertDialog.open();
	                	return;
	                }
				}
				//sync is on
				if(pref){
					if(linkClearedTestStepsObj && linkClearedTestStepsObj.length > 0){	//remove requirements from script step
						jazz.core.loader.batch_load_async(["com.ibm.rqm.defects.web.client.internal.ResourceLinkService2"],dojo.hitch(this,function(){
							com.ibm.rqm.defects.web.client.internal.ResourceLinkService2.getRequirementsLinkToTestcaseOnly(getCurrentProjectArea().itemId, this.scriptObj.name, results.versionableItemId,
									(newAddedReqtObj.length > 0) ? dojo.toJson(newAddedReqtObj) : "", (linkClearedTestStepsObj.length > 0) ? dojo.toJson(linkClearedTestStepsObj) : "", dojo.hitch(this,function(result){
								var tcWithReqs = result.testcaseWithRequirements;
								var requirementsRemoveFromTestCase = [];
								if(tcWithReqs){
									for(var i = 0; i < tcWithReqs.length; i++){
										var tcReq = {};
										tcReq.testcaseItemId = tcWithReqs[i].itemId;
										tcReq.requirements = tcWithReqs[i].calmRequirements;
										requirementsRemoveFromTestCase.push(tcReq);
									}
								}
								
								var	reqsRemoveFromTC = (requirementsRemoveFromTestCase.length >0) ? dojo.toJson(requirementsRemoveFromTestCase) : "";
								
								if(tcWithReqs){	//propmt confirmation dialog only if there are requirements linking to no child script but only to parent test case
									var preference = Preferences.getTestScriptPreference();
									//check whether prompt confirmation according to test script user preference
									if(preference.syncOption == "ask"){
										var args = {
											'primaryTitle': Constants.getMessage("labelConfirmation"),
											'message': this._getConfirmationMessage(tcWithReqs, result.calmRequirementsLinkingToTestCase),
											'width': "450px",
											'yesCallback':
								                dojo.hitch(this,function() {
								                    this._doUpdateStepToRequirementLinks(pref, results, newAddedReqtObj, linkClearedTestStepsObj, reqsRemoveFromTC, true, deferred);
								                }),
								            'yesCallbackData': this,
								            'noCallback':
												dojo.hitch(this,function() {
								                    this._doUpdateStepToRequirementLinks(pref, results, newAddedReqtObj, linkClearedTestStepsObj, reqsRemoveFromTC, false, deferred);
								                }),
								            'noCallbackData': this,
										};
										this.confirmationDialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
										this.confirmationDialog.open();
										
									} else{
										var syncRequirement = (preference.syncOption == "yes") ? true : false;
										this._doUpdateStepToRequirementLinks(pref, results, newAddedReqtObj, linkClearedTestStepsObj, reqsRemoveFromTC, syncRequirement, deferred);
									}
									
								} else{
									this._doUpdateStepToRequirementLinks(pref, results, newAddedReqtObj, linkClearedTestStepsObj, reqsRemoveFromTC, false, deferred);
								}
							}),
							dojo.hitch(this,function(result){
								if(deferred) {
									deferred.resolve();
								}
							}));
						}));
					} else{	//add requirement to script step
						this._doUpdateStepToRequirementLinks(pref, results, newAddedReqtObj, linkClearedTestStepsObj, "", false, deferred);
					}
				} else{	//sync is off
					this._doUpdateStepToRequirementLinks(false, results, newAddedReqtObj, linkClearedTestStepsObj, "", false, deferred);
				}
				
			},
			
			_doUpdateStepToRequirementLinks:function(syncRequirementToParentTestCase, results, newAddedReqtObj, linkClearedTestStepsObj, requirementsRemoveFromTestCase, removeTestcaseReq, /*optioanl*/deferred){
				jazz.core.loader.batch_load_async(["com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil", "com.ibm.rqm.defects.web.ui.view.common.ProgressDialog", 
										   "com.ibm.rqm.defects.web.client.internal.ResourceLinkService2"], 
					dojo.hitch(this, function(){
						com.ibm.rqm.defects.web.ui.view.common.ProgressDialog
							.doRunWithProgress(com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil.getMessage("msgUpdatingLinks"), 
							dojo.hitch(this, function(monitor){
								monitor.beginTask(com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil.getMessage("msgUpdateLocalLinks"));
								
								var uriToSummaryMap = {};
								var addedLinksLength = 0;
								for (var i = 0; i < newAddedReqtObj.length; i++){
									var newAddedRequirements = newAddedReqtObj[i].newAddedRequirements;
									if(newAddedRequirements){
										addedLinksLength += newAddedRequirements.length;
										for (var k = 0; k < newAddedRequirements.length; k++){
											uriToSummaryMap[newAddedRequirements[k].uri] = newAddedRequirements[k].summary;
										}
									}
								}
								var removedLinksLength = 0;
								for (var i = 0; i < linkClearedTestStepsObj.length; i++){
									var removedRequirements = linkClearedTestStepsObj[i].removedRequirements;
									if(removedRequirements){
										removedLinksLength += removedRequirements.length;
										for (var k = 0; k < removedRequirements.length; k++){
											uriToSummaryMap[removedRequirements[k].uri] = removedRequirements[k].summary;
										}
									}
								}
				                
				                var	newAddedReqtJson = (newAddedReqtObj.length >0) ? dojo.toJson(newAddedReqtObj) : "";
				    			var	linkClearedTestStepsJson = (linkClearedTestStepsObj.length >0) ? dojo.toJson(linkClearedTestStepsObj) : "";
				                
            					com.ibm.rqm.defects.web.client.internal.ResourceLinkService2.updateStepToRequirementLinks(getCurrentProjectArea().itemId, syncRequirementToParentTestCase, this.scriptObj.name, results.versionableItemId,
									newAddedReqtJson, linkClearedTestStepsJson, requirementsRemoveFromTestCase ? requirementsRemoveFromTestCase : "", removeTestcaseReq ? removeTestcaseReq : false, uriToSummaryMap, addedLinksLength, removedLinksLength, 
									dojo.hitch(this, function(result, refresh){
										monitor.done();
										// we've a deferred object. We just need to call it to
										// notify completion and we're done. 
										//
										// NOTE: the deferred is used in post merge save to process
										// requirement merge changes. Any normal script editor processing
										// should be place below this block.
										if(deferred) {
											deferred.resolve();
											return;
										}
										var requirementPlugin = new com.ibm.rqm.planning.web.ui.internal.view.construction.manualscripteditor.RequirementClipboardPlugin({
											stepEditor: this._stepsEditor
										});
										if(requirementPlugin.isOpen()){
											requirementPlugin.refresh();
										}
										if(refresh && refresh == true){
											this.getParentEditor()._reloadEditor();
										} else{
											this.getParentEditor().refreshValidateLinksInfo();
										}
								}),
								dojo.hitch(this, function(result){
										monitor.done();
										if(deferred) {
											deferred.resolve();
										}
								}));						
						}));
				}));
		},
		
		_getConfirmationMessage: function(tcWithRequirements, reqsLinkingToTC){
			var len = tcWithRequirements.length;
			var message = "";
			
			var reqListStr = "";
			reqListStr += "<ul>";
			if(reqsLinkingToTC){
				for(var i = 0; i < reqsLinkingToTC.length; i++){
					var reqHtmlStr =  "<li>" + reqsLinkingToTC[i] + "</li>";
					reqListStr += reqHtmlStr;
				}
				reqListStr += "</ul>";
			}
				
			if(len == 1){
				message = Constants.getMessage("msgSyncRequirementWithSingleTestCase");
				message += reqListStr;
				message += Constants.getMessage("msgSyncRequirementWithSingleTestCaseQuestion", tcWithRequirements[0].name);
			} else if(len > 1){
				message = Constants.getMessage("msgSyncRequirementWithMultipleTestCases");
				message += reqListStr;
				message += Constants.getMessage("msgSyncRequirementWithMultipleTestCaseQuestion");
			}
			return message;
		},
		
		//filter out which have been associated
		_addNewReqLinkObj:function(scriptObj){
			var newAddedRmLinks = [];
            for (var i = 0; i < scriptObj.elements.length; i++) {
                var element = scriptObj.elements[i].elementObj;
				//old row obj
				var oldReqLinks = element.oldReqLinks;
				
				if(!oldReqLinks){
					//if there are no req linking to the step before, all linked are new.
					scriptObj.elements[i].elementObj.reqLinks = element.reqLinks;					
					continue;
				}
				
				//This is so wrong!!
				if (element.reqLinks && oldReqLinks.length == element.reqLinks.length) {
					scriptObj.elements[i].elementObj.reqLinks = [];
					break;
				}else {
					dojo.forEach(element.reqLinks, function(req){
						for (var t = 0; t < oldReqLinks.length; t++) {
							if (req.uri == oldReqLinks[t].uri) {
								break;
							}
						}
						if (t == oldReqLinks.length) {
							newAddedRmLinks.push(req);
						}
					});
					scriptObj.elements[i].elementObj.reqLinks = newAddedRmLinks;
				}
            }
            return scriptObj;
		},
		
        getResourceLabel: function(){
            return Constants.getMessage("labelTestScript");
        },
        
        getResourcePrefix: function(){
            return '/resources/com.ibm.rqm.planning.VersionedExecutionScript/';
        },
        
        /**
         * Provides post merged save processing required by the manual test script type. 
		 * We need to update any changes in the requirement links in test steps.
         */
        postMergeSavedProcess: function(savedItem, deferred) {
        	this.saveFinished(savedItem, deferred);
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem, /*optional*/deferred){        	
			if(!savedItem)
			{
				return;
			}
        	this._script = savedItem;
			//save others
			this.saveOthers(savedItem, deferred);
			this._stepsEditor.clearRemovedSteps();	//the previously removed step series has been handled in the save action, so clear it
			
            //            dojo.style(this._stepsEditor._stepsContainer, "height", "auto");
            this._steps = savedItem && savedItem.elements ? dojo.clone(savedItem.elements) : [];
			var currentElements = this._stepsEditor.getElements();
			for(var i = 0; i < currentElements.length; i++){
				var step = currentElements[i];
				if (!step.deleteReqLinks) {
					if (step.reqLinks && step.reqLinks.length > 0) {
						this._steps[i].oldReqLinks = dojo.clone(step.reqLinks);
						this._steps[i].reqLinks = dojo.clone(step.reqLinks);
					}
					else {
						this._steps[i].oldReqLinks = step.oldReqLinks ? dojo.clone(step.oldReqLinks) : [];
						this._steps[i].reqLinks = [];
					}
				} else{
					this._steps[i].oldReqLinks = [];
					this._steps[i].reqLinks = [];
				}
			}
			this._stepsEditor.updateScript(savedItem);
            this._stepsEditor.updateSteps(this._steps);
            this._stepsEditor.refreshExecutionVariablePanel(savedItem);
            this._dealWithDatapool(savedItem.elements);
        },
        
        _dealWithDatapool: function(elements){
            if (elements && elements.length) {
                var stepPair = new Array();
				var len = elements.length;
                for (var i = 0; i < len; i++) {
                    var varArray = this._parseVariableId(elements[i].content);
                    var varArray2 = this._parseVariableId(elements[i].expectedResult);
                    if (varArray2 && varArray2.length) {
                        if (!varArray) 
                            varArray = varArray2;
                        else 
                            varArray = varArray.concat(varArray2);
                    }
                    if (varArray && varArray.length > 0) {
                        stepPair.push({
                            stepItemId: elements[i].itemId,
                            variables: varArray
                        });
                    }
                }
                if (stepPair.length > 0) {
                    var param = dojo.toJson(stepPair);
                    var callbackObj = {
                        success: function(results){
                        },
                        failure: function(e){
                            console.error("Datapool createLink failed");
                        }
                    };
                    var responseHandler = new ServiceResponseHandler(callbackObj, "success", "failure");
                    DatapoolClient.postDatapoolVariableLink2(responseHandler, {
                        jsonString: param
                    });
                }
            }
        },
        
        _findIn: function(elements, element){
            var len = (elements || []).length;
            for (var i = 0; i < len; i++) {
                if (elements[i].itemId != null && elements[i].itemId == element.itemId) {
                    return elements[i];
                }
            }
            return null;
        },
        
        _parseVariableId: function(content){
            var newContent = unescape(content);
            var contentDiv = document.createElement('div');
            contentDiv.innerHTML = newContent;
            var children = contentDiv.getElementsByTagName("BUTTON");//desDiv.childNodes;
            var newArray = [];
			var len = children ? children.length : 0;
            for (var i = 0; i < len; i++) {
                newArray.push(children[i]);
            }
            
            var idArray = null;
            if (newArray && newArray.length) {
                idArray = new Array();
				var len = newArray.length;
                for (var i = 0; i < len; i++) {
                    if (newArray[i].name == "variable") {
                        var itemId = newArray[i].id;
                        idArray.push(itemId);
                    }
                }
            }
            return idArray;
        },
        
        onArtifactChanged: function(changeType, artifactType, artifactArray){
            if (this._stepsEditor && this._stepsEditor._kwView) 
                this._stepsEditor._kwView.onArtifactChanged(changeType, artifactType, artifactArray);
        },
        
        _onDatapoolChange: function(){
            this._stepsEditor._removeAllVariables();
        },
        
		//Record functions
		isRecordable: function(){
			return true;
		},
		
		onRecord: function(){			
			this.parentEditor.setReadOnly(true);
			this.lastReceivedStepsLen = 0;
			this._originalSteps = dojo.clone(this._stepsEditor.getElements());			
			this.inherited(arguments);
			this.initialize();
		},
		
		isProgressIndicatorNeededByExtender: function(){
			return true;
		},
		
		setProgressDomNode: function(domNodeForProgress){
			//Disable the attributes of the extender, disable steps editor and add the progress indicator in the middle
			this.stepsEditorAppendNode = this._stepsEditor.getNodeToAppendProgress();
			if(this.stepsEditorAppendNode){
				this.progressIndicatorNode = this.stepsEditorAppendNode.parentNode;
				this.stepsEditorAppendNode.style.display = "none"; 
				this.progressIndicatorNode.appendChild(domNodeForProgress);
			}else{
				if(this.progressIndicatorNode){
					dojox.xml.parser.removeChildren(this.progressIndicatorNode);
				}
				this.progressIndicatorNode = null;
			}
		},
		
		onRecordComplete: function(/*Request object*/ requestObject){
			if(this.progressIndicatorNode){
				dojox.xml.parser.removeChildren(this.progressIndicatorNode);
				this.progressIndicatorNode.appendChild(this.stepsEditorAppendNode);
				this.stepsEditorAppendNode.style.display = "inline";
			}
			this.updatingLive = false;
			this._updateRecordingSteps(requestObject);		
			this.parentEditor.setReadOnly(false);
			this.initialize();            
			this.inherited(arguments);
			var self = this;
			setTimeout(function(){
				self.parentEditor.setDirty(true);
			}, 200);
		},
		
		_updateRecordingSteps: function(requestObject){
			var doc = document;
			var _dojo = dojo;
			var draftScriptElements = requestObject && requestObject.draftScript && requestObject.draftScript.elements ?
										requestObject.draftScript.elements : [];
			var recEleLen = draftScriptElements.length;
			var imgContent = null;
			var imgExpResult = null;
			var preferences = this.getRecorderPreferences();
			var imageDisplayType = preferences.showImageAs && preferences.showImageAs != "" ? preferences.showImageAs : "TN";
			var recordedSteps = [];
			//upescape
			for(var i=0;i<recEleLen;i++){
				imgContent = null;
				imgExpResult = null;
				var content = draftScriptElements[i].content ? unescape(draftScriptElements[i].content) : "";
				var expectedResult = draftScriptElements[i].expectedResult ? unescape(draftScriptElements[i].expectedResult) : "";
				if (imageDisplayType != "FS" && dojo.isFunction(this._stepsEditor._manageImages)) {
					imgContent = this._stepsEditor._manageImages(imageDisplayType, content, doc, _dojo);
					imgExpResult = this._stepsEditor._manageImages(imageDisplayType, expectedResult, doc, _dojo);
				}else{
					imgContent = content;
					imgExpResult = expectedResult;
				}
				draftScriptElements[i].content = imgContent ? imgContent : content;
				draftScriptElements[i].expectedResult = imgExpResult ? imgExpResult : expectedResult;
				delete draftScriptElements[i].itemId;//Remove the ItemId info from the draftscript as this is non-existant
				recordedSteps.push(draftScriptElements[i]);
			}
			
			if(recordedSteps.length > 0){
				if(this.inlineRecording){
					//insert the steps here
					var originalSteps = this._originalSteps;
					var insertPoint = this.recordingStepNum;
					
					var concatinatedSteps = new Array();
					
					var originalStepsLen = originalSteps.length;
					var recordedStepsLen = recordedSteps.length;
					var filledRecordedSteps = false;
					var concatLen = recordedStepsLen+originalStepsLen;
					for(var i=0;i<concatLen;i++){
						if(filledRecordedSteps || i < insertPoint ){
							var index = filledRecordedSteps ? i-recordedStepsLen : i; 
							concatinatedSteps[i] =  originalSteps[index];	
						}
						else{
							filledRecordedSteps = true;
							for(var j=0;j<recordedStepsLen;j++){
								concatinatedSteps[i] = recordedSteps[j];
								if(j != (recordedStepsLen-1))
									i++;
							}
						}
					}
					if(!this.updatingLive){
						this._updateStepEditorAfterRecording();
					}	
					this._steps = concatinatedSteps;
				}else{
					//if new recording
					this._steps = recordedSteps;
				}
			}else{
				//if nothing was recorded bring back to the original state
				this._steps = this._originalSteps;
			}
		},
		
		onStopRecording: function(){
			this.parentEditor.setReadOnly(false);
			this.initialize();
			this.lastReceivedStepsLen = 0;
			this.updatingLive = false;
			this.inherited(arguments);
			if(this.progressIndicatorNode){
				dojox.xml.parser.removeChildren(this.progressIndicatorNode);
				this.progressIndicatorNode.appendChild(this.stepsEditorAppendNode);
				this.stepsEditorAppendNode.style.display = "inline";
			}	
		},
		
		_onInlineRecording: function(currentRow, /*boolean to show if insert after is called*/after){
			this.inlineRecording = true;
			var adjustValue = after ? 1 : 0;
			this.recordingStepNum = currentRow.sectionRowIndex + adjustValue;
			this._steps = this._stepsEditor.getElements();
			this._stepsEditor.validateElements();
			//Need to call the onrecord for TestScript editor which internally calls the required methods
			//Pass true to inform that this was an inline recording scenario
			this.getParentEditor().onRecord(true);
		},
		
		setRecordingAdapter: function(adapter){
			this.inherited(arguments);
			if(adapter == "-1"){
				this.onStopRecording();
			}
			else {
				this.setRecordingInProgress(true);
			}
			if(this.inlineRecording && adapter == "-1"){
				//Perform cleanup
				this._updateStepEditorAfterRecording();
			}
		},
		
		/*
		 * This functions is used to show Live data
		 */
		setCurrentFetchedObject: function(/*request object*/requestObject){
			if(!this.updatingLive){
				var draftScriptElements = requestObject && requestObject.draftScript ? requestObject.draftScript.elements : [];
				var draftScriptElementsLen = draftScriptElements ? draftScriptElements.length : 0; 
				if(draftScriptElementsLen > 0 && draftScriptElementsLen > this.lastReceivedStepsLen){
					this.lastReceivedStepsLen = draftScriptElementsLen;
					this.updatingLive = true;
					this._updateRecordingSteps(requestObject);		
					this.initialize();//Dont change the readonly flag yet, record complete will handle it
					this.updatingLive = false;
				}
			}	
		},
		
		
		 /**
		  * The uri should looks like:
		  * https://<Public URI>/oslc_qm/contexts/<project area item id>/resources/com.ibm.rqm.planning.ExecutionElement2/<execution element item id>
		  * 
		  * @see Work item 78546 for more
		  * @param {Object} artifact
		  */
		_getStepResourceUri: function(itemId){
		    var projectAreaId = getCurrentProjectArea().itemId;
		    var resourceUri = dojo.getObject(Constants.PUBLIC_URL) + "oslc_qm/contexts/" + projectAreaId 
						+ '/resources/com.ibm.rqm.planning.ExecutionElement2/' + itemId;
		    return resourceUri;
		},
    		
		_updateStepEditorAfterRecording: function(){
			this.inlineRecording = false;
			this._stepsEditor.setNodeToAppendProgress(null);
			this.recordingStepNum = null;	
		},
		
		getCurrentElements: function(){
			//function to return the latest steps details
			return this._stepsEditor ? this._stepsEditor.getElements() : []; 
		},
		
		//Record functions over
		
		onBeforeClose: function(){
			var canClosePage = true;
			if (this._stepsEditor && this._stepsEditor.onBeforeClose) {
				canClosePage = this._stepsEditor.onBeforeClose();
			}
			
			return canClosePage;
		},
		
        destroy: function(){
            if (this._stepsEditor) 
                this._stepsEditor.destroy();
            this.inherited(arguments);
        },
        onDeactivate:function(){
            if(this._stepsEditor && this._stepsEditor.onDeactivate)
                this._stepsEditor.onDeactivate();
        },
		onActivated:function(){
			if(this._stepsEditor && this._stepsEditor.onActivated)
				this._stepsEditor.onActivated();
		},
		
		// @override - we're being re-started in merge mode. The node is 
		// about to be removed. Clear the ckeditor and remove the actions
		// in the view.
        preInitializeMerge: function() {
        	this.onDeactivate();
            if(this._stepsEditor) {
            	 this._stepsEditor.removeEditor();
            	 this._stepsEditor.enableActions(false);
            }
        },
        
        // @override - The size of the container area has changed, make
        // the step view re-calculate its size.
        onMergingActivated: function() {
            if(this._beingDestroyed){ return; }
            if(this._stepsEditor) {
           	 	this._stepsEditor._handleWindowSizeChanged();
            }        	
        }
    });
})();
