/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.execution.ManualScriptExecutionView");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.Button");
dojo.require("com.ibm.asq.common.web.ui.internal.util.DomConstructor");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.ManualScriptView");

(function(){
var Button    				= com.ibm.asq.common.web.ui.internal.widgets.Button;    
var Constants               = com.ibm.rqm.planning.web.client.internal.Constants;    
var LabelResume 			= Constants.getMessage("labelClickToResumeExe");
var _d						= dojo;
var SelectUtility 			= com.ibm.asq.common.web.ui.internal.util.SelectUtility;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.execution.ManualScriptExecutionView", 
             com.ibm.rqm.planning.web.ui.internal.view.construction.manualscript2.ManualScriptView, 
{
	editContentVSpacing: 30,
    iterationDisplayTemplateString: 
        '<div dojoAttachPoint="iterDisplayNode" class="label iteration-display">' +
        '  <div dojoAttachPoint="iterDetailNode">' +
        '    <span dojoAttachPoint="iterationDisplayNode"></span>' +
        '    <span dojoAttachPoint="actionsNode"></span>' +
        '  </div>' +
       '</div>',
	
	/**
	 * @override
	 */
	postCreate: function() {
		this.doInitFocusOnFirstStep = false;
		this.inherited(arguments);
		_d.addClass(this.domNode, "manual-script-execution-view");
	},
	
	_createInactiveDisplay: function() {
		// create an inactive cover node. If the execution is paused,
		// we will lay it over the area to prevent clicks in the view
		// as well as show an inactive display.
		//
		this.coverDiv = _d.create("div", {
			'className': "inactive-cover hidden"
		});
		var resumeDiv = this.resumeDiv = _d.create("div", {
			'title': LabelResume,
			'tabIndex': "0",
			'className': "hidden resume-node clickable"
		});
		_d.place(resumeDiv, this.domNode);
		_d.place(this.coverDiv, this.domNode);
		
		this.connect(resumeDiv, "ondijitclick", function() {
			this.controller.onPauseResumeClicked();
		});		
	},
	
	/**
	 * Set to show the iteration display area on the top. 
	 */
	_setShowIterationAttr: function() {
		if (this.printView) {
			// Do not show iteration display area in print view.
			return;
		}
		if(!this.iterationDisplayNode) {
	        // attach the widgets created from the dom constructor to our object
	        var dom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor(this.iterationDisplayTemplateString);
	        this.iterationDisplayNode = dom.iterationDisplayNode;
	        this.iterDetailNode = dom.iterDetailNode;
	        
	        // create the changing iteration display buttons. 
	    	var iterCtrlNode = _d.create("div");
	        this.prevIter = new Button({
	            showLabel: false,
	            label: Constants.getMessage("labelPrevIteration"),
	            onClick: _d.hitch(this, "_onIterationButtonClick", -1)
	        });
	        this.prevIter.set('iconClass', 'com-ibm-asq-common-web-ui-option-chooser-move-left-button');
	        this.prevIter.set('disabled', true);
	        _d.place(this.prevIter.domNode, iterCtrlNode);	
	        
	        this.nextIter = new Button({
	            label: Constants.getMessage("labelNextIteration"),
	            showLabel: false,
	            onClick: _d.hitch(this, "_onIterationButtonClick", 1)
	        });
	        this.nextIter.set('iconClass', 'com-ibm-asq-common-web-ui-option-chooser-move-right-button');
	        _d.place(this.nextIter.domNode, iterCtrlNode);	
	        _d.place(iterCtrlNode, dom.actionsNode);
	        
			_d.place(dom.iterDisplayNode, this.stepAreaNode, "before");
		}
	},
	
	_onIterationButtonClick: function(value) {
		this.controller.moveToIteration(value);
	},
	
	_updateIterationButtonsState: function() {
		var currIter = this.model.getCurrentDisplayIteration();
		this.prevIter.set('disabled', currIter < 1);
		this.nextIter.set('disabled', currIter >= this.model.getNumExecutionIterations() - 1);		
	},
	
	/**
	 * Creates the iteration configuration UI. This allows the user to change the number of
	 * iterations to execution and a 'start' action to start the execution.
	 */
	_setConfigureIterationCtrlAttr: function() {
		if(this.iterationDisplayNode) {
    		var len = this.model.getNumExecutionIterations();
    		
    		this._iterationSelect = _d.create("select");
    		
            for (var i = 1; i <= len ; i++) {
                var opt = new Option(i, i);
                SelectUtility.add(this._iterationSelect, opt);
            }
            SelectUtility.select(this._iterationSelect, len);
            this.connect(this._iterationSelect, "onchange", function() {
            	this.controller.setMaxIteration(this._iterationSelect.value);
        		this._updateIterationButtonsState();
            });
            
            this.configureNode = _d.create("div");
    		var span = _d.create("span", {
    			innerHTML: Constants.getMessage("labelSelectNumIterationToRun")
    		});
    		_d.place(span, this.configureNode);
    		_d.place(this._iterationSelect, this.configureNode);
			
			_d.place(this.configureNode, this.iterationDisplayNode, "before");
			
	        this.iterationInfoNode = _d.create("div", {
	        	className: "info",
	        	innerHTML: Constants.getMessage("labelSelectIterationInfo")
	        });
	        _d.place(this.iterationInfoNode, this.iterDetailNode);
	        
			var startLink = _d.query("a", this.iterationInfoNode)[0];
			
			startLink.setAttribute("tabIndex", "0");
			this.startConnect = this.connect(startLink, "ondijitclick", function() {
				this.disconnect(this.startConnect);
				
				this.iterDetailNode.removeChild(this.configureNode);
				this.iterDetailNode.removeChild(this.iterationInfoNode);
				_d.empty(this.iterationInfoNode);
				_d.empty(this.configureNode);
				
				this.controller.beginIterationExecution();
			});
		}
	},
	
	_setIterationDisplayAttr: function(iteration) {
		if(this.iterationDisplayNode) {
			var dispIter = this.model.getCurrentDisplayIteration();
			var execIter = this.model.getCurrentExecutionIteration();
			var msg = (this.controller._inIterationConfigureMode || dispIter != execIter) ? "labelShowingIteration" : "labelRunningIteration";
			this.iterationDisplayNode.innerHTML = Constants.getMessage(msg, iteration, this.model.getNumExecutionIterations());
			
			this._updateIterationButtonsState();
		}
	},
	
	/**
	 * Handles the advancing to the next iteration of steps. This indicates that
	 * the model's current executing steps had changed to the next iteration.
	 * We will do a wholesale update of the step UI to load in the new set of step models.
	 */
	onIterationAdvanced: function() {
		this.scriptContainer.reloadAllSteps();
		this._setScrollTopPos(this.iterDetailNode, {x: 0, y: 0});
    	this.set("iterationDisplay", this.model.getCurrentDisplayIteration() + 1);    		
	},
	
    /**
     * When the view display is ready, i.e., the initial steps are loaded, etc., this method is
     * called. We're going to instruct the controller to initiate the execution.
     * 
     * @override
     */
    onViewReady: function() {
    	// provide the controller to get a step from the view.
        this.controller.getViewStepAt = _d.hitch(this, function(stepPos, onReady) {
        	// call the container to show to the requested step, but don't scroll to
        	// it. If the step is a keyword, we want to scroll to the sub-step and not
        	// to the keyword step. We'll provide the controller to scroll to the exact
        	// step position.
        	this.scriptContainer.showToStep(stepPos, /*scroll to*/false, _d.hitch(this, function() {
        		onReady(this.scriptContainer.getChildren()[stepPos]);
        	}));
        });
        // hook up the controller to the container's scroll to step method. Let the
        // controller direct the scrolling
        this.controller.scrollToStep = _d.hitch(this.scriptContainer, "_scrollToStep");
        this.controller.startExecution();
    },
    
    /**
     * Change the execution view's display to paused or active mode.
     * @param paused true, if to set to paused (inactive mode)
     */
    _setPausedAttr: function(paused) {
    	if(paused && !this.coverDiv) {
    		this._createInactiveDisplay();
    	}
    	this._paused = paused;
    	_d.toggleClass(this.domNode, "inactive-mode", paused);
    	_d.toggleClass(this.coverDiv, "hidden", !paused);
    	_d.toggleClass(this.resumeDiv, "hidden", !paused);
    	this.scriptContainer.set("disableEdit", paused);
    	this._updateInactiveModeDim();
    },
    
    _updateInactiveModeDim: function() {
    	var dim = _d.position(this.domNode);
    	_d.marginBox(this.coverDiv, {w: dim.w, h: dim.h - 20});
    	_d.marginBox(this.resumeDiv, {t: (dim.h-75)/2, l: (dim.w-75)/2});    	
    },
    
    /**
     * @override
     * To receive window size changed event to update the inactive cover node.
     */
    _handleSizeChanged: function(onCompleted) {
    	this.inherited(arguments);
    	if(this._paused) {
    		setTimeout(_d.hitch(this, "_updateInactiveModeDim"), 100);
    	}
    },
    
    /**
     * @override
     *   We completely taken over the left status area to show the status bar and script execution controls.
     *   Don't have to respond to the scrolling context display.
     */
    _updateStepDisplayContext: function(start, end) {
    },
    
    /**
     * @override
     * Implements the process image interface. This is just a pass-thru of the
     * image without any processing.
     */
    processImage: function(imageString) {
    	return imageString;
	},

	/**
     * Public API
     */
    onError: function(message) {
    	// override to receive the reporting of error conditions    	
    },
    
    onExecutionCompleted: function(stateId) {
    	// override to receive execution completion event.
    },

    onExecutionCancelled: function() {
    	// override to receive execution completion event.
    },
    
    onOpenSuitePage: function() {
    	// override to receive open to a suite page.
    },
    
    onCancel: function() {
		if(this.controller.currentEditedStepModel){
			var args = {
				primaryTitle: Constants.getMessage("labelConfirmation"),
				message: Constants.getMessage("labelUnSavedProperties"),
				yesCallback: _d.hitch(this, "performCancel"),
				noCallback: null
			};
			var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
            dialog.open();
            return;
		}
		this.performCancel();
    },
    
    /**
     * Cancel the current execution.
     */
    performCancel: function() {
    	this.model.cancelExecution(_d.hitch(this, function(errorObj) {
    		if(errorObj && errorObj.message) {
    			this.onError(errorObj.messagej);
    			return;
    		}
    		this.onExecutionCancelled();
    	}));
    },
    
    // Notify on before the parent page is closed. Final processing to save any pending
    // edits on a step before close and destroy the editor.
    //
    onBeforeClose: function(onComplete) {
    	// check and update any current editing in the steps.
    	this.controller._updateCurrentEditedStep(dojo.hitch(this, function() {
    		this.scriptContainer.set("disableEdit", true);
    		onComplete();
    	}));    	
    },
    
    onEndOfExecution: function() {
    	// host view will handle any unsaved editing by calling onBeforeClose. 
    	// It's an no-op here.
    },
    
    isDirty: function(){
    	var hasEdit = this.controller.currentEditedStepModel != null;
        return hasEdit;
    },
    
    checkForActualResultConstraint: function(){
        return true;
    },
    
    isExecutionComplete: function(){
        return this.model.isExecutionCompleted();
    },
    
    hideToolBar: function() {
    	// old interface, we can ignore this.
    },
    
    onStop: function() {
    },
    
    dummy: null
});

})();


